// RXMQR.CPP : A DLL to give REXX Support for IBM/MQ/OS2
//
// Written by, and only begetter, Robert Harris,
//                                Object Technology Products,
//                                IBM Hursley.
//
// This module builds into a DLL with provides Rexx Support for
//      MQ/OS2. Note that the name of the resultant DLL is
//      set to the THISDLL macro. (I also use the trigraph
//      representation for square brackets, as this gets round
//      character set problems).
//
// This is a nearly full implementation of the MQ API with these
//      omissions :
//
//
//      PUT1 is not supported, because I dislike this call
//                             immensely and think that it should
//                             be banned from the face of the earth
//
//      INQ and SET only take a single attribute.
//
//
// The MQ API is extended in several ways:
//
//
//      * The QM Handle is not externally referenced, as this is held
//            within this DLL
//
//      * The DLL knows about threads. It will allow multiple Rexx threads
//                to talk to multiple (or the same) Queue Managers
//
//      * Structures that get updated are managed via seperate
//                   input and output areas, so avoiding the need
//                   to do all that tedious rebuilding for each
//                   call
//
//      * Output areas contain .ZLIST which contains a list of all the
//               components within the output stem (without the dot)
//
//      * A Browse function is provided which just returns the
//                 message data
//
//      * A Header Extraction function will interprete Dead Letter
//                 and Transmission Headers from suitable messages
//
//      * Expansion of Events into components (like the Header Extraction)
//
//      * Interpretation of Trigger Messages and Trigger Data
//
//      * Setting up ONLY of MQ Constants
//
//
//   In order to use this Rexx/MQ Interface, this DLL's initialisation
//      function must be called before usage.
//
//   This is done by a RxFuncAdd('RXMQRINIT','RXMQR','RXMQRINIT')
//        call to let Rexx know about the DLL, and then a
//        rcc = RXMQRINIT() to call the initialisation function.
//        As part of this initialisation function, all the other
//        RXMQR functions are registered with Rexx.
//
//   There is no equivalent termination function, as the MQ routines
//         attach to process termination to stop access at this
//         time, and a Termination operation would interfere
//         with this processing. However, a function called
//         RXMQRTERM will deregister the interface functions WITHOUT
//         doing end-of-process actions.
//
//   All the RXMQR functions return a standard Rexx Return string.
//       This is structured so that the numeric Return Code
//       (which may be negative) is obtained by a word(RCC,1) call.
//
//   The Return Code for an operation can be negative to
//         show that this DLL has detected the error, otherwise
//         it will be the MQ Completion Code
//
//         The Return String is in text format as follows:
//
//            Word 1 : Return Code
//            Word 2 : MQ Completion Code (or 0 if MQ not done)
//            Word 2 : MQ Reason     Code (or 0 if MQ not done)
//            Word 4 : RXMQR function being run
//            Word > : OK or an helpful error message
//
//   In addition, the current (ie: the settings last set) values are
//      available in these variables:
//
//          RXMQR.LASTRC    -> current operation Return Code
//          RXMQR.LASTCC    -> current operation MQ Completion Code
//          RXMQR.LASTAC    -> current operation MQ Reason     Code
//          RXMQR.LASTOP    -> current operation RXMQR function
//          RXMQR.LASTMSG   -> current operation Return String
//
//   As a nice little bonus, there are also defined lots of variables
//        called RXMQR.RCMAP.nn , where nn is a MQAC number, whose
//        value is the name of the Reason Code (MQRC_ERROR_THING).
//        These variables can then be used in the Rexx Exec to get
//        the name from the Reason code via a
//                 interpret 'name = RXMQR.RCMAP.'word(rcc,1)
//        statement.
//
//        A TERM function leaves the MQ Rexx Variables known; if removed via
//          a DROP call, they may be restored via RXMQRCONS (registering if
//          apt via RxFuncAdd('RXMQRCONS',RXQML','RXMQRCONS') if needed).
//
//
//   Tracing MAY be enabled (if this module has been compiled with
//           the relevant setting of the DEBUG macro) by setting
//           values into a Rexx Variable call RXMQRTRACE. The
//           settings are:
//
//                              CONN  -> mqconn
//                              DISC  -> mqdisc
//                              OPEN  -> mqopen
//                              CLOSE -> mqopen
//                              GET   -> mqget
//                              PUT   -> mqput
//                              INQ   -> mqinq
//                              SET   -> mqset
//                              CMIT  -> mqcmit
//                              BACK  -> mqback
//
//                              BRO   -> Browse extension
//
//                              HXT   -> Header extraction extension
//                              EVENT -> Event determination extension
//                              TM    -> Trigger message extension
//
//                              MMD   -> Rexx stem var --> MQMD
//                              MOD   -> Rexx stem var --> MQOD
//                              MPO   -> Rexx stem var --> MQPMO
//                              MGO   -> Rexx stem var --> MQGMO
//
//                              BMD   -> MQMD  --> Rexx stem var
//                              BOD   -> MQOD  --> Rexx stem var
//                              BPO   -> MQPMO --> Rexx stem var
//                              BGO   -> MQGMO --> Rexx stem var
//
//                              GV    -> Obtaining a Rexx Variable
//                              PV    -> Setting   a Rexx Variable
//                              SK    -> Return Code processing
//                              TR    -> Thread based processing
//
//                              INIT  -> Initialisation processing
//                              TERM  -> Deregistration processing
//
//                              *     -> Trace everything!!!
//
//
//  Thread Support: In order to provide Thread support for non-IBM Rexxs,
//                  the RAHMQOTR Class library records the current thread upon
//                  which the MQCONN & MQOPEN is issued. This is then
//                  available for policing by the other functions. This
//                  results in calls to see that the Connection/Open thread
//                  is the current one for which an access is made.
//
//                   RC -98 indicates that the thread is already connected
//                      -97                     owning thread is not the
//                                              one doing the access
//
//                  This also means that the scanning and allocation of
//                  objects from the global process-based objects must be
//                  protected against multiple accesses. Therefore, an
//                  anomous private Mutex is created at INIT time (which is
//                  thus available in all threads for the process) to
//                  engender this single threading. The Mutex is removed
//                  in the TERM call. This locking is active OVER MQ calls
//                  in the CONN, OPEN, CLOSE and DISC functions.
//
//                   RC -96 indicates that the Mutex was not acquired
//                      -95                                  released
//
//                  The TIMEOUT for the acquisition is set to MUTEXWAIT. This
//                      should be set to a big enough value to cope with long
//                      remote-type accesses.
//
//
//
//  Copyright : This program is Copyrighted
//              by IBM UK Labs Limited, 1995.
//
//
//   Support  : This program is not supported in any way by IBM.
//              It is distributed only to show techniques for
//              Message Queueing (if so distributed).
//              It may be freely modified and adapted by your
//              installation.
//
//   Author   : Robert Harris,
//              Object Technology Products,
//              IBM Hursley Park,
//              England.
//
//
//
//

//
// Standard Header file includes
//

 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <iostream.h>
 #include <time.h>

//
// OS/2 Header file includes
//

 #define INCL_DOSMEMMGR
 #define INCL_DOSPROCESS
 #define INCL_DOSSEMAPHORES
 #include <os2.h>

//
// Rexx Header file includes
//

 #define INCL_RXSHV
 #define INCL_RXFUNC
 #include <REXXSAA.H>

//
// MQ/OS2 Header file includes
//

 #include <CMQC.H>
 #include <CMQCFC.H>

// Define a few constants that are not (yet) in the includes

 #define MQ_ENV_DATA_LENGTH        MQ_PROCESS_ENV_DATA_LENGTH
 #define MQ_APPL_ID_LENGTH         MQ_PROCESS_APPL_ID_LENGTH
 #define MQ_USER_DATA_LENGTH       MQ_PROCESS_USER_DATA_LENGTH
 #define MQ_TO_Q_NAME_LENGTH       MQ_Q_NAME_LENGTH
 #define MQ_EXIT_USER_DATA_LENGTH  MQ_PROCESS_USER_DATA_LENGTH

//
//  The DEBUG macro is used to enable tracing. This may be perminantly
//            enabled/disabled, or dynamically under the control
//            of the RXMQRTRACE Rexx variable.
//
//
//  DEBUG settings :
//
//  DEBUG  -> istrace   for dynamic debugging based on RXMQRTRACE
//  DEBUG  -> cout <<   for static trace everything debugging
//  DEBUG  -> ;         for stop all tracing
//
//  DEBUGW -> istrace   for dyna1ic debugging based on RXMQRTRACE
//  DEBUGW ->           for stat2c trace everything debugging
//  DEBUGW -> ;         for stop3all tracing
//
//  DEBUGS -> cout <<   for    tracing in istrace (lots of lines!!!)
//  DEBUGS -> ;         for no tracing in istrace routine
//
//
//  The setting are thus:
//
// #define DEBUG(p1,p2) if (istrace( p1 ) == TRUE) cout << p2 << endl
// #define DEBUG(p1,p2) cout << p2 << endl
// #define DEBUG(p1,p2)
//
// #define DEBUGW(p1,p2,p3,p4) if (istrace( p1 ) == TRUE) { cout.p2 ; cout.p3 ; cout.p4 ; }
// #define DEBUGW(p1,p2,p3,p4) { cout.p2 ; cout.p3 ; cout.p4 ; }
// #define DEBUGW(p1,p2,p3,p4)
//
// #define DEBUGS(p1) cout << p1 << endl
// #define DEBUGS(p1)
//

#define DEBUG(p1,p2) if (istrace( p1 ) == TRUE) cout << p2 << endl
#define DEBUGW(p1,p2,p3,p4) if (istrace( p1 ) == TRUE) { cout.p2 ; cout.p3 ; cout.p4 ; }
#define DEBUGS(p1)

//
// Global definitions
//
//  MINQS     -> Start scan of Queues allowed to process (not 0!)
//  MAXQS     -> maximum number of Queues allowed to process
//  THISDLL   -> name of this DLL
//  MUTEXWAIT -> wait time for critical section mutex (in s)
//
//

#define MINQS 1
#define MAXQS 100
#define THISDLL "RXMQR"
#define MUTEXWAIT 30000

//
// Definitions of datatypes
//

#define FULLWORD     long
#define HALFWORD     int
#define MAXSFULLWORD (68719476735)
#define MAXUFULLWORD (4294967295)
#define MAXSHALFWORD (4095)
#define MAXUHALFWORD (65535)
#define MAXFILENAME  (255)

//
// All the actual MQ calls are preformed within the RAHMQOTR
//     class library. This is bought in at compile time
//     for ease of customer compilation. There is no problem
//     about using the Linkage editor to bind at link time
//     if this is so desired.
//
//     !!! Note that this version of RAHMQO has info about Threads
//     !!! within it. It is the responsibility of the caller to ensure
//     !!! that the current thread is the one that was used when the MQCONN
//     !!! was issued (if this is not done, MQ will return Invalid Handle
//     !!! for the QM messages etc.)
//

//
// RAHMQOTR Class Library - Thread aware
//

class rahmqotr
 {
   public :

    char        qm  ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name
    char        q   ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name
    MQHCONN     qmh                              ; //QM handle
    MQHOBJ      qh                               ; //Q  handle
    MQLONG      rc                               ; //Return Code
    MQLONG      ac                               ; //Reason Code
    MQLONG      lastrc                           ; //Prev Return Code
    MQLONG      lastac                           ; //Prev Reason Code
    ULONG       threadid                         ; //Thread for MQCONN


    rahmqotr ( void )               ;  //Contructor for new  use of QM
    rahmqotr ( MQHCONN const uqmh)  ;  //Contructor for same use of QM

    MQLONG      conn ( void )               ;  //Connect    to QM
    MQLONG      conn ( char * const uqm )   ;  //  with QM name supplied

    MQLONG      disc ( void )               ;  //Disconnect from QM

    void        cleanup ( void )            ;  //Close and Disconnect

    MQOD        odopen                      ;  //OD for MQOPEN usage
    MQLONG      open (MQLONG const uopts )  ;  //MQOPEN call
    MQLONG      open (char * const uq    ,     //  with Q name supplied
                      MQLONG const uopts )  ;  //
    MQLONG      open (MQOD * const uod   ,     //  with OD and options
                      MQLONG const uopts )  ;  //

    MQLONG      close (MQLONG const uopts ) ;  //MQCLOSE call

    MQLONG      commit   ( void ) ; //Syncpoint UOW
    MQLONG      rollback ( void ) ; //Backout   UOW

    MQLONG      inq ( MQLONG   const  uselnum ,             //MQINQ call
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  ) ;

    MQLONG      set ( MQLONG   const uselnum  ,             //MQSET call
                      MQLONG         usels    ,
                      MQLONG   const uinum    ,
                      MQLONG *       uiattr   ,
                      MQLONG   const ucharl   ,
                      char   *       ucattr   ) ;

    MQMD        pmd                             ;           //PUT Message Dwescriptor
    MQLONG      put ( MQPMO  * const  uopts   ,             //MQPUT call
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

    MQLONG      put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

                                                            //PUT Zero-Length record
    MQLONG      putz ( MQPMO  * const  uopts     ) ;        //No Tag, Use Instance MD

    MQLONG      putz ( MQMD   *        upmd      ,          //No Tag, use supplied MD
                       MQPMO  * const  uopts     ) ;

    MQLONG      putz ( MQPMO    * const  uopts   ,          //Tag, Use Instance MD
                       MQBYTE24 * const  utag    ) ;

    MQLONG      putz ( MQMD     *        upmd    ,          //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    ) ;

    MQMD        bmd                                  ;      //Browse Message Descriptor
    MQGMO       bmo                                  ;      //Browse MQGET options
    MQLONG      browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;

    MQLONG      browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;


    MQMD        gmd                               ;         //GET Message Descriptor
    MQLONG      get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;

    MQLONG      get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;


    double      gettimeconn     () { return (time_conn     ) ; }
    double      gettimedisc     () { return (time_disc     ) ; }
    double      gettimeopen     () { return (time_open     ) ; }
    double      gettimeclose    () { return (time_close    ) ; }
    double      gettimeinq      () { return (time_inq      ) ; }
    double      gettimeset      () { return (time_set      ) ; }
    double      gettimeput      () { return (time_put      ) ; }
    double      gettimeget      () { return (time_get      ) ; }
    double      gettimecommit   () { return (time_commit   ) ; }
    double      gettimerollback () { return (time_rollback ) ; }
    double      getcputconn     () { return (cput_conn     / CLOCKS_PER_SEC) ; }
    double      getcputdisc     () { return (cput_disc     / CLOCKS_PER_SEC) ; }
    double      getcputopen     () { return (cput_open     / CLOCKS_PER_SEC) ; }
    double      getcputclose    () { return (cput_close    / CLOCKS_PER_SEC) ; }
    double      getcputinq      () { return (cput_inq      / CLOCKS_PER_SEC) ; }
    double      getcputset      () { return (cput_set      / CLOCKS_PER_SEC) ; }
    double      getcputput      () { return (cput_put      / CLOCKS_PER_SEC) ; }
    double      getcputget      () { return (cput_get      / CLOCKS_PER_SEC) ; }
    double      getcputcommit   () { return (cput_commit   / CLOCKS_PER_SEC) ; }
    double      getcputrollback () { return (cput_rollback / CLOCKS_PER_SEC) ; }

 private :

    double      time_conn      ;
    double      time_disc      ;
    double      time_open      ;
    double      time_close     ;
    double      time_inq       ;
    double      time_set       ;
    double      time_put       ;
    double      time_get       ;
    double      time_commit    ;
    double      time_rollback  ;


    double      cput_conn      ;
    double      cput_disc      ;
    double      cput_open      ;
    double      cput_close     ;
    double      cput_inq       ;
    double      cput_set       ;
    double      cput_put       ;
    double      cput_get       ;
    double      cput_commit    ;
    double      cput_rollback  ;
 } ;

//
// rahmqotr member function definitions
//

//
// Constructor functions
//
//     without a parm, then this is a new use of a QM
//     with          , then this is an additional access on a
//                     previously connected QM
//

rahmqotr::rahmqotr ( void )
{
  memset(qm,0,sizeof(qm))   ;
  memset(q ,0,sizeof(q ))   ;
  qmh      = 0              ;
  qh       = 0              ;
  rc       = 0              ;
  ac       = 0              ;
  lastrc   = 0              ;
  lastac   = 0              ;
  threadid = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

rahmqotr::rahmqotr ( MQHCONN const uqmh )
{
  memset(qm,0,sizeof(qm))   ;
  memset(q ,0,sizeof(q ))   ;
  qmh      = uqmh           ;
  qh       = 0              ;
  rc       = 0              ;
  ac       = 0              ;
  lastrc   = 0              ;
  lastac   = 0              ;
  threadid = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

//
// rahmqotr destructor functions
//
//   these are not provided
//
//    (Note that a disc call removes access to ALL
//     instances of RAHMQOTR sharing the Handle. as I
//     am not aiming at a 'proper' C++ implementation
//     I have not provided code to cope with this event).
//

//
// rahmqotr functions to access a Queue Manager
//

//
// conn  will connect to a Queue Manager
//       saving the QMH
//
//       The version with no parms will use the
//           pre-set Queue Manager name
//
//       The alternative access is to quote a
//           Queue Manager Name which is the
//           one to be used
//

MQLONG  rahmqotr::conn ( char * const uqm )
{
  double c1, c2, t1, t2         ;
  PTIB   ptinfo                 ;
  PPIB   ppinfo                 ;

  strncpy(qm,uqm,sizeof(qm)-1)  ;

  lastrc = rc ;
  lastac = ac ;

  DosGetInfoBlocks(&ptinfo,&ppinfo) ;
  threadid = ptinfo->tib_ordinal    ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqotr::conn ( void )
{
  double c1, c2, t1, t2         ;
  PTIB   ptinfo                 ;
  PPIB   ppinfo                 ;

  lastrc = rc ;
  lastac = ac ;

  DosGetInfoBlocks(&ptinfo,&ppinfo) ;
  threadid = ptinfo->tib_ordinal    ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// disc  will disconnect from a Queue Manager
//       deleting the QMH
//


MQLONG  rahmqotr::disc ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQDISC ( &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_disc += 1.0000 * ( c2 - c1 )      ;
  time_disc += 1.0000 * difftime(t2,t1)  ;

  qmh      = 0 ;
  threadid = 0 ;

  return (ac)  ;
}

//
// cleanup will Close the Q (if known) and Disconnect from
//         the QM (if known). This will effect other instances
//         which were created with a QM handle
//

void    rahmqotr::cleanup ( void )
{
  double c1, c2, t1, t2         ;

  if ( qh  != 0 ) close(MQCO_NONE) ;
  if ( qmh != 0 ) disc ()          ;
  return                           ;
}

//
// commit will Syncpoint ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqotr::commit ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCMIT ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_commit += 1.0000 * ( c2 - c1 )      ;
  time_commit += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// rollback will backout ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqotr::rollback ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQBACK ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_rollback += 1.0000 * ( c2 - c1 )      ;
  time_rollback += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// open  will provide access to a Queue
//
//       The OPEN options are always supplied
//           by the caller.
//
//       If the version without a Queue Name is
//          used, then the Name from the Instance
//          is taken.
//

MQLONG  rahmqotr::open ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqotr::open ( char * const uq    ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  strncpy(q,uq,sizeof(q)-1)   ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqotr::open ( MQOD * const uod   ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memcpy(&odopen,uod,sizeof(odopen)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  memcpy(uod,&odopen,sizeof(odopen))     ;
  return (ac) ;
}

//
// close will stop access to a Queue
//
//       The CLOSE options are always supplied
//           by the caller.
//

MQLONG  rahmqotr::close ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCLOSE ( qmh, &qh, uopts, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_close += 1.0000 * ( c2 - c1 )      ;
  time_close += 1.0000 * difftime(t2,t1)  ;

  qh = 0      ;

  return (ac) ;
}

//
// inq   will inquire upon the Attributes of
//       a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqotr::inq ( MQLONG   const  uselnum ,
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQINQ ( qmh, qh, uselnum, &usels ,
                   uinum  , uiattr ,
                   ucharl , ucattr ,
                                     &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_inq += 1.0000 * ( c2 - c1 )      ;
  time_inq += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}
//
// set   will set the Attributes of a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqotr::set ( MQLONG   const   uselnum ,
                      MQLONG           usels   ,
                      MQLONG   const   uinum   ,
                      MQLONG *         uiattr  ,
                      MQLONG   const   ucharl  ,
                      char   *         ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQSET ( qmh, qh, uselnum , &usels  ,
                   uinum   , uiattr  ,
                   ucharl  , ucattr  ,
                                       &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_set += 1.0000 * ( c2 - c1 )      ;
  time_set += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// put       will write a Message into a Queue
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqotr::put ( MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqotr::put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// putz      will write a Zero-Length Message into a Queue
//
//           If a Tag is supplied, then this is used as the Message Descriptor
//              this must be an area of MQBYTE24 (24 bytes long)
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqotr::putz ( MQPMO  * const  uopts )               //No Tag, Use Instance MD
{
  double c1, c2, t1, t2         ;

  memset(&this->pmd.MsgId,0,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqotr::putz ( MQMD   *        upmd    ,              //No Tag, use supplied MD
                       MQPMO  * const  uopts   )
{
  double c1, c2, t1, t2         ;

  memset(upmd->MsgId,0,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqotr::putz ( MQPMO    * const  uopts   ,            //Tag, Use Instance MD
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(&this->pmd.MsgId,utag,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqotr::putz ( MQMD     *        upmd    ,            //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(upmd->MsgId,utag,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
//    Browse    will non-destructively read a Queue
//
//              Versions are supplied that use an
//              instance Message dewscriptor, or
//              one supplied by the caller
//

MQLONG  rahmqotr::browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_FAIL_IF_QUIESCING               ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  memset(&bmd,0,sizeof(bmd))                                 ;
  memcpy(&bmd.StrucId,MQMD_STRUC_ID,sizeof(bmd.StrucId))     ;
  bmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &bmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqotr::browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_CONVERT+MQGMO_FAIL_IF_QUIESCING ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ubmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqotr::get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  bmo.Version        = MQMD_VERSION_1                        ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  gmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &gmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqotr::get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ugmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// End of RAHMQOTR Class definition and functions
//


//
// Global Variables
//

rahmqotr  gmqo??(MAXQS+2??)   ; //Queue Objects
HMTX      mutex               ; //Mutex Handle to lock gmqo access

//
// Polymorphic C++ functions
//

//
// Output Stream functions - convert MQ objects into cout
//
//        The MQOD, MQMD, MQPMO and MQGMO structures are
//            enabled for use with the << output stream
//            operators via the usual operator overloadings.
//
//        Each field is dumped in character and hex.
//
//        (At the time of writing, then functions are
//         only enabled when the revelant trace B or M
//         is made active)
//

//
// Print a MQOD  - Object descriptor
//

ostream& operator<<(ostream& os, MQOD * od)
 {

  LONG     i    ; //scanner

  os << "MQOD Structid        :"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << od->StrucId??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << hex << (unsigned short)od->StrucId??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "     Version         :"  ;
        os << od->Version ;
  os << " <"  ;
        os << hex << od->Version ;
  os << "> " ;
  os << endl ;

  os << "     ObjectType      :"  ;
        os << od->ObjectType ;
  os << " <"  ;
        os << hex << od->ObjectType ;
  os << "> " ;
  os << endl ;

  os << "     ObjectName      :"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << od->ObjectName??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << hex << (unsigned short)od->ObjectName??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "     ObjectQMgrName  :"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << od->ObjectQMgrName??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << hex << (unsigned short)od->ObjectQMgrName??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "     DynamicQName    :"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << od->DynamicQName??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << hex << (unsigned short)od->DynamicQName??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "     AlternateUserId :"  ;
        for(i=0; i < sizeof(MQCHAR12) ;
            os << od->AlternateUserId??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR12) ;
            os << hex << (unsigned short)od->AlternateUserId??(i++??)) ;
  os << "> " ;
  os << endl ;

  return (os) ;

 }

//
// Print a MQGMO - Get Message Options block
//

ostream& operator<<(ostream& os, MQGMO * gmo)
 {

  LONG     i    ; //scanner

  os << "MQGMO Structid        :"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << gmo->StrucId??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << hex << (unsigned short)gmo->StrucId??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      Version         :"  ;
        os << gmo->Version ;
  os << " <"  ;
        os << hex << gmo->Version ;
  os << "> " ;
  os << endl ;

  os << "      Options         :"  ;
        os << gmo->Options   ;
  os << " <"  ;
        os << hex << gmo->Options ;
  os << "> " ;
  os << endl ;

  os << "      WaitInterval    :"  ;
        os << gmo->WaitInterval ;
  os << " <"  ;
        os << hex << gmo->WaitInterval ;
  os << "> " ;
  os << endl ;

  os << "      Signal1         :"  ;
        os << gmo->Signal1 ;
  os << " <"  ;
        os << hex << gmo->Signal1 ;
  os << "> " ;
  os << endl ;

  os << "      Signal2         :"  ;
        os << gmo->Signal2 ;
  os << " <"  ;
        os << hex << gmo->Signal2 ;
  os << "> " ;
  os << endl ;

  os << "      ResolvedQName   :"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << gmo->ResolvedQName??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << hex << (unsigned short)gmo->ResolvedQName??(i++??)) ;
  os << "> " ;
  os << endl ;

  return (os) ;

 }

//
// Print a MQPMO - Put Message Options block
//

ostream& operator<<(ostream& os, MQPMO * pmo )
 {

  LONG     i    ; //scanner

  os << "MQPMO Structid        :"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << pmo->StrucId??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << hex << (unsigned short)pmo->StrucId??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      Version         :"  ;
        os << pmo->Version ;
  os << " <"  ;
        os << hex << pmo->Version ;
  os << "> " ;
  os << endl ;

  os << "      Options         :"  ;
        os << pmo->Options ;
  os << " <"  ;
        os << hex << pmo->Options ;
  os << "> " ;
  os << endl ;

  os << "      Timeout         :"  ;
        os << pmo->Timeout ;
  os << " <"  ;
        os << hex << pmo->Timeout ;
  os << "> " ;
  os << endl ;

  os << "      Context         :"  ;
        os << pmo->Context ;
  os << " <"  ;
        os << hex << pmo->Context ;
  os << "> " ;
  os << endl ;

  os << "      KnownDestCount  :"  ;
        os << pmo->KnownDestCount ;
  os << " <"  ;
        os << hex << pmo->KnownDestCount ;
  os << "> " ;
  os << endl ;

  os << "      UnknownDestCount:"  ;
        os << pmo->UnknownDestCount ;
  os << " <"  ;
        os << hex << pmo->UnknownDestCount ;
  os << "> " ;
  os << endl ;

  os << "      InvalidDestCount:"  ;
        os << pmo->InvalidDestCount ;
  os << " <"  ;
        os << hex << pmo->InvalidDestCount ;
  os << "> " ;
  os << endl ;

  os << "      ResolvedQName   :"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << pmo->ResolvedQName??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << hex << (unsigned short)pmo->ResolvedQName??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      ResolvedQMgrName:"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << pmo->ResolvedQMgrName??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << hex << (unsigned short)pmo->ResolvedQMgrName??(i++??));
  os << "> " ;
  os << endl ;


  return (os) ;

 }

//
// Print a MQMD  - MQ Message Descriptor
//

ostream& operator<<(ostream& os, MQMD * md )
 {

  LONG     i    ; //scanner

  os << "MQMD  Structid        :"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << md->StrucId??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << hex << (unsigned short)md->StrucId??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      Version         :"  ;
        os << md->Version ;
  os << " <"  ;
        os << hex << md->Version ;
  os << "> " ;
  os << endl ;

  os << "      Report          :"  ;
        os << md->Report ;
  os << " <"  ;
        os << hex << md->Report ;
  os << "> " ;
  os << endl ;

  os << "      MsgType         :"  ;
        os << md->MsgType ;
  os << " <"  ;
        os << hex << md->MsgType ;
  os << "> " ;
  os << endl ;

  os << "      Expiry          :"  ;
        os << md->Expiry  ;
  os << " <"  ;
        os << hex << md->Expiry ;
  os << "> " ;
  os << endl ;

  os << "      Feedback        :"  ;
        os << md->Feedback ;
  os << " <"  ;
        os << hex << md->Feedback ;
  os << "> " ;
  os << endl ;

  os << "      Encoding        :"  ;
        os << md->Encoding ;
  os << " <"  ;
        os << hex << md->Encoding ;
  os << "> " ;
  os << endl ;

  os << "      CodedCharSetId  :"  ;
        os << md->CodedCharSetId ;
  os << " <"  ;
        os << hex << md->CodedCharSetId ;
  os << "> " ;
  os << endl ;

  os << "      Format          :"  ;
        for(i=0; i < sizeof(MQCHAR8) ;
            os << md->Format??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR8) ;
            os << hex << (unsigned short)md->Format??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      Priority        :"  ;
        os << md->Priority ;
  os << " <"  ;
        os << hex << md->Priority ;
  os << "> " ;
  os << endl ;

  os << "      Persistence     :"  ;
        os << md->Persistence ;
  os << " <"  ;
        os << hex << md->Persistence ;
  os << "> " ;
  os << endl ;

  os << "      MsgId           :"  ;
        for(i=0; i < sizeof(MQBYTE24) ;
            os << md->MsgId??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQBYTE24) ;
            os << hex << (unsigned short)md->MsgId??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      CorrelId        :"  ;
        for(i=0; i < sizeof(MQBYTE24) ;
            os << md->CorrelId??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQBYTE24) ;
            os << hex << (unsigned short)md->CorrelId??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      BackoutCount    :"  ;
        os << md->BackoutCount ;
  os << " <"  ;
        os << hex << md->BackoutCount ;
  os << "> " ;
  os << endl ;

  os << "      ReplyToQ        :"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << md->ReplyToQ??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << hex << (unsigned short)md->ReplyToQ??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      ReplyToQMgr     :"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << md->ReplyToQMgr??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR48) ;
            os << hex << (unsigned short)md->ReplyToQMgr??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      UserIdentifier  :"  ;
        for(i=0; i < sizeof(MQCHAR12) ;
            os << md->UserIdentifier??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR12) ;
            os << hex << (unsigned short)md->UserIdentifier??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      AccountingToken :"  ;
        for(i=0; i < sizeof(MQBYTE32) ;
            os << md->AccountingToken??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQBYTE32) ;
            os << hex << (unsigned short)md->AccountingToken??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      ApplIdentityData:"  ;
        for(i=0; i < sizeof(MQCHAR32) ;
            os << md->ApplIdentityData??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR32) ;
            os << hex << (unsigned short)md->ApplIdentityData??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      PutApplType     :"  ;
        os << md->PutApplType ;
  os << " <"  ;
        os << hex << md->PutApplType ;
  os << "> " ;
  os << endl ;

  os << "      PutApplName     :"  ;
        for(i=0; i < sizeof(MQCHAR28) ;
            os << md->PutApplName??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR28) ;
            os << hex << (unsigned short)md->PutApplName??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      PutDate         :"  ;
        for(i=0; i < sizeof(MQCHAR8) ;
            os << md->PutDate??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR8) ;
            os << hex << (unsigned short)md->PutDate??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      PutTime         :"  ;
        for(i=0; i < sizeof(MQCHAR8) ;
            os << md->PutTime??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR8) ;
            os << hex << (unsigned short)md->PutTime??(i++??)) ;
  os << "> " ;
  os << endl ;

  os << "      ApplOriginData  :"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << md->ApplOriginData??(i++??)) ;
  os << " <"  ;
        for(i=0; i < sizeof(MQCHAR4) ;
            os << hex << (unsigned short)md->ApplOriginData??(i++??)) ;
  os << "> " ;
  os << endl ;

  return (os) ;

 }


//
// Internal Functions
//

//
// Tracing is controlled by the DEBUG macro.
//
//         If the dynamic usage of DEBUG is set, then the
//            istrace routine determines whether or not data
//            is written to cout (based on the RXMQRTRACE Rexx var).
//

//
// istrace will obtain the trace setting currently in effect, match
//                       up a substring of the request, and return
//                       TRUE if that trace is active.
//
//         Known settings are :
//
//                              CONN  -> mqconn
//                              DISC  -> mqdisc
//                              OPEN  -> mqopen
//                              CLOSE -> mqopen
//                              GET   -> mqget
//                              PUT   -> mqput
//                              INQ   -> mqinq
//                              SET   -> mqset
//                              CMIT  -> mqcmit
//                              BACK  -> mqback
//
//                              BRO   -> Browse extension
//
//                              HXT   -> Header extraction extension
//                              EVENT -> Event determination extension
//                              TM    -> Trigger message extension
//
//                              MMD   -> Rexx stem var --> MQMD
//                              MOD   -> Rexx stem var --> MQOD
//                              MPO   -> Rexx stem var --> MQPMO
//                              MGO   -> Rexx stem var --> MQGMO
//
//                              BMD   -> MQMD  --> Rexx stem var
//                              BOD   -> MQOD  --> Rexx stem var
//                              BPO   -> MQPMO --> Rexx stem var
//                              BGO   -> MQGMO --> Rexx stem var
//
//                              GV    -> Obtaining a Rexx Variable
//                              PV    -> Setting   a Rexx Variable
//                              SK    -> Return Code processing
//                              TR    -> Thread based processing
//
//                              INIT  -> Initialisation processing
//                              TERM  -> Deregistration processing
//
//                              *     -> Trace everything!!!
//
//
//  These settings are extracted from the RXMQRTRACE Rexx Variable
//    (so RXMQRTRACE = 'INQ OPEN' will trace Open and Inq operations)
//
//
//

int  istrace ( const char * test )   //Test setting
{
 RXSTRING                varname          ;  // Variable name
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 unsigned char           varnamc??(600??) ;  // Char version of var name
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver
 int                     tf      = FALSE  ;  // Return setting (unfound)


 DEBUGS( "Entering istrace "        ) ;
 DEBUGS( "Test is <" << test << ">" ) ;

 if ( strstr(test          ,"*" ) != 0 ) tf = TRUE ;

 strcpy((char *)varnamc,"RXMQRTRACE")                              ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if ( (sv1rc == RXSHV_OK) && (varvalu.strlength != 0) )
    {
      if ( strstr((char *)varvalu.strptr,"*" ) != 0 ) tf = TRUE ;
      if ( strstr((char *)varvalu.strptr,test) != 0 ) tf = TRUE ;
    } ;

 DEBUGS( "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength << " = " << tf ) ;

 DEBUGS( "Leaving istrace " ) ;

 return(tf) ;

} // End of istrace function

//
//  Functions to extract data from Rexx Variables
//
//       get_var   : <varname>   -> RexxString
//       get_varl  : <varname>   -> unsigned long
//       get_var0  : <varname.>0 -> unsigned long
//       get_var1  : <varname.>1 -> RexxString
//
//       where <variable> is the NAME of the Rexx Variable whose
//                           value is to be obtained
//
//             <variable.> indicates that the named variable is
//                           a Rexx Stem variable (so the name
//                           ends in a dot)
//
//       The functions that return data as a RXSTRING require as input
//           the RXSTRING in question; then string pointer part will
//           be updated to contain a pointer to a Rexx-obtained
//           area. This area must be Freemained when usage of the
//           RXString has ended.
//
//       I have deliberately duplicated these functions, so that
//         processing can be readily adapted for subsequent
//         customer extension (although I know this is a bad show!).
//


//
// get_var will obtain a Rexx item as a RXSTRING
//

void get_var (  RXSTRING        name ,  //Var name (without .1)
                RXSTRING      * valu    //Var value
             )
{
 RXSTRING                varname          ;  // Variable name
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varnamc??(600??) ;  // Char version of var name
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver


 DEBUG("GV", "Entering get_var for " << name.strptr ) ;

 sprintf((char *)varnamc,"%s",name.strptr)                         ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 varvalu.strlength= 0                                              ;
 varvalu.strptr   = 0                                              ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 valu->strlength = sv1.shvvalue.strlength                          ;
 valu->strptr    = sv1.shvvalue.strptr                             ;

 DEBUG("GV", "rc " << sv1rc << " " << varnamc << " ->" << valu->strptr
          << "<- " << strlen((const char *)valu->strptr)
          << "/"   << valu->strlength ) ;

 DEBUG("GV", "Leaving get_var " ) ;

 return ;

} // End of get_var function

//
// get_varl will obtain the stem.n of a REXX Stem variable as an
//                                      unsigned long.
//

void get_varl (  RXSTRING    name   ,  //Var name
                 MQLONG    * valu      //Var value
              )
{
 RXSTRING                varname          ;  // Variable name
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 unsigned char           varnamc??(600??) ;  // Char version of var name
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver

 DEBUG("GV", "Entering get_varl for " << name.strptr ) ;

 strcpy((char *)varnamc,(const char *)name.strptr)                 ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if ( (sv1rc == RXSHV_OK) && (varvalu.strlength != 0) )
           *valu = number ;
      else *valu = 0      ;

 DEBUG("GV", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
          << "<- " << strlen((const char *)varvalu.strptr)
          << "/"   << varvalu.strlength ) ;


 DEBUG("GV", "Leaving get_varl " ) ;

 return ;

} // End of get_varl function

//
// get_var0 will obtain the stem.0 of a REXX Stem variable as an
//                                      unsigned long.
//

void get_var0 (  RXSTRING    name ,  //Var name (without .0)
                 MQLONG    * valu    //Var value
              )
{
 RXSTRING                varname          ;  // Variable name
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 unsigned char           varnamc??(600??) ;  // Char version of var name
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver

 DEBUG("GV", "Entering get_var0 for " << name.strptr ) ;

 sprintf((char *)varnamc,"%s0",name.strptr)                        ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if ( (sv1rc == RXSHV_OK) && (varvalu.strlength != 0) )
           *valu = number ;
      else *valu = 0      ;

 DEBUG("GV", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
          << "<- " << strlen((const char *)varvalu.strptr)
          << "/"   << varvalu.strlength ) ;


 DEBUG("GV", "Leaving get_var0 " ) ;

 return ;

} // End of get_var0 function

//
// get_var1 will obtain the stem.1 of a REXX Stem variable as a
//                                      RXSTRING
//

void get_var1 (  RXSTRING        name ,  //Var name (without .1)
                 RXSTRING      * valu    //Var value
              )
{
 RXSTRING                varname          ;  // Variable name
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varnamc??(600??) ;  // Char version of var name
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver


 DEBUG("GV", "Entering get_var1 for " << name.strptr ) ;

 sprintf((char *)varnamc,"%s1",name.strptr)                        ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 valu->strlength = sv1.shvvalue.strlength                          ;
 valu->strptr    = sv1.shvvalue.strptr                             ;

 DEBUG("GV", "rc " << sv1rc << " " << varnamc << " ->" << valu->strptr
          << "<- " << strlen((const char *)valu->strptr)
          << "/"   << valu->strlength ) ;

 DEBUG("GV", "Leaving get_var1 " ) ;


 return ;

} // End of get_var1 function

//
//  Functions to place data in Rexx Variables
//
//       put_var   : RexxString    -> <varname>
//       put_var0  : unsigned long -> <varname.>0
//       put_var1  : RexxString    -> <varname.>1
//
//       where <variable> is the NAME of the Rexx Variable whose
//                           value is to be set
//
//             <variable.> indicates that the named variable is
//                           a Rexx Stem variable (so the name
//                           ends in a dot)
//
//       Note that, as all Rexx Variables are character strings,
//            the general use of put_var requires that the data
//            has already been converted into a string format if
//            it was of a numeric type.
//
//       I have deliberately duplicated these functions, so that
//         processing can be readily adapted for subsequent
//         customer extension (although I know this is a bad show!).
//
//       I know that I could have used C++ polymorphic matching
//         to use merely a single function name and get the right
//         one with overloading ..... but call me old fashioned, I
//         happen to think that with non-class functions it is better
//         to avoid this and use explicit function names.
//
//

//
// put_var0 will set the stem.0 of a REXX Stem variable as an
//                                   signed long.
//

void put_var0 (  RXSTRING    name ,  //Var name (without .0)
                 long        valu    //Var value
              )
{
 RXSTRING                varname          ;  // Variable name
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 unsigned char           varnamc??(600??) ;  // Char version of var name
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver


 DEBUG("PV", "Entering put_var0 for " << name.strptr) ;
 DEBUG("PV", varnamc << " ->" << varvalu.strptr
          << "<- " << strlen((const char *)varvalu.strptr)
          << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%lu",valu)                               ;
 sprintf((char *)varnamc,"%s0",name.strptr)                        ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("PV", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
          << "<- " << strlen((const char *)varvalu.strptr)
          << "/"   << varvalu.strlength ) ;

 DEBUG("PV", "Leaving put_var0 " ) ;


 return ;

} // End of put_var0 function

//
// put_var1 will set the stem.1 of a REXX Stem variable as a
//                                   RXSTRING
//

void put_var1 (  RXSTRING        name  ,  //Var name (without .1)
                 MQLONG          valul ,  //Var value length
                 void          * valu     //Var value
              )
{
 RXSTRING                varname          ;  // Variable name
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 unsigned char           varnamc??(600??) ;  // Char version of var name
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC


 DEBUG("PV", "Entering put_var1 for " << name.strptr) ;
 DEBUG("PV", "Pointer is " << hex << valu << " Len is " << valul ) ;

 sprintf((char *)varnamc,"%s1",name.strptr)                        ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,valu,valul)                                  ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = valul                                           ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("PV", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
          << "<- " << strlen((const char *)varvalu.strptr)
          << "/"   << varvalu.strlength ) ;

 DEBUG("PV", "Leaving put_var1 " ) ;

 return ;

} // End of put_var1 function

//
// put_var will write a Rexx Variable
//
//

void put_var (  RXSTRING name ,  //Var value
                RXSTRING valu    //Var name
             )
{
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC


 DEBUG("PV", "Entering put_var for " << name.strptr) ;

 sv1.shvnext      = 0                            ;
 memcpy(&sv1.shvname,&name,sizeof(name))         ;
 sv1.shvnamelen  = name.strlength                ;
 memcpy(&sv1.shvvalue,&valu,sizeof(valu))        ;
 sv1.shvvaluelen = valu.strlength                ;
 sv1.shvcode     = RXSHV_SYSET                   ;
 sv1.shvret      = 0                             ;
 sv1rc           = RexxVariablePool(&sv1)        ;

 DEBUG("PV", "rc " << sv1rc << " " << name.strptr << " ->" << valu.strptr
          << "<- " << strlen((const char *)valu.strptr)
          << "/"   << valu.strlength ) ;

 DEBUG("PV", "Leaving put_var " ) ;

 return ;
} // End of put_var function

//
// Return processing functions
//
//      stack_and_set   : sets up the return variables
//

//
// stack_and_set will update the Rexx RXMQR status variables
//
//
//          RXMQR.LASTRC    -> current operation Return Code
//          RXMQR.LASTCC    -> current operation MQ Completion Code
//          RXMQR.LASTAC    -> current operation MQ Reason     Code
//          RXMQR.LASTOP    -> current operation RXMQR function
//          RXMQR.LASTMSG   -> current operation text message
//
//          The Return Code for the operation can be negative to
//              show that this DLL has detected the error, otherwise
//              it will be the MQ Completion Code
//
//          The Message is in text format as follows:
//
//            Word 1 : Return Code
//            Word 2 : MQ Completion Code (or 0 if MQ not done)
//            Word 2 : MQ Reason     Code (or 0 if MQ not done)
//            Word 4 : RXMQR function being run
//            Word > : OK or an helpful error message
//
//

void stack_and_set ( const LONG            rc , //Function return Code
                     const MQLONG          cc , //MQ Completion Code
                     const MQLONG          ac , //MQ Reason Code
                     const char          * op , //Function name
                     const unsigned char * ms   //Function return string
                   )
{
 RXSTRING                varname          ;  // Variable name
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC


 DEBUG("SK", "Entering stack_and_set " ) ;

 MAKERXSTRING(varname,"RXMQR.LASTRC",sizeof("RXMQR.LASTRC")-1)     ;
 sprintf((char *)varvalc,"%li",rc)                                 ;
 varvalu.strlength= strlen((const char *)varvalc)                  ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("SK", "rc " << sv1rc << " " << varname.strptr
          << "-> " << varvalu.strptr
          << "/"   << varvalu.strlength ) ;


 MAKERXSTRING(varname,"RXMQR.LASTCC",sizeof("RXMQR.LASTCC")-1)     ;
 sprintf((char *)varvalc,"%lu",cc)                                 ;
 varvalu.strlength= strlen((const char *)varvalc)                  ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("SK", "rc " << sv1rc << " " << varname.strptr
          << "-> " << varvalu.strptr
          << "/"   << varvalu.strlength ) ;

 MAKERXSTRING(varname,"RXMQR.LASTAC",sizeof("RXMQR.LASTAC")-1)     ;
 sprintf((char *)varvalc,"%lu",ac)                                 ;
 varvalu.strlength= strlen((const char *)varvalc)                  ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("SK", "rc " << sv1rc << " " << varname.strptr
          << "-> " << varvalu.strptr
          << "/"   << varvalu.strlength ) ;

 MAKERXSTRING(varname,"RXMQR.LASTOP",sizeof("RXMQR.LASTOP")-1)     ;
 strcpy((char *)varvalc,(const char *)op)                          ;
 varvalu.strlength= strlen((const char *)varvalc)                  ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("SK", "rc " << sv1rc << " " << varname.strptr
          << "-> " << varvalu.strptr
          << "/"   << varvalu.strlength ) ;

 MAKERXSTRING(varname,"RXMQR.LASTMSG",sizeof("RXMQR.LASTMSG")-1)   ;
 strcpy((char *)varvalc,(const char *)ms)                          ;
 varvalu.strlength= strlen((const char *)varvalc)                  ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("SK", "rc " << sv1rc << " " << varname.strptr
          << "-> " << varvalu.strptr
          << "/"   << varvalu.strlength ) ;


 DEBUG("SK", "Leaving stack_and_set " ) ;

 return ;
} // End of stack_and_set function

//
// Functions that manipulate MQ objects and Rexx Variables
//
//           The make_??_from_stem  functions build the ??
//                                  MQ object by trying to reference
//                                  a stem variable (whose name ending
//                                  with a dot) whose name is supplied.
//                                  If the .xx part is present, then
//                                  the contents are copied into the
//                                  MQ Object.
//
//
//            The make_stem_from_?? functions work the other way around.
//                                  they are supplied with the name of
//                                  a stem variable (including the dot)
//                                  and build a Rexx representatation of
//                                  the MQ object by setting xx.yy
//                                  variables, where the yy is a
//                                  component of the MQ object. In addition,
//                                  (.)ZLIST is set to a list of the components
//                                  in the stem (without the leading dot).
//
//
//   The names of the .extensions are described in the functions.
//
//   Note that the make_stem_from_?? functions will AWAYS build
//        all of the .extensions, whereas the make_??_from_stem
//        functions can cope with the omission of extensions.
//
//

//
// make_od_from_stem will return an Object Descriptor from the
//                   contents of a Stem Variable:
//
//                     .OT   -> ObjectType
//                     .ON   -> ObjectName
//                     .OQM  -> ObjectQMgrName
//                     .DQN  -> DynamicQueue
//                     .AUID -> AlternateUserid
//

void make_od_from_stem ( MQOD     * odopen , //Object desc to be built
                         RXSTRING   stem     //name of stem variable
                       )
{
 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(600??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver
 unsigned int            varlen           ;  // Length of Stem.? var


 DEBUG("MOD", "Entering make_od_from_stem " ) ;


 memset(odopen,0,sizeof(MQOD))                                  ;
 strncpy(odopen->StrucId,MQOD_STRUC_ID,sizeof(odopen->StrucId)) ;
 odopen->Version  = MQOD_VERSION_1                              ;
 memcpy(&odopen->DynamicQName,"AMQ.*",sizeof("AMQ.*"))          ;

 //If the given variable is not a stem. variable, then take the
 //   quicker option of assuming it's a Queue to open

 if ( stem.strptr??((strlen((const char *)stem.strptr)-1)??) != '.' )
    {
     odopen->ObjectType = MQOT_Q                ;
     memcpy(&odopen->ObjectName,
            stem.strptr,
            strlen((const char *)stem.strptr) ) ;
     DEBUG("MOD",  odopen ) ;
     return ;
    }

 //The given variable is a stem. variable, so get its contents

 sprintf((char *)varnamc,"%sOT",stem.strptr)                       ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      odopen->ObjectType = number                                  ;
                          else
      odopen->ObjectType = MQOT_Q                                  ;

 DEBUG("MOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sON",stem.strptr)                       ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR48) )
            varvalu.strlength = sizeof(MQCHAR48) ;

        memcpy(&odopen->ObjectName,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sOQM",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
      if (varvalu.strlength > sizeof(MQCHAR48) )
          varvalu.strlength = sizeof(MQCHAR48) ;

      memcpy(&odopen->ObjectQMgrName,
             varvalu.strptr,
             strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sDQM",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
      if (varvalu.strlength > sizeof(MQCHAR48) )
          varvalu.strlength = sizeof(MQCHAR48) ;

      memcpy(&odopen->DynamicQName,
             varvalu.strptr,
             strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sAUID",stem.strptr)                     ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
      if (varvalu.strlength > sizeof(MQCHAR12) )
          varvalu.strlength = sizeof(MQCHAR12) ;

      memcpy(&odopen->AlternateUserId,
             varvalu.strptr,
             strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 DEBUG("MOD", odopen ) ;


 DEBUG("MOD",  "Leaving make_od_from_stem " ) ;


 return ;
} // End of make_od_from_stem function

//
// make_stem_from_od will return an Object Descriptor as the
//                   contents of a Stem Variable:
//
//                     .OT    -> ObjectType
//                     .ON    -> ObjectName
//                     .OQM   -> ObjectQMgrName
//                     .DQN   -> DynamicQueue
//                     .AUID  -> AlternateUserid
//
//                     .ZLIST -> 'AUID ON OT OQM DQN'
//

void make_stem_from_od ( MQOD     * odopen , //Object desc to be built
                         RXSTRING   stem     //name of stem variable
                       )
{
 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(600??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver


 DEBUG("BOD", "Entering make_stem_from_od " ) ;

 DEBUG("BOD", odopen ) ;

 sprintf((char *)varvalc,"%lu",odopen->ObjectType)                 ;
 sprintf((char *)varnamc,"%sOT",stem.strptr)                       ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),odopen->ObjectName) ;
 sprintf((char *)varnamc,"%sON",stem.strptr)                       ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),odopen->ObjectQMgrName);
 sprintf((char *)varnamc,"%sOQM",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),odopen->DynamicQName);
 sprintf((char *)varnamc,"%sDQN",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR12),odopen->AlternateUserId);
 sprintf((char *)varnamc,"%sAUID",stem.strptr)                     ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"AUID ON OQM OT DQN")                     ;
 sprintf((char *)varnamc,"%sZLIST",stem.strptr)                    ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BOD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 DEBUG("BOD", "Leaving make_stem_from_od " ) ;

 return ;
} // End of make_stem_from_od function

//
// make_po_from_stem will return a Put Message Options Desc from the
//                   contents of a Stem Variable:
//
//                     .OPT   -> Options
//                     .TIME  -> Timeout
//                     .CON   -> Context
//                     .KDC   -> KnownDestCount
//                     .UDC   -> UnKnownDestCount
//                     .IDC   -> InvalidDestCount
//                     .RQN   -> ResolvedQueueName
//                     .RQMN  -> ResolvedQueueMgrName
//

void make_po_from_stem ( MQPMO    * pmo    , //PMO to be built
                         RXSTRING   stem     //name of stem variable
                       )
{
 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(600??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver
 unsigned int            varlen           ;  // Length of Stem.? var


 DEBUG("MPO", "Entering make_po_from_stem " ) ;


 memset(pmo,0,sizeof(MQPMO))                                  ;
 strncpy(pmo->StrucId,MQPMO_STRUC_ID,sizeof(pmo->StrucId))    ;
 pmo->Version  = MQPMO_VERSION_1                              ;
 pmo->Timeout  = -1                                           ;


 sprintf((char *)varnamc,"%sOPT",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
         pmo->Options = number ;
      }

 DEBUG("MPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sTIME",stem.strptr)                     ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        pmo->Timeout = number ;
      }

 DEBUG("MPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sCON",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        memcpy(&pmo->Context,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sKDC",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        pmo->KnownDestCount = number ;
      }

 DEBUG("MPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sUDC",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        pmo->UnknownDestCount = number ;
      }

 DEBUG("MPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sIDC",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        pmo->InvalidDestCount = number ;
      }

 DEBUG("MPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sRQN",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR48) )
            varvalu.strlength = sizeof(MQCHAR48) ;

        memcpy(&pmo->ResolvedQName,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sRQMN",stem.strptr)                     ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR48) )
            varvalu.strlength = sizeof(MQCHAR48) ;

        memcpy(&pmo->ResolvedQMgrName,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 DEBUG("MPO", pmo ) ;

 DEBUG("MPO", "Leaving make_po_from_stem " ) ;


 return ;
} // End of make_po_from_stem function

//
// make_stem_from_po will return a Put Message Options Desc as the
//                   contents of a Stem Variable:
//
//                     .OPT   -> Options
//                     .TIME  -> Timeout
//                     .CON   -> Context
//                     .KDC   -> KnownDestCount
//                     .UDC   -> UnKnownDestCount
//                     .IDC   -> InvalidDestCount
//                     .RQN   -> ResolvedQueueName
//                     .RQMN  -> ResolvedQueueMgrName
//
//                     .ZLIST -> 'CON IDC KDC OPT RQMN RQN TIME UDC'
//

void make_stem_from_po ( MQPMO    * pmo    , //PMO to be built
                         RXSTRING   stem     //name of stem variable
                       )
{
 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(600??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver


 DEBUG("BPO", "Entering make_stem_from_po " ) ;

 DEBUG("BPO",  pmo ) ;

 sprintf((char *)varvalc,"%lu",pmo->Options)                       ;
 sprintf((char *)varnamc,"%sOPT",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",pmo->Timeout)                       ;
 sprintf((char *)varnamc,"%sTIME",stem.strptr)                     ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",pmo->Context)                       ;
 sprintf((char *)varnamc,"%sCON",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",pmo->KnownDestCount)                ;
 sprintf((char *)varnamc,"%sKDC",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",pmo->UnknownDestCount)              ;
 sprintf((char *)varnamc,"%sUDC",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",pmo->InvalidDestCount)              ;
 sprintf((char *)varnamc,"%sIDC",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),pmo->ResolvedQName);
 sprintf((char *)varnamc,"%sRQN",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),pmo->ResolvedQMgrName);
 sprintf((char *)varnamc,"%sRQMN",stem.strptr)                     ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"CON IDC KDC OPT RQMN RQN TIME UDC")      ;
 sprintf((char *)varnamc,"%sZLIST",stem.strptr)                    ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BPO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 DEBUG("BPO",  "Leaving make_stem_from_po " ) ;

 return ;
} // End of make_stem_from_po function

//
// make_go_from_stem will return a Get Message Options Desc from the
//                   contents of a Stem Variable:
//
//                     .OPT   -> Options
//                     .WAIT  -> WaitInterval
//                     .RQN   -> ResolvedQueueName
//

void make_go_from_stem ( MQGMO    * gmo    , //GMO to be built
                         RXSTRING   stem     //name of stem variable
                       )
{
 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(600??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver
 unsigned int            varlen           ;  // Length of Stem.? var


 DEBUG("MGO",  "Entering make_go_from_stem " ) ;

 memset(gmo,0,sizeof(MQGMO))                                  ;
 strncpy(gmo->StrucId,MQGMO_STRUC_ID,sizeof(gmo->StrucId))    ;
 gmo->Version  = MQGMO_VERSION_1                              ;


 sprintf((char *)varnamc,"%sOPT",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        gmo->Options = number ;
      }

 DEBUG("MGO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sWAIT",stem.strptr)                     ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        gmo->WaitInterval = number ;
      }

 DEBUG("MGO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sRQN",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR48) )
            varvalu.strlength = sizeof(MQCHAR48) ;

        memcpy(&gmo->ResolvedQName,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;

      }
 DEBUG("MGO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 DEBUG("MGO",  gmo ) ;

 DEBUG("MGO",  "Leaving make_go_from_stem " ) ;


 return ;
} // End of make_go_from_stem function

//
// make_stem_from_go will return a Get Message Options Desc as the
//                   contents of a Stem Variable:
//
//                     .OPT   -> Options
//                     .WAIT  -> WaitInterval
//                     .RQN   -> ResolvedQueueName
//
//                     .ZLIST -> 'OPT RQN WAIT'
//

void make_stem_from_go ( MQGMO    * gmo    , //GMO to be built
                         RXSTRING   stem     //name of stem variable
                       )
{
 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(600??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver


 DEBUG("BGO",  "Entering make_stem_from_go " ) ;

 DEBUG("BGO",  gmo ) ;

 sprintf((char *)varvalc,"%lu",gmo->Options)                       ;
 sprintf((char *)varnamc,"%sOPT",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BGO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",gmo->WaitInterval)                  ;
 sprintf((char *)varnamc,"%sWAIT",stem.strptr)                     ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BGO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),gmo->ResolvedQName) ;
 sprintf((char *)varnamc,"%sRQN",stem.strptr)                        ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))         ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))         ;
 sv1.shvnext      = 0                                                ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                      ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                      ;
 sv1.shvvaluelen = varvalu.strlength                                 ;
 sv1.shvcode     = RXSHV_SYSET                                       ;
 sv1.shvret      = 0                                                 ;
 sv1rc           = RexxVariablePool(&sv1)                            ;

 DEBUG("BGO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"OPT RQN WAIT")                             ;
 sprintf((char *)varnamc,"%sZLIST",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))         ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))         ;
 sv1.shvnext      = 0                                                ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                      ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                      ;
 sv1.shvvaluelen = varvalu.strlength                                 ;
 sv1.shvcode     = RXSHV_SYSET                                       ;
 sv1.shvret      = 0                                                 ;
 sv1rc           = RexxVariablePool(&sv1)                            ;

 DEBUG("BGO", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 DEBUG("BGO",  "Leaving make_stem_from_go " ) ;

 return ;
} // End of make_stem_from_go function

//
// make_md_from_stem will return a Put Message Options Desc from the
//                   contents of a Stem Variable:
//
//                     .REP   -> Report
//                     .MSG   -> MsgType
//                     .EXP   -> Expiry
//                     .FBK   -> Feedback
//                     .ENC   -> Encoding
//                     .CCSI  -> CodedCharSetId
//                     .FORM  -> Format
//                     .PRI   -> Priority
//                     .PER   -> Persistence
//                     .MSGID -> MsgId
//                     .CID   -> CorrelId
//                     .BC    -> BackoutCount
//                     .RTOQ  -> ReplyToQ
//                     .RTOQM -> ReplyToQMgr
//                     .UID   -> UserIdentifier
//                     .AT    -> AccountingToken
//                     .AID   -> ApplyIdentityData
//                     .PAT   -> PutApplType
//                     .PAN   -> PutApplName
//                     .PD    -> PutDate
//                     .PT    -> PutTime
//                     .AOD   -> ApplOriginData
//
//        Note that MSGID, CID, AT, AID and AOD are directly passed
//

void make_md_from_stem ( MQMD     * md     , //MQMD to be built
                         RXSTRING   stem     //name of stem variable
                       )
{
 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(600??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver
 unsigned int            varlen           ;  // Length of Stem.? var



 DEBUG("MMD",  "Entering make_md_from_stem " ) ;

 memset(md,0,sizeof(MQMD))                                   ;
 strncpy(md->StrucId,MQMD_STRUC_ID,sizeof(md->StrucId))      ;
 md->Version     = MQMD_VERSION_1                            ;
 md->MsgType     = MQMT_DATAGRAM                             ;
 md->Expiry      = MQEI_UNLIMITED                            ;
 md->Encoding    = MQENC_NATIVE                              ;
 md->Priority    = MQPRI_PRIORITY_AS_Q_DEF                   ;
 md->Persistence = MQPER_PERSISTENCE_AS_Q_DEF                ;


 sprintf((char *)varnamc,"%sREP",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->Report = number ;

      }
 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sMSG",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->MsgType = number ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sEXP",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->Expiry = number ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sFBK",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->Feedback = number ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sENC",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->Encoding = number ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sCCSI",stem.strptr)                     ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->CodedCharSetId = number ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sFORM",stem.strptr)                     ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR8) )
            varvalu.strlength = sizeof(MQCHAR8) ;

        memcpy(&md->Format,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sPRI",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->Priority = number ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sPER",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->Persistence = number ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sMSGID",stem.strptr)                    ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQBYTE24) )
            varvalu.strlength = sizeof(MQBYTE24) ;

        memcpy(&md->MsgId,
               varvalu.strptr,
               varvalu.strlength ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sCID",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQBYTE24) )
            varvalu.strlength = sizeof(MQBYTE24) ;

        memcpy(&md->CorrelId,
               varvalu.strptr,
               varvalu.strlength ) ;
      }

 sprintf((char *)varnamc,"%sBC",stem.strptr)                       ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->BackoutCount = number ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sRTOQ",stem.strptr)                     ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR48) )
            varvalu.strlength = sizeof(MQCHAR48) ;

        memcpy(&md->ReplyToQ,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sRTOQM",stem.strptr)                    ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR48) )
            varvalu.strlength = sizeof(MQCHAR48) ;

        memcpy(&md->ReplyToQMgr,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sUID",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR12) )
            varvalu.strlength = sizeof(MQCHAR12) ;

        memcpy(&md->UserIdentifier,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sAT",stem.strptr)                       ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQBYTE32) )
            varvalu.strlength = sizeof(MQBYTE32) ;

        memcpy(&md->AccountingToken,
               varvalu.strptr,
               varvalu.strlength ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sAID",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR32) )
            varvalu.strlength = sizeof(MQCHAR32) ;

        memcpy(&md->ApplIdentityData,
               varvalu.strptr,
               varvalu.strlength ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sPAT",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 number          = atol((const char *)varvalu.strptr)              ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        md->PutApplType = number ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varnamc,"%sPAN",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR28) )
            varvalu.strlength = sizeof(MQCHAR28) ;

        memcpy(&md->PutApplName,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sPD",stem.strptr)                       ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR8) )
            varvalu.strlength = sizeof(MQCHAR8) ;

        memcpy(&md->PutDate,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sPT",stem.strptr)                       ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR8) )
            varvalu.strlength = sizeof(MQCHAR8) ;

        memcpy(&md->PutTime,
               varvalu.strptr,
               strlen((const char *)varvalu.strptr) ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varnamc,"%sAOD",stem.strptr)                      ;
 varlen = strlen((const char *)varnamc)                            ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 memset(&varvalu,0,sizeof(varvalu))                                ;
 memset(&varvalc,0,sizeof(varvalc))                                ;
 varvalu.strlength= sizeof(varvalc)                                ;
 varvalu.strptr    = (char *)varvalc                                        ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 sv1.shvnamelen  = varname.strlength                               ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYFET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;
 if (    (sv1rc                                    == RXSHV_OK)
      && (varvalu.strlength                        != 0       )
      && (memicmp(varnamc, varvalu.strptr, varlen) != 0       ) )
      {
        if (varvalu.strlength > sizeof(MQCHAR4) )
            varvalu.strlength = sizeof(MQCHAR4) ;

        memcpy(&md->ApplOriginData,
               varvalu.strptr,
               varvalu.strlength ) ;
      }

 DEBUG("MMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 DEBUG("MMD",  md ) ;

 DEBUG("MMD",  "Leaving make_md_from_stem " ) ;

 return ;
} // End of make_md_from_stem function

//
// make_stem_from_md will return a Message Descriptor as the
//                   contents of a Stem Variable:
//
//                     .REP   -> Report
//                     .MSG   -> MsgType
//                     .EXP   -> Expiry
//                     .FBK   -> Feedback
//                     .ENC   -> Encoding
//                     .CCSI  -> CodedCharSetId
//                     .FORM  -> Format
//                     .PRI   -> Priority
//                     .PER   -> Persistence
//                     .MSGID -> MsgId
//                     .CID   -> CorrelId
//                     .BC    -> BackoutCount
//                     .RTOQ  -> ReplyToQ
//                     .RTOQM -> ReplyToQMgr
//                     .UID   -> UserIdentifier
//                     .AT    -> AccountingToken
//                     .AID   -> ApplyIdentityData
//                     .PAT   -> PutApplType
//                     .PAN   -> PutApplName
//                     .PD    -> PutDate
//                     .PT    -> PutTime
//                     .AOD   -> ApplOriginData
//
//                     .ZLIST -> AID AOD AT BC CID CCSI ENC EXP FBK
//                               FORM MSG MSGID PAN PAT PD PER PRI
//                               PT REP RTOQ RTOQM UID
//
//       Note that MSGID, CID, AT, AID and AOD are moved en-block
//

void make_stem_from_md ( MQMD     * md     , //MQMD to be built
                         RXSTRING   stem     //name of stem variable
                       )
{
 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(600??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver


 DEBUG("BMD",  "Entering make_stem_from_md " ) ;


 DEBUG("BMD",  md ) ;

 sprintf((char *)varvalc,"%lu",md->Report)                         ;
 sprintf((char *)varnamc,"%sREP",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%lu",md->MsgType)                        ;
 sprintf((char *)varnamc,"%sMSG",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%lu",md->Expiry)                         ;
 sprintf((char *)varnamc,"%sEXP",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%lu",md->Feedback)                       ;
 sprintf((char *)varnamc,"%sFBK",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%lu",md->Encoding)                       ;
 sprintf((char *)varnamc,"%sENC",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%lu",md->CodedCharSetId)                 ;
 sprintf((char *)varnamc,"%sCCSI",stem.strptr)                     ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR8),md->Format);
 sprintf((char *)varnamc,"%sFORM",stem.strptr)                     ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",md->Priority)                       ;
 sprintf((char *)varnamc,"%sPRI",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",md->Persistence)                    ;
 sprintf((char *)varnamc,"%sPER",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 //sprintf((char *)varvalc,"%.*s",sizeof(MQBYTE24),md->MsgId);
 //MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc));
 memcpy(&varvalc,&md->MsgId,sizeof(MQBYTE24))                      ;
 MAKERXSTRING(varvalu,varvalc,sizeof(MQBYTE24))                    ;
 sprintf((char *)varnamc,"%sMSGID",stem.strptr)                    ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 //sprintf((char *)varvalc,"%.*s",sizeof(MQBYTE24),md->CorrelId);
 //MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc)) ;
 memcpy(&varvalc,&md->CorrelId,sizeof(MQBYTE24))                   ;
 MAKERXSTRING(varvalu,varvalc,sizeof(MQBYTE24))                    ;
 sprintf((char *)varnamc,"%sCID",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",md->BackoutCount)                   ;
 sprintf((char *)varnamc,"%sBC",stem.strptr)                       ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),md->ReplyToQ);
 sprintf((char *)varnamc,"%sRTOQ",stem.strptr)                     ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),md->ReplyToQMgr);
 sprintf((char *)varnamc,"%sRTOQM",stem.strptr)                    ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR12),md->UserIdentifier);
 sprintf((char *)varnamc,"%sUID",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 //sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR32),md->AccountingToken);
 //MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc));
 memcpy(&varvalc,&md->AccountingToken,sizeof(MQCHAR32))            ;
 MAKERXSTRING(varvalu,varvalc,sizeof(MQCHAR32))                    ;
 sprintf((char *)varnamc,"%sAT",stem.strptr)                       ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 //sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR32),md->ApplIdentityData);
 //MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc));
 memcpy(&varvalc,&md->ApplIdentityData,sizeof(MQCHAR32))           ;
 MAKERXSTRING(varvalu,varvalc,sizeof(MQCHAR32))                    ;
 sprintf((char *)varnamc,"%sAID",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%lu",md->PutApplType)                    ;
 sprintf((char *)varnamc,"%sPAT",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR28),md->PutApplName);
 sprintf((char *)varnamc,"%sPAN",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR8),md->PutDate);
 sprintf((char *)varnamc,"%sPD",stem.strptr)                       ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR8),md->PutTime);
 sprintf((char *)varnamc,"%sPT",stem.strptr)                       ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 //sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR4),md->ApplOriginData);
 //MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc));
 memcpy(&varvalc,&md->ApplOriginData,sizeof(MQCHAR4))              ;
 MAKERXSTRING(varvalu,varvalc,sizeof(MQCHAR4))                     ;
 sprintf((char *)varnamc,"%sAOD",stem.strptr)                      ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;


 sprintf((char *)varvalc,"AID AOD AT BC CID CCSI ENC EXP FBK FORM MSG MSGID PAN PAT PD PER PRI PT REP RTOQ RTOQM UID" ) ;
 sprintf((char *)varnamc,"%sZLIST",stem.strptr)                    ;
 MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
 MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
 sv1.shvnext      = 0                                              ;
 memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
 memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
 sv1.shvvaluelen = varvalu.strlength                               ;
 sv1.shvcode     = RXSHV_SYSET                                     ;
 sv1.shvret      = 0                                               ;
 sv1rc           = RexxVariablePool(&sv1)                          ;

 DEBUG("BMD", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
           << "<- " << strlen((const char *)varvalu.strptr)
           << "/"   << varvalu.strlength ) ;

 DEBUG("BMD",  "Leaving make_stem_from_md " ) ;

 return ;
} // End of make_stem_from_md function

//
// Genuine Internal functions, not concerned with specific things
//

//
// setcons - sets up the MQ constants in the Rexx Variable Space
//           Traced by INIT setting
//

void setcons ()
 {


//
// Structure to define MQ literals, to be setup in Variable Space
//

 struct s_define_mq_ints
        {
          char           s_define_mq_ints_name??(255??) ;
          ULONG          s_define_mq_ints_value         ;
          ULONG          s_define_mq_ints_type          ;  // 0 -> number
                                                           // 1 -> rc number
                                                           // 2 -> nulls
        } ;

 typedef struct s_define_mq_ints   t_define_mq_ints ;
 typedef        t_define_mq_ints * p_define_mq_ints ;

 //
 //  Usual local variables
 //

 ULONG                    i                ;  // Looper
 RXSTRING                 intname          ;  // Variable name
 RXSTRING                 intvalu          ;  // Variable setting
 p_define_mq_ints         pi               ;  // Table pointer
 unsigned char            intvalc??(600??) ;  // Char version of variable
 unsigned char            returnc??(600??) ;  // Return String
 SHVBLOCK                 sv1              ;  // Var interface CB
 ULONG                    sv1rc            ;  // Var interface RC
 ULONG                    rc               ;  // Function interface

 char                     nulls??(600??)   ;  // Empty Settings

//
// Array and Structure to define MQ literals, to be setup in Variable Space
//

 static struct s_define_mq_ints  define_mq_ints??(??) =
 {

   // General MQ/NT Numeric Literals
 { "MQ_ACCOUNTING_TOKEN_LENGTH"       , MQ_ACCOUNTING_TOKEN_LENGTH      , 0},
 { "MQ_APPL_IDENTITY_DATA_LENGTH"     , MQ_APPL_IDENTITY_DATA_LENGTH    , 0},
 { "MQ_APPL_NAME_LENGTH"              , MQ_APPL_NAME_LENGTH             , 0},
 { "MQ_APPL_ORIGIN_DATA_LENGTH"       , MQ_APPL_ORIGIN_DATA_LENGTH      , 0},
 { "MQ_CHANNEL_DATE_LENGTH"           , MQ_CHANNEL_DATE_LENGTH          , 0},
 { "MQ_CHANNEL_DESC_LENGTH"           , MQ_CHANNEL_DESC_LENGTH          , 0},
 { "MQ_CHANNEL_NAME_LENGTH"           , MQ_CHANNEL_NAME_LENGTH          , 0},
 { "MQ_CHANNEL_TIME_LENGTH"           , MQ_CHANNEL_TIME_LENGTH          , 0},
 { "MQ_CONN_NAME_LENGTH"              , MQ_CONN_NAME_LENGTH             , 0},
 { "MQ_CORREL_ID_LENGTH"              , MQ_CORREL_ID_LENGTH             , 0},
 { "MQ_CREATION_DATE_LENGTH"          , MQ_CREATION_DATE_LENGTH         , 0},
 { "MQ_CREATION_TIME_LENGTH"          , MQ_CREATION_TIME_LENGTH         , 0},
 { "MQ_EXIT_DATA_LENGTH"              , MQ_EXIT_DATA_LENGTH             , 0},
 { "MQ_EXIT_NAME_LENGTH"              , MQ_EXIT_NAME_LENGTH             , 0},
 { "MQ_EXIT_USER_AREA_LENGTH"         , MQ_EXIT_USER_AREA_LENGTH        , 0},
 { "MQ_FORMAT_LENGTH"                 , MQ_FORMAT_LENGTH                , 0},
 { "MQ_LUWID_LENGTH"                  , MQ_LUWID_LENGTH                 , 0},
 { "MQ_MCA_JOB_NAME_LENGTH"           , MQ_MCA_JOB_NAME_LENGTH          , 0},
 { "MQ_MCA_NAME_LENGTH"               , MQ_MCA_NAME_LENGTH              , 0},
 { "MQ_MODE_NAME_LENGTH"              , MQ_MODE_NAME_LENGTH             , 0},
 { "MQ_MSG_HEADER_LENGTH"             , MQ_MSG_HEADER_LENGTH            , 0},
 { "MQ_MSG_ID_LENGTH"                 , MQ_MSG_ID_LENGTH                , 0},
 { "MQ_PASSWORD_LENGTH"               , MQ_PASSWORD_LENGTH              , 0},
 { "MQ_PROCESS_APPL_ID_LENGTH"        , MQ_PROCESS_APPL_ID_LENGTH       , 0},
 { "MQ_PROCESS_DESC_LENGTH"           , MQ_PROCESS_DESC_LENGTH          , 0},
 { "MQ_PROCESS_ENV_DATA_LENGTH"       , MQ_PROCESS_ENV_DATA_LENGTH      , 0},
 { "MQ_PROCESS_NAME_LENGTH"           , MQ_PROCESS_NAME_LENGTH          , 0},
 { "MQ_PROCESS_USER_DATA_LENGTH"      , MQ_PROCESS_USER_DATA_LENGTH     , 0},
 { "MQ_PUT_APPL_NAME_LENGTH"          , MQ_PUT_APPL_NAME_LENGTH         , 0},
 { "MQ_PUT_DATE_LENGTH"               , MQ_PUT_DATE_LENGTH              , 0},
 { "MQ_PUT_TIME_LENGTH"               , MQ_PUT_TIME_LENGTH              , 0},
 { "MQ_Q_DESC_LENGTH"                 , MQ_Q_DESC_LENGTH                , 0},
 { "MQ_Q_MGR_DESC_LENGTH"             , MQ_Q_MGR_DESC_LENGTH            , 0},
 { "MQ_Q_MGR_NAME_LENGTH"             , MQ_Q_MGR_NAME_LENGTH            , 0},
 { "MQ_Q_NAME_LENGTH"                 , MQ_Q_NAME_LENGTH                , 0},
 { "MQ_SHORT_CONN_NAME_LENGTH"        , MQ_SHORT_CONN_NAME_LENGTH       , 0},
 { "MQ_STORAGE_CLASS_LENGTH"          , MQ_STORAGE_CLASS_LENGTH         , 0},
 { "MQ_TP_NAME_LENGTH"                , MQ_TP_NAME_LENGTH               , 0},
 { "MQ_TRIGGER_DATA_LENGTH"           , MQ_TRIGGER_DATA_LENGTH          , 0},
 { "MQ_USER_ID_LENGTH"                , MQ_USER_ID_LENGTH               , 0},
 { "MQAT_AIX"                         , MQAT_AIX                        , 0},
 { "MQAT_CICS"                        , MQAT_CICS                       , 0},
 { "MQAT_CICS_VSE"                    , MQAT_CICS_VSE                   , 0},
 { "MQAT_DEFAULT"                     , MQAT_DEFAULT                    , 0},
 { "MQAT_DOS"                         , MQAT_DOS                        , 0},
 { "MQAT_GUARDIAN"                    , MQAT_GUARDIAN                   , 0},
 { "MQAT_IMS"                         , MQAT_IMS                        , 0},
 { "MQAT_MVS"                         , MQAT_MVS                        , 0},
 { "MQAT_NO_CONTEXT"                  , MQAT_NO_CONTEXT                 , 0},
 { "MQAT_OS2"                         , MQAT_OS2                        , 0},
 { "MQAT_OS400"                       , MQAT_OS400                      , 0},
 { "MQAT_QMGR"                        , MQAT_QMGR                       , 0},
 { "MQAT_UNIX"                        , MQAT_UNIX                       , 0},
 { "MQAT_UNKNOWN"                     , MQAT_UNKNOWN                    , 0},
 { "MQAT_USER_FIRST"                  , MQAT_USER_FIRST                 , 0},
 { "MQAT_USER_LAST"                   , MQAT_USER_LAST                  , 0},
 { "MQAT_VMS"                         , MQAT_VMS                        , 0},
 { "MQAT_VOS"                         , MQAT_VOS                        , 0},
 { "MQAT_WINDOWS"                     , MQAT_WINDOWS                    , 0},
 { "MQCA_APPL_ID"                     , MQCA_APPL_ID                    , 0},
 { "MQCA_BACKOUT_REQ_Q_NAME"          , MQCA_BACKOUT_REQ_Q_NAME         , 0},
 { "MQCA_BASE_Q_NAME"                 , MQCA_BASE_Q_NAME                , 0},
 { "MQCA_COMMAND_INPUT_Q_NAME"        , MQCA_COMMAND_INPUT_Q_NAME       , 0},
 { "MQCA_CREATION_DATE"               , MQCA_CREATION_DATE              , 0},
 { "MQCA_CREATION_TIME"               , MQCA_CREATION_TIME              , 0},
 { "MQCA_DEAD_LETTER_Q_NAME"          , MQCA_DEAD_LETTER_Q_NAME         , 0},
 { "MQCA_DEF_XMIT_Q_NAME"             , MQCA_DEF_XMIT_Q_NAME            , 0},
 { "MQCA_ENV_DATA"                    , MQCA_ENV_DATA                   , 0},
 { "MQCA_FIRST"                       , MQCA_FIRST                      , 0},
 { "MQCA_INITIATION_Q_NAME"           , MQCA_INITIATION_Q_NAME          , 0},
 { "MQCA_LAST"                        , MQCA_LAST                       , 0},
 { "MQCA_LAST_USED"                   , MQCA_LAST_USED                  , 0},
 { "MQCA_NAMES"                       , MQCA_NAMES                      , 0},
 { "MQCA_PROCESS_DESC"                , MQCA_PROCESS_DESC               , 0},
 { "MQCA_PROCESS_NAME"                , MQCA_PROCESS_NAME               , 0},
 { "MQCA_Q_DESC"                      , MQCA_Q_DESC                     , 0},
 { "MQCA_Q_MGR_DESC"                  , MQCA_Q_MGR_DESC                 , 0},
 { "MQCA_Q_MGR_NAME"                  , MQCA_Q_MGR_NAME                 , 0},
 { "MQCA_Q_NAME"                      , MQCA_Q_NAME                     , 0},
 { "MQCA_REMOTE_Q_MGR_NAME"           , MQCA_REMOTE_Q_MGR_NAME          , 0},
 { "MQCA_REMOTE_Q_NAME"               , MQCA_REMOTE_Q_NAME              , 0},
 { "MQCA_STORAGE_CLASS"               , MQCA_STORAGE_CLASS              , 0},
 { "MQCA_TRIGGER_DATA"                , MQCA_TRIGGER_DATA               , 0},
 { "MQCA_USER_DATA"                   , MQCA_USER_DATA                  , 0},
 { "MQCA_XMIT_Q_NAME"                 , MQCA_XMIT_Q_NAME                , 0},
 { "MQCACF_ALIAS_Q_NAMES"             , MQCACF_ALIAS_Q_NAMES            , 0},
 { "MQCACF_APPL_NAME"                 , MQCACF_APPL_NAME                , 0},
 { "MQCACF_AUX_ERROR_DATA_STR_1"      , MQCACF_AUX_ERROR_DATA_STR_1     , 0},
 { "MQCACF_AUX_ERROR_DATA_STR_2"      , MQCACF_AUX_ERROR_DATA_STR_2     , 0},
 { "MQCACF_AUX_ERROR_DATA_STR_3"      , MQCACF_AUX_ERROR_DATA_STR_3     , 0},
 { "MQCACF_ESCAPE_TEXT"               , MQCACF_ESCAPE_TEXT              , 0},
 { "MQCACF_FIRST"                     , MQCACF_FIRST                    , 0},
 { "MQCACF_FROM_CHANNEL_NAME"         , MQCACF_FROM_CHANNEL_NAME        , 0},
 { "MQCACF_FROM_PROCESS_NAME"         , MQCACF_FROM_PROCESS_NAME        , 0},
 { "MQCACF_FROM_Q_NAME"               , MQCACF_FROM_Q_NAME              , 0},
 { "MQCACF_LAST_USED"                 , MQCACF_LAST_USED                , 0},
 { "MQCACF_LOCAL_Q_NAMES"             , MQCACF_LOCAL_Q_NAMES            , 0},
 { "MQCACF_MODEL_Q_NAMES"             , MQCACF_MODEL_Q_NAMES            , 0},
 { "MQCACF_OBJECT_Q_MGR_NAME"         , MQCACF_OBJECT_Q_MGR_NAME        , 0},
 { "MQCACF_PROCESS_NAMES"             , MQCACF_PROCESS_NAMES            , 0},
 { "MQCACF_Q_NAMES"                   , MQCACF_Q_NAMES                  , 0},
 { "MQCACF_RECEIVER_CHANNEL_NAMES"    , MQCACF_RECEIVER_CHANNEL_NAMES   , 0},
 { "MQCACF_REMOTE_Q_NAMES"            , MQCACF_REMOTE_Q_NAMES           , 0},
 { "MQCACF_REQUESTER_CHANNEL_NAMES"   , MQCACF_REQUESTER_CHANNEL_NAMES  , 0},
 { "MQCACF_SENDER_CHANNEL_NAMES"      , MQCACF_SENDER_CHANNEL_NAMES     , 0},
 { "MQCACF_SERVER_CHANNEL_NAMES"      , MQCACF_SERVER_CHANNEL_NAMES     , 0},
 { "MQCACF_TO_CHANNEL_NAME"           , MQCACF_TO_CHANNEL_NAME          , 0},
 { "MQCACF_TO_PROCESS_NAME"           , MQCACF_TO_PROCESS_NAME          , 0},
 { "MQCACF_TO_Q_NAME"                 , MQCACF_TO_Q_NAME                , 0},
 { "MQCACF_USER_IDENTIFIER"           , MQCACF_USER_IDENTIFIER          , 0},
 { "MQCACH_CHANNEL_NAME"              , MQCACH_CHANNEL_NAME             , 0},
 { "MQCACH_CHANNEL_NAMES"             , MQCACH_CHANNEL_NAMES            , 0},
 { "MQCACH_CHANNEL_START_DATE"        , MQCACH_CHANNEL_START_DATE       , 0},
 { "MQCACH_CHANNEL_START_TIME"        , MQCACH_CHANNEL_START_TIME       , 0},
 { "MQCACH_CONNECTION_NAME"           , MQCACH_CONNECTION_NAME          , 0},
 { "MQCACH_CURRENT_LUWID"             , MQCACH_CURRENT_LUWID            , 0},
 { "MQCACH_DESC"                      , MQCACH_DESC                     , 0},
 { "MQCACH_FIRST"                     , MQCACH_FIRST                    , 0},
 { "MQCACH_FORMAT_NAME"               , MQCACH_FORMAT_NAME              , 0},
 { "MQCACH_LAST_LUWID"                , MQCACH_LAST_LUWID               , 0},
 { "MQCACH_LAST_MSG_DATE"             , MQCACH_LAST_MSG_DATE            , 0},
 { "MQCACH_LAST_MSG_TIME"             , MQCACH_LAST_MSG_TIME            , 0},
 { "MQCACH_LAST_USED"                 , MQCACH_LAST_USED                , 0},
 { "MQCACH_MCA_JOB_NAME"              , MQCACH_MCA_JOB_NAME             , 0},
 { "MQCACH_MCA_NAME"                  , MQCACH_MCA_NAME                 , 0},
 { "MQCACH_MCA_USER_ID"               , MQCACH_MCA_USER_ID              , 0},
 { "MQCACH_MODE_NAME"                 , MQCACH_MODE_NAME                , 0},
 { "MQCACH_MR_EXIT_NAME"              , MQCACH_MR_EXIT_NAME             , 0},
 { "MQCACH_MR_EXIT_USER_DATA"         , MQCACH_MR_EXIT_USER_DATA        , 0},
 { "MQCACH_MSG_EXIT_NAME"             , MQCACH_MSG_EXIT_NAME            , 0},
 { "MQCACH_MSG_EXIT_USER_DATA"        , MQCACH_MSG_EXIT_USER_DATA       , 0},
 { "MQCACH_PASSWORD"                  , MQCACH_PASSWORD                 , 0},
 { "MQCACH_RCV_EXIT_NAME"             , MQCACH_RCV_EXIT_NAME            , 0},
 { "MQCACH_RCV_EXIT_USER_DATA"        , MQCACH_RCV_EXIT_USER_DATA       , 0},
 { "MQCACH_SEC_EXIT_NAME"             , MQCACH_SEC_EXIT_NAME            , 0},
 { "MQCACH_SEC_EXIT_USER_DATA"        , MQCACH_SEC_EXIT_USER_DATA       , 0},
 { "MQCACH_SEND_EXIT_NAME"            , MQCACH_SEND_EXIT_NAME           , 0},
 { "MQCACH_SEND_EXIT_USER_DATA"       , MQCACH_SEND_EXIT_USER_DATA      , 0},
 { "MQCACH_TP_NAME"                   , MQCACH_TP_NAME                  , 0},
 { "MQCACH_USER_ID"                   , MQCACH_USER_ID                  , 0},
 { "MQCACH_XMIT_Q_NAME"               , MQCACH_XMIT_Q_NAME              , 0},
 { "MQCC_FAILED"                      , MQCC_FAILED                     , 0},
 { "MQCC_OK"                          , MQCC_OK                         , 0},
 { "MQCC_UNKNOWN"                     , MQCC_UNKNOWN                    , 0},
 { "MQCC_WARNING"                     , MQCC_WARNING                    , 0},
 { "MQCCSI_DEFAULT"                   , MQCCSI_DEFAULT                  , 0},
 { "MQCCSI_EMBEDDED"                  , MQCCSI_EMBEDDED                 , 0},
 { "MQCCSI_Q_MGR"                     , MQCCSI_Q_MGR                    , 0},
 { "MQCFC_LAST"                       , MQCFC_LAST                      , 0},
 { "MQCFC_NOT_LAST"                   , MQCFC_NOT_LAST                  , 0},
 { "MQCFH_STRUC_LENGTH"               , MQCFH_STRUC_LENGTH              , 0},
 { "MQCFH_VERSION_1"                  , MQCFH_VERSION_1                 , 0},
 { "MQCFIL_STRUC_LENGTH_FIXED"        , MQCFIL_STRUC_LENGTH_FIXED       , 0},
 { "MQCFIN_STRUC_LENGTH"              , MQCFIN_STRUC_LENGTH             , 0},
 { "MQCFSL_STRUC_LENGTH_FIXED"        , MQCFSL_STRUC_LENGTH_FIXED       , 0},
 { "MQCFST_STRUC_LENGTH_FIXED"        , MQCFST_STRUC_LENGTH_FIXED       , 0},
 { "MQCFT_COMMAND"                    , MQCFT_COMMAND                   , 0},
 { "MQCFT_EVENT"                      , MQCFT_EVENT                     , 0},
 { "MQCFT_INTEGER_LIST"               , MQCFT_INTEGER_LIST              , 0},
 { "MQCFT_INTEGER"                    , MQCFT_INTEGER                   , 0},
 { "MQCFT_RESPONSE"                   , MQCFT_RESPONSE                  , 0},
 { "MQCFT_STRING_LIST"                , MQCFT_STRING_LIST               , 0},
 { "MQCFT_STRING"                     , MQCFT_STRING                    , 0},
 { "MQCHIDS_INDOUBT"                  , MQCHIDS_INDOUBT                 , 0},
 { "MQCHIDS_NOT_INDOUBT"              , MQCHIDS_NOT_INDOUBT             , 0},
 { "MQCHS_BINDING"                    , MQCHS_BINDING                   , 0},
 { "MQCHS_PAUSED"                     , MQCHS_PAUSED                    , 0},
 { "MQCHS_REQUESTING"                 , MQCHS_REQUESTING                , 0},
 { "MQCHS_RETRYING"                   , MQCHS_RETRYING                  , 0},
 { "MQCHS_RUNNING"                    , MQCHS_RUNNING                   , 0},
 { "MQCHS_STARTING"                   , MQCHS_STARTING                  , 0},
 { "MQCHS_STOPPED"                    , MQCHS_STOPPED                   , 0},
 { "MQCHS_STOPPING"                   , MQCHS_STOPPING                  , 0},
 { "MQCHSR_STOP_NOT_REQUESTED"        , MQCHSR_STOP_NOT_REQUESTED       , 0},
 { "MQCHSR_STOP_REQUESTED"            , MQCHSR_STOP_REQUESTED           , 0},
 { "MQCHTAB_CLNTCONN"                 , MQCHTAB_CLNTCONN                , 0},
 { "MQCHTAB_Q_MGR"                    , MQCHTAB_Q_MGR                   , 0},
 { "MQCMD_CHANGE_CHANNEL"             , MQCMD_CHANGE_CHANNEL            , 0},
 { "MQCMD_CHANGE_PROCESS"             , MQCMD_CHANGE_PROCESS            , 0},
 { "MQCMD_CHANGE_Q_MGR"               , MQCMD_CHANGE_Q_MGR              , 0},
 { "MQCMD_CHANGE_Q"                   , MQCMD_CHANGE_Q                  , 0},
 { "MQCMD_CHANNEL_EVENT"              , MQCMD_CHANNEL_EVENT             , 0},
 { "MQCMD_CLEAR_Q"                    , MQCMD_CLEAR_Q                   , 0},
 { "MQCMD_COPY_CHANNEL"               , MQCMD_COPY_CHANNEL              , 0},
 { "MQCMD_COPY_PROCESS"               , MQCMD_COPY_PROCESS              , 0},
 { "MQCMD_COPY_Q"                     , MQCMD_COPY_Q                    , 0},
 { "MQCMD_CREATE_CHANNEL"             , MQCMD_CREATE_CHANNEL            , 0},
 { "MQCMD_CREATE_PROCESS"             , MQCMD_CREATE_PROCESS            , 0},
 { "MQCMD_CREATE_Q"                   , MQCMD_CREATE_Q                  , 0},
 { "MQCMD_DELETE_CHANNEL"             , MQCMD_DELETE_CHANNEL            , 0},
 { "MQCMD_DELETE_PROCESS"             , MQCMD_DELETE_PROCESS            , 0},
 { "MQCMD_DELETE_Q"                   , MQCMD_DELETE_Q                  , 0},
 { "MQCMD_ESCAPE"                     , MQCMD_ESCAPE                    , 0},
 { "MQCMD_INQUIRE_CHANNEL_NAMES"      , MQCMD_INQUIRE_CHANNEL_NAMES     , 0},
 { "MQCMD_INQUIRE_CHANNEL_STATUS"     , MQCMD_INQUIRE_CHANNEL_STATUS    , 0},
 { "MQCMD_INQUIRE_CHANNEL"            , MQCMD_INQUIRE_CHANNEL           , 0},
 { "MQCMD_INQUIRE_PROCESS_NAMES"      , MQCMD_INQUIRE_PROCESS_NAMES     , 0},
 { "MQCMD_INQUIRE_PROCESS"            , MQCMD_INQUIRE_PROCESS           , 0},
 { "MQCMD_INQUIRE_Q_MGR"              , MQCMD_INQUIRE_Q_MGR             , 0},
 { "MQCMD_INQUIRE_Q_NAMES"            , MQCMD_INQUIRE_Q_NAMES           , 0},
 { "MQCMD_INQUIRE_Q"                  , MQCMD_INQUIRE_Q                 , 0},
 { "MQCMD_PERFM_EVENT"                , MQCMD_PERFM_EVENT               , 0},
 { "MQCMD_PING_CHANNEL"               , MQCMD_PING_CHANNEL              , 0},
 { "MQCMD_PING_Q_MGR"                 , MQCMD_PING_Q_MGR                , 0},
 { "MQCMD_Q_MGR_EVENT"                , MQCMD_Q_MGR_EVENT               , 0},
 { "MQCMD_RESET_CHANNEL"              , MQCMD_RESET_CHANNEL             , 0},
 { "MQCMD_RESET_Q_STATS"              , MQCMD_RESET_Q_STATS             , 0},
 { "MQCMD_RESOLVE_CHANNEL"            , MQCMD_RESOLVE_CHANNEL           , 0},
 { "MQCMD_START_CHANNEL_INIT"         , MQCMD_START_CHANNEL_INIT        , 0},
 { "MQCMD_START_CHANNEL_LISTENER"     , MQCMD_START_CHANNEL_LISTENER    , 0},
 { "MQCMD_START_CHANNEL"              , MQCMD_START_CHANNEL             , 0},
 { "MQCMD_STOP_CHANNEL"               , MQCMD_STOP_CHANNEL              , 0},
 { "MQCMDL_LEVEL_1"                   , MQCMDL_LEVEL_1                  , 0},
 { "MQCMDL_LEVEL_114"                 , MQCMDL_LEVEL_114                , 0},
 { "MQCMDL_LEVEL_201"                 , MQCMDL_LEVEL_201                , 0},
 { "MQCMDL_LEVEL_221"                 , MQCMDL_LEVEL_221                , 0},
 { "MQCMDL_LEVEL_320"                 , MQCMDL_LEVEL_320                , 0},
 { "MQCO_DELETE_PURGE"                , MQCO_DELETE_PURGE               , 0},
 { "MQCO_DELETE"                      , MQCO_DELETE                     , 0},
 { "MQCO_NONE"                        , MQCO_NONE                       , 0},
 { "MQDLH_VERSION_1"                  , MQDLH_VERSION_1                 , 0},
 { "MQEI_UNLIMITED"                   , MQEI_UNLIMITED                  , 0},
 { "MQENC_DECIMAL_MASK"               , MQENC_DECIMAL_MASK              , 0},
 { "MQENC_DECIMAL_NORMAL"             , MQENC_DECIMAL_NORMAL            , 0},
 { "MQENC_DECIMAL_REVERSED"           , MQENC_DECIMAL_REVERSED          , 0},
 { "MQENC_DECIMAL_UNDEFINED"          , MQENC_DECIMAL_UNDEFINED         , 0},
 { "MQENC_FLOAT_IEEE_NORMAL"          , MQENC_FLOAT_IEEE_NORMAL         , 0},
 { "MQENC_FLOAT_IEEE_REVERSED"        , MQENC_FLOAT_IEEE_REVERSED       , 0},
 { "MQENC_FLOAT_MASK"                 , MQENC_FLOAT_MASK                , 0},
 { "MQENC_FLOAT_S390"                 , MQENC_FLOAT_S390                , 0},
 { "MQENC_FLOAT_UNDEFINED"            , MQENC_FLOAT_UNDEFINED           , 0},
 { "MQENC_INTEGER_MASK"               , MQENC_INTEGER_MASK              , 0},
 { "MQENC_INTEGER_NORMAL"             , MQENC_INTEGER_NORMAL            , 0},
 { "MQENC_INTEGER_REVERSED"           , MQENC_INTEGER_REVERSED          , 0},
 { "MQENC_INTEGER_UNDEFINED"          , MQENC_INTEGER_UNDEFINED         , 0},
 { "MQENC_NATIVE"                     , MQENC_NATIVE                    , 0},
 { "MQENC_RESERVED_MASK"              , MQENC_RESERVED_MASK             , 0},
 { "MQET_MQSC"                        , MQET_MQSC                       , 0},
 { "MQEVR_DISABLED"                   , MQEVR_DISABLED                  , 0},
 { "MQEVR_ENABLED"                    , MQEVR_ENABLED                   , 0},
 { "MQFB_APPL_CANNOT_BE_STARTED"      , MQFB_APPL_CANNOT_BE_STARTED     , 0},
 { "MQFB_APPL_FIRST"                  , MQFB_APPL_FIRST                 , 0},
 { "MQFB_APPL_LAST"                   , MQFB_APPL_LAST                  , 0},
 { "MQFB_APPL_TYPE_ERROR"             , MQFB_APPL_TYPE_ERROR            , 0},
 { "MQFB_CHANNEL_COMPLETED"           , MQFB_CHANNEL_COMPLETED          , 0},
 { "MQFB_CHANNEL_FAIL_RETRY"          , MQFB_CHANNEL_FAIL_RETRY         , 0},
 { "MQFB_CHANNEL_FAIL"                , MQFB_CHANNEL_FAIL               , 0},
 { "MQFB_COA"                         , MQFB_COA                        , 0},
 { "MQFB_COD"                         , MQFB_COD                        , 0},
 { "MQFB_EXPIRATION"                  , MQFB_EXPIRATION                 , 0},
 { "MQFB_NONE"                        , MQFB_NONE                       , 0},
 { "MQFB_QUIT"                        , MQFB_QUIT                       , 0},
 { "MQFB_STOPPED_BY_MSG_EXIT"         , MQFB_STOPPED_BY_MSG_EXIT        , 0},
 { "MQFB_SYSTEM_FIRST"                , MQFB_SYSTEM_FIRST               , 0},
 { "MQFB_SYSTEM_LAST"                 , MQFB_SYSTEM_LAST                , 0},
 { "MQFB_TM_ERROR"                    , MQFB_TM_ERROR                   , 0},
 { "MQFB_XMIT_Q_MSG_ERROR"            , MQFB_XMIT_Q_MSG_ERROR           , 0},
 { "MQFC_NO"                          , MQFC_NO                         , 0},
 { "MQFC_YES"                         , MQFC_YES                        , 0},
 { "MQGMO_ACCEPT_TRUNCATED_MSG"       , MQGMO_ACCEPT_TRUNCATED_MSG      , 0},
 { "MQGMO_BROWSE_FIRST"               , MQGMO_BROWSE_FIRST              , 0},
 { "MQGMO_BROWSE_MSG_UNDER_CURSOR"    , MQGMO_BROWSE_MSG_UNDER_CURSOR   , 0},
 { "MQGMO_BROWSE_NEXT"                , MQGMO_BROWSE_NEXT               , 0},
 { "MQGMO_CONVERT"                    , MQGMO_CONVERT                   , 0},
 { "MQGMO_FAIL_IF_QUIESCING"          , MQGMO_FAIL_IF_QUIESCING         , 0},
 { "MQGMO_LOCK"                       , MQGMO_LOCK                      , 0},
 { "MQGMO_MSG_UNDER_CURSOR"           , MQGMO_MSG_UNDER_CURSOR          , 0},
 { "MQGMO_NO_SYNCPOINT"               , MQGMO_NO_SYNCPOINT              , 0},
 { "MQGMO_NO_WAIT"                    , MQGMO_NO_WAIT                   , 0},
 { "MQGMO_NONE"                       , MQGMO_NONE                      , 0},
 { "MQGMO_SYNCPOINT"                  , MQGMO_SYNCPOINT                 , 0},
 { "MQGMO_UNLOCK"                     , MQGMO_UNLOCK                    , 0},
 { "MQGMO_VERSION_1"                  , MQGMO_VERSION_1                 , 0},
 { "MQGMO_WAIT"                       , MQGMO_WAIT                      , 0},
 { "MQHC_UNUSABLE_HCONN"              , MQHC_UNUSABLE_HCONN             , 0},
 { "MQHO_UNUSABLE_HOBJ"               , MQHO_UNUSABLE_HOBJ              , 0},
 { "MQIA_APPL_TYPE"                   , MQIA_APPL_TYPE                  , 0},
 { "MQIA_AUTHORITY_EVENT"             , MQIA_AUTHORITY_EVENT            , 0},
 { "MQIA_BACKOUT_THRESHOLD"           , MQIA_BACKOUT_THRESHOLD          , 0},
 { "MQIA_CODED_CHAR_SET_ID"           , MQIA_CODED_CHAR_SET_ID          , 0},
 { "MQIA_COMMAND_LEVEL"               , MQIA_COMMAND_LEVEL              , 0},
 { "MQIA_CURRENT_Q_DEPTH"             , MQIA_CURRENT_Q_DEPTH            , 0},
 { "MQIA_DEF_INPUT_OPEN_OPTION"       , MQIA_DEF_INPUT_OPEN_OPTION      , 0},
 { "MQIA_DEF_PERSISTENCE"             , MQIA_DEF_PERSISTENCE            , 0},
 { "MQIA_DEF_PRIORITY"                , MQIA_DEF_PRIORITY               , 0},
 { "MQIA_DEFINITION_TYPE"             , MQIA_DEFINITION_TYPE            , 0},
 { "MQIA_FIRST"                       , MQIA_FIRST                      , 0},
 { "MQIA_HARDEN_GET_BACKOUT"          , MQIA_HARDEN_GET_BACKOUT         , 0},
 { "MQIA_HIGH_Q_DEPTH"                , MQIA_HIGH_Q_DEPTH               , 0},
 { "MQIA_INHIBIT_EVENT"               , MQIA_INHIBIT_EVENT              , 0},
 { "MQIA_INHIBIT_GET"                 , MQIA_INHIBIT_GET                , 0},
 { "MQIA_INHIBIT_PUT"                 , MQIA_INHIBIT_PUT                , 0},
 { "MQIA_LAST"                        , MQIA_LAST                       , 0},
 { "MQIA_LAST_USED"                   , MQIA_LAST_USED                  , 0},
 { "MQIA_LOCAL_EVENT"                 , MQIA_LOCAL_EVENT                , 0},
 { "MQIA_MAX_HANDLES"                 , MQIA_MAX_HANDLES                , 0},
 { "MQIA_MAX_MSG_LENGTH"              , MQIA_MAX_MSG_LENGTH             , 0},
 { "MQIA_MAX_PRIORITY"                , MQIA_MAX_PRIORITY               , 0},
 { "MQIA_MAX_Q_DEPTH"                 , MQIA_MAX_Q_DEPTH                , 0},
 { "MQIA_MAX_UNCOMMITTED_MSGS"        , MQIA_MAX_UNCOMMITTED_MSGS       , 0},
 { "MQIA_MSG_DELIVERY_SEQUENCE"       , MQIA_MSG_DELIVERY_SEQUENCE      , 0},
 { "MQIA_MSG_DEQ_COUNT"               , MQIA_MSG_DEQ_COUNT              , 0},
 { "MQIA_MSG_ENQ_COUNT"               , MQIA_MSG_ENQ_COUNT              , 0},
 { "MQIA_NAME_COUNT"                  , MQIA_NAME_COUNT                 , 0},
 { "MQIA_OPEN_INPUT_COUNT"            , MQIA_OPEN_INPUT_COUNT           , 0},
 { "MQIA_OPEN_OUTPUT_COUNT"           , MQIA_OPEN_OUTPUT_COUNT          , 0},
 { "MQIA_PERFORMANCE_EVENT"           , MQIA_PERFORMANCE_EVENT          , 0},
 { "MQIA_PLATFORM"                    , MQIA_PLATFORM                   , 0},
 { "MQIA_Q_DEPTH_HIGH_EVENT"          , MQIA_Q_DEPTH_HIGH_EVENT         , 0},
 { "MQIA_Q_DEPTH_HIGH_LIMIT"          , MQIA_Q_DEPTH_HIGH_LIMIT         , 0},
 { "MQIA_Q_DEPTH_LOW_EVENT"           , MQIA_Q_DEPTH_LOW_EVENT          , 0},
 { "MQIA_Q_DEPTH_LOW_LIMIT"           , MQIA_Q_DEPTH_LOW_LIMIT          , 0},
 { "MQIA_Q_DEPTH_MAX_EVENT"           , MQIA_Q_DEPTH_MAX_EVENT          , 0},
 { "MQIA_Q_SERVICE_INTERVAL_EVENT"    , MQIA_Q_SERVICE_INTERVAL_EVENT   , 0},
 { "MQIA_Q_SERVICE_INTERVAL"          , MQIA_Q_SERVICE_INTERVAL         , 0},
 { "MQIA_Q_TYPE"                      , MQIA_Q_TYPE                     , 0},
 { "MQIA_REMOTE_EVENT"                , MQIA_REMOTE_EVENT               , 0},
 { "MQIA_RETENTION_INTERVAL"          , MQIA_RETENTION_INTERVAL         , 0},
 { "MQIA_SCOPE"                       , MQIA_SCOPE                      , 0},
 { "MQIA_SHAREABILITY"                , MQIA_SHAREABILITY               , 0},
 { "MQIA_START_STOP_EVENT"            , MQIA_START_STOP_EVENT           , 0},
 { "MQIA_SYNCPOINT"                   , MQIA_SYNCPOINT                  , 0},
 { "MQIA_TIME_SINCE_RESET"            , MQIA_TIME_SINCE_RESET           , 0},
 { "MQIA_TRIGGER_CONTROL"             , MQIA_TRIGGER_CONTROL            , 0},
 { "MQIA_TRIGGER_DEPTH"               , MQIA_TRIGGER_DEPTH              , 0},
 { "MQIA_TRIGGER_INTERVAL"            , MQIA_TRIGGER_INTERVAL           , 0},
 { "MQIA_TRIGGER_MSG_PRIORITY"        , MQIA_TRIGGER_MSG_PRIORITY       , 0},
 { "MQIA_TRIGGER_TYPE"                , MQIA_TRIGGER_TYPE               , 0},
 { "MQIA_USAGE"                       , MQIA_USAGE                      , 0},
 { "MQIACF_ALL"                       , MQIACF_ALL                      , 0},
 { "MQIACF_AUX_ERROR_DATA_INT_1"      , MQIACF_AUX_ERROR_DATA_INT_1     , 0},
 { "MQIACF_AUX_ERROR_DATA_INT_2"      , MQIACF_AUX_ERROR_DATA_INT_2     , 0},
 { "MQIACF_CHANNEL_ATTRS"             , MQIACF_CHANNEL_ATTRS            , 0},
 { "MQIACF_COMMAND"                   , MQIACF_COMMAND                  , 0},
 { "MQIACF_CONV_REASON_CODE"          , MQIACF_CONV_REASON_CODE         , 0},
 { "MQIACF_ERROR_ID"                  , MQIACF_ERROR_ID                 , 0},
 { "MQIACF_ERROR_IDENTIFIER"          , MQIACF_ERROR_IDENTIFIER         , 0},
 { "MQIACF_ERROR_OFFSET"              , MQIACF_ERROR_OFFSET             , 0},
 { "MQIACF_ESCAPE_TYPE"               , MQIACF_ESCAPE_TYPE              , 0},
 { "MQIACF_FIRST"                     , MQIACF_FIRST                    , 0},
 { "MQIACF_FORCE"                     , MQIACF_FORCE                    , 0},
 { "MQIACF_LAST_USED"                 , MQIACF_LAST_USED                , 0},
 { "MQIACF_OPEN_OPTIONS"              , MQIACF_OPEN_OPTIONS             , 0},
 { "MQIACF_PARAMETER_ID"              , MQIACF_PARAMETER_ID             , 0},
 { "MQIACF_PROCESS_ATTRS"             , MQIACF_PROCESS_ATTRS            , 0},
 { "MQIACF_PURGE"                     , MQIACF_PURGE                    , 0},
 { "MQIACF_Q_ATTRS"                   , MQIACF_Q_ATTRS                  , 0},
 { "MQIACF_Q_MGR_ATTRS"               , MQIACF_Q_MGR_ATTRS              , 0},
 { "MQIACF_QUIESCE"                   , MQIACF_QUIESCE                  , 0},
 { "MQIACF_REASON_QUALIFIER"          , MQIACF_REASON_QUALIFIER         , 0},
 { "MQIACF_REPLACE"                   , MQIACF_REPLACE                  , 0},
 { "MQIACF_SELECTOR"                  , MQIACF_SELECTOR                 , 0},
 { "MQIACH_BATCH_SIZE"                , MQIACH_BATCH_SIZE               , 0},
 { "MQIACH_BATCHES"                   , MQIACH_BATCHES                  , 0},
 { "MQIACH_BUFFERS_RECEIVED"          , MQIACH_BUFFERS_RECEIVED         , 0},
 { "MQIACH_BUFFERS_SENT"              , MQIACH_BUFFERS_SENT             , 0},
 { "MQIACH_BYTES_RECEIVED"            , MQIACH_BYTES_RECEIVED           , 0},
 { "MQIACH_BYTES_SENT"                , MQIACH_BYTES_SENT               , 0},
 { "MQIACH_CHANNEL_ERROR_DATA"        , MQIACH_CHANNEL_ERROR_DATA       , 0},
 { "MQIACH_CHANNEL_INSTANCE_ATTRS"    , MQIACH_CHANNEL_INSTANCE_ATTRS   , 0},
 { "MQIACH_CHANNEL_INSTANCE_TYPE"     , MQIACH_CHANNEL_INSTANCE_TYPE    , 0},
 { "MQIACH_CHANNEL_STATUS"            , MQIACH_CHANNEL_STATUS           , 0},
 { "MQIACH_CHANNEL_TABLE"             , MQIACH_CHANNEL_TABLE            , 0},
 { "MQIACH_CHANNEL_TYPE"              , MQIACH_CHANNEL_TYPE             , 0},
 { "MQIACH_CURRENT_MSGS"              , MQIACH_CURRENT_MSGS             , 0},
 { "MQIACH_CURRENT_SEQUENCE_NUMBER"   , MQIACH_CURRENT_SEQUENCE_NUMBER  , 0},
 { "MQIACH_DATA_CONVERSION"           , MQIACH_DATA_CONVERSION          , 0},
 { "MQIACH_DATA_COUNT"                , MQIACH_DATA_COUNT               , 0},
 { "MQIACH_DISC_INTERVAL"             , MQIACH_DISC_INTERVAL            , 0},
 { "MQIACH_FIRST"                     , MQIACH_FIRST                    , 0},
 { "MQIACH_IN_DOUBT"                  , MQIACH_IN_DOUBT                 , 0},
 { "MQIACH_INDOUBT_STATUS"            , MQIACH_INDOUBT_STATUS           , 0},
 { "MQIACH_LAST_SEQUENCE_NUMBER"      , MQIACH_LAST_SEQUENCE_NUMBER     , 0},
 { "MQIACH_LAST_USED"                 , MQIACH_LAST_USED                , 0},
 { "MQIACH_LONG_RETRIES_LEFT"         , MQIACH_LONG_RETRIES_LEFT        , 0},
 { "MQIACH_LONG_RETRY"                , MQIACH_LONG_RETRY               , 0},
 { "MQIACH_LONG_TIMER"                , MQIACH_LONG_TIMER               , 0},
 { "MQIACH_MAX_MSG_LENGTH"            , MQIACH_MAX_MSG_LENGTH           , 0},
 { "MQIACH_MCA_STATUS"                , MQIACH_MCA_STATUS               , 0},
 { "MQIACH_MCA_TYPE"                  , MQIACH_MCA_TYPE                 , 0},
 { "MQIACH_MR_COUNT"                  , MQIACH_MR_COUNT                 , 0},
 { "MQIACH_MR_INTERVAL"               , MQIACH_MR_INTERVAL              , 0},
 { "MQIACH_MSG_SEQUENCE_NUMBER"       , MQIACH_MSG_SEQUENCE_NUMBER      , 0},
 { "MQIACH_MSGS"                      , MQIACH_MSGS                     , 0},
 { "MQIACH_PUT_AUTHORITY"             , MQIACH_PUT_AUTHORITY            , 0},
 { "MQIACH_SEQUENCE_NUMBER_WRAP"      , MQIACH_SEQUENCE_NUMBER_WRAP     , 0},
 { "MQIACH_SHORT_RETRIES_LEFT"        , MQIACH_SHORT_RETRIES_LEFT       , 0},
 { "MQIACH_SHORT_RETRY"               , MQIACH_SHORT_RETRY              , 0},
 { "MQIACH_SHORT_TIMER"               , MQIACH_SHORT_TIMER              , 0},
 { "MQIACH_STOP_REQUESTED"            , MQIACH_STOP_REQUESTED           , 0},
 { "MQIACH_XMIT_PROTOCOL_TYPE"        , MQIACH_XMIT_PROTOCOL_TYPE       , 0},
 { "MQIAV_NOT_APPLICABLE"             , MQIAV_NOT_APPLICABLE            , 0},
 { "MQIDO_BACKOUT"                    , MQIDO_BACKOUT                   , 0},
 { "MQIDO_COMMIT"                     , MQIDO_COMMIT                    , 0},
 { "MQMCAS_RUNNING"                   , MQMCAS_RUNNING                  , 0},
 { "MQMCAS_STOPPED"                   , MQMCAS_STOPPED                  , 0},
 { "MQMD_VERSION_1"                   , MQMD_VERSION_1                  , 0},
 { "MQMDS_FIFO"                       , MQMDS_FIFO                      , 0},
 { "MQMDS_PRIORITY"                   , MQMDS_PRIORITY                  , 0},
 { "MQMT_APPL_FIRST"                  , MQMT_APPL_FIRST                 , 0},
 { "MQMT_APPL_LAST"                   , MQMT_APPL_LAST                  , 0},
 { "MQMT_DATAGRAM"                    , MQMT_DATAGRAM                   , 0},
 { "MQMT_REPLY"                       , MQMT_REPLY                      , 0},
 { "MQMT_REPORT"                      , MQMT_REPORT                     , 0},
 { "MQMT_REQUEST"                     , MQMT_REQUEST                    , 0},
 { "MQMT_SYSTEM_FIRST"                , MQMT_SYSTEM_FIRST               , 0},
 { "MQMT_SYSTEM_LAST"                 , MQMT_SYSTEM_LAST                , 0},
 { "MQOD_VERSION_1"                   , MQOD_VERSION_1                  , 0},
 { "MQOO_ALTERNATE_USER_AUTHORITY"    , MQOO_ALTERNATE_USER_AUTHORITY   , 0},
 { "MQOO_BROWSE"                      , MQOO_BROWSE                     , 0},
 { "MQOO_FAIL_IF_QUIESCING"           , MQOO_FAIL_IF_QUIESCING          , 0},
 { "MQOO_INPUT_AS_Q_DEF"              , MQOO_INPUT_AS_Q_DEF             , 0},
 { "MQOO_INPUT_EXCLUSIVE"             , MQOO_INPUT_EXCLUSIVE            , 0},
 { "MQOO_INPUT_SHARED"                , MQOO_INPUT_SHARED               , 0},
 { "MQOO_INQUIRE"                     , MQOO_INQUIRE                    , 0},
 { "MQOO_OUTPUT"                      , MQOO_OUTPUT                     , 0},
 { "MQOO_PASS_ALL_CONTEXT"            , MQOO_PASS_ALL_CONTEXT           , 0},
 { "MQOO_PASS_IDENTITY_CONTEXT"       , MQOO_PASS_IDENTITY_CONTEXT      , 0},
 { "MQOO_SAVE_ALL_CONTEXT"            , MQOO_SAVE_ALL_CONTEXT           , 0},
 { "MQOO_SET"                         , MQOO_SET                        , 0},
 { "MQOO_SET_ALL_CONTEXT"             , MQOO_SET_ALL_CONTEXT            , 0},
 { "MQOO_SET_IDENTITY_CONTEXT"        , MQOO_SET_IDENTITY_CONTEXT       , 0},
 { "MQOT_ALIAS_Q"                     , MQOT_ALIAS_Q                    , 0},
 { "MQOT_ALL"                         , MQOT_ALL                        , 0},
 { "MQOT_CHANNEL"                     , MQOT_CHANNEL                    , 0},
 { "MQOT_CURRENT_CHANNEL"             , MQOT_CURRENT_CHANNEL            , 0},
 { "MQOT_LOCAL_Q"                     , MQOT_LOCAL_Q                    , 0},
 { "MQOT_MODEL_Q"                     , MQOT_MODEL_Q                    , 0},
 { "MQOT_PROCESS"                     , MQOT_PROCESS                    , 0},
 { "MQOT_Q"                           , MQOT_Q                          , 0},
 { "MQOT_Q_MGR"                       , MQOT_Q_MGR                      , 0},
 { "MQOT_RECEIVER_CHANNEL"            , MQOT_RECEIVER_CHANNEL           , 0},
 { "MQOT_REMOTE_Q"                    , MQOT_REMOTE_Q                   , 0},
 { "MQOT_REQUESTER_CHANNEL"           , MQOT_REQUESTER_CHANNEL          , 0},
 { "MQOT_SAVED_CHANNEL"               , MQOT_SAVED_CHANNEL              , 0},
 { "MQOT_SENDER_CHANNEL"              , MQOT_SENDER_CHANNEL             , 0},
 { "MQOT_SERVER_CHANNEL"              , MQOT_SERVER_CHANNEL             , 0},
 { "MQPER_NOT_PERSISTENT"             , MQPER_NOT_PERSISTENT            , 0},
 { "MQPER_PERSISTENCE_AS_Q_DEF"       , MQPER_PERSISTENCE_AS_Q_DEF      , 0},
 { "MQPER_PERSISTENT"                 , MQPER_PERSISTENT                , 0},
 { "MQPL_AIX"                         , MQPL_AIX                        , 0},
 { "MQPL_MVS"                         , MQPL_MVS                        , 0},
 { "MQPL_OS2"                         , MQPL_OS2                        , 0},
 { "MQPL_OS400"                       , MQPL_OS400                      , 0},
 { "MQPL_UNIX"                        , MQPL_UNIX                       , 0},
 { "MQPMO_ALTERNATE_USER_AUTHORITY"   , MQPMO_ALTERNATE_USER_AUTHORITY  , 0},
 { "MQPMO_DEFAULT_CONTEXT"            , MQPMO_DEFAULT_CONTEXT           , 0},
 { "MQPMO_FAIL_IF_QUIESCING"          , MQPMO_FAIL_IF_QUIESCING         , 0},
 { "MQPMO_NO_CONTEXT"                 , MQPMO_NO_CONTEXT                , 0},
 { "MQPMO_NO_SYNCPOINT"               , MQPMO_NO_SYNCPOINT              , 0},
 { "MQPMO_NONE"                       , MQPMO_NONE                      , 0},
 { "MQPMO_PASS_ALL_CONTEXT"           , MQPMO_PASS_ALL_CONTEXT          , 0},
 { "MQPMO_PASS_IDENTITY_CONTEXT"      , MQPMO_PASS_IDENTITY_CONTEXT     , 0},
 { "MQPMO_SET_ALL_CONTEXT"            , MQPMO_SET_ALL_CONTEXT           , 0},
 { "MQPMO_SET_IDENTITY_CONTEXT"       , MQPMO_SET_IDENTITY_CONTEXT      , 0},
 { "MQPMO_SYNCPOINT"                  , MQPMO_SYNCPOINT                 , 0},
 { "MQPMO_VERSION_1"                  , MQPMO_VERSION_1                 , 0},
 { "MQPO_NO"                          , MQPO_NO                         , 0},
 { "MQPO_YES"                         , MQPO_YES                        , 0},
 { "MQPRI_PRIORITY_AS_Q_DEF"          , MQPRI_PRIORITY_AS_Q_DEF         , 0},
 { "MQQA_BACKOUT_HARDENED"            , MQQA_BACKOUT_HARDENED           , 0},
 { "MQQA_BACKOUT_NOT_HARDENED"        , MQQA_BACKOUT_NOT_HARDENED       , 0},
 { "MQQA_GET_ALLOWED"                 , MQQA_GET_ALLOWED                , 0},
 { "MQQA_GET_INHIBITED"               , MQQA_GET_INHIBITED              , 0},
 { "MQQA_NOT_SHAREABLE"               , MQQA_NOT_SHAREABLE              , 0},
 { "MQQA_PUT_ALLOWED"                 , MQQA_PUT_ALLOWED                , 0},
 { "MQQA_PUT_INHIBITED"               , MQQA_PUT_INHIBITED              , 0},
 { "MQQA_SHAREABLE"                   , MQQA_SHAREABLE                  , 0},
 { "MQQDT_PERMANENT_DYNAMIC"          , MQQDT_PERMANENT_DYNAMIC         , 0},
 { "MQQDT_PREDEFINED"                 , MQQDT_PREDEFINED                , 0},
 { "MQQDT_TEMPORARY_DYNAMIC"          , MQQDT_TEMPORARY_DYNAMIC         , 0},
 { "MQQO_NO"                          , MQQO_NO                         , 0},
 { "MQQO_YES"                         , MQQO_YES                        , 0},
 { "MQQSIE_HIGH"                      , MQQSIE_HIGH                     , 0},
 { "MQQSIE_NONE"                      , MQQSIE_NONE                     , 0},
 { "MQQSIE_OK"                        , MQQSIE_OK                       , 0},
 { "MQQT_ALIAS"                       , MQQT_ALIAS                      , 0},
 { "MQQT_ALL"                         , MQQT_ALL                        , 0},
 { "MQQT_LOCAL"                       , MQQT_LOCAL                      , 0},
 { "MQQT_MODEL"                       , MQQT_MODEL                      , 0},
 { "MQQT_REMOTE"                      , MQQT_REMOTE                     , 0},
 { "MQRO_ACCEPT_UNSUP_IF_XMIT_MASK"   , MQRO_ACCEPT_UNSUP_IF_XMIT_MASK  , 0},
 { "MQRO_ACCEPT_UNSUP_MASK"           , MQRO_ACCEPT_UNSUP_MASK          , 0},
 { "MQRO_COA"                         , MQRO_COA                        , 0},
 { "MQRO_COA_WITH_DATA"               , MQRO_COA_WITH_DATA              , 0},
 { "MQRO_COA_WITH_FULL_DATA"          , MQRO_COA_WITH_FULL_DATA         , 0},
 { "MQRO_COD"                         , MQRO_COD                        , 0},
 { "MQRO_COD_WITH_DATA"               , MQRO_COD_WITH_DATA              , 0},
 { "MQRO_COD_WITH_FULL_DATA"          , MQRO_COD_WITH_FULL_DATA         , 0},
 { "MQRO_COPY_MSG_ID_TO_CORREL_ID"    , MQRO_COPY_MSG_ID_TO_CORREL_ID   , 0},
 { "MQRO_DEAD_LETTER_Q"               , MQRO_DEAD_LETTER_Q              , 0},
 { "MQRO_DISCARD_MSG"                 , MQRO_DISCARD_MSG                , 0},
 { "MQRO_EXCEPTION_WITH_DATA"         , MQRO_EXCEPTION_WITH_DATA        , 0},
 { "MQRO_EXCEPTION_WITH_FULL_DATA"    , MQRO_EXCEPTION_WITH_FULL_DATA   , 0},
 { "MQRO_EXCEPTION"                   , MQRO_EXCEPTION                  , 0},
 { "MQRO_EXPIRATION_WITH_DATA"        , MQRO_EXPIRATION_WITH_DATA       , 0},
 { "MQRO_EXPIRATION_WITH_FULL_DATA"   , MQRO_EXPIRATION_WITH_FULL_DATA  , 0},
 { "MQRO_EXPIRATION"                  , MQRO_EXPIRATION                 , 0},
 { "MQRO_NEW_MSG_ID"                  , MQRO_NEW_MSG_ID                 , 0},
 { "MQRO_NONE"                        , MQRO_NONE                       , 0},
 { "MQRO_PASS_CORREL_ID"              , MQRO_PASS_CORREL_ID             , 0},
 { "MQRO_PASS_MSG_ID"                 , MQRO_PASS_MSG_ID                , 0},
 { "MQRO_REJECT_UNSUP_MASK"           , MQRO_REJECT_UNSUP_MASK          , 0},
 { "MQRP_NO"                          , MQRP_NO                         , 0},
 { "MQRP_YES"                         , MQRP_YES                        , 0},
 { "MQRQ_CHANNEL_STOPPED_DISABLED"    , MQRQ_CHANNEL_STOPPED_DISABLED   , 0},
 { "MQRQ_CHANNEL_STOPPED_ERROR"       , MQRQ_CHANNEL_STOPPED_ERROR      , 0},
 { "MQRQ_CHANNEL_STOPPED_OK"          , MQRQ_CHANNEL_STOPPED_OK         , 0},
 { "MQRQ_CHANNEL_STOPPED_RETRY"       , MQRQ_CHANNEL_STOPPED_RETRY      , 0},
 { "MQRQ_CLOSE_NOT_AUTHORIZED"        , MQRQ_CLOSE_NOT_AUTHORIZED       , 0},
 { "MQRQ_CMD_NOT_AUTHORIZED"          , MQRQ_CMD_NOT_AUTHORIZED         , 0},
 { "MQRQ_CONN_NOT_AUTHORIZED"         , MQRQ_CONN_NOT_AUTHORIZED        , 0},
 { "MQRQ_OPEN_NOT_AUTHORIZED"         , MQRQ_OPEN_NOT_AUTHORIZED        , 0},
 { "MQRQ_Q_MGR_QUIESCING"             , MQRQ_Q_MGR_QUIESCING            , 0},
 { "MQRQ_Q_MGR_STOPPING"              , MQRQ_Q_MGR_STOPPING             , 0},
 { "MQSCO_CELL"                       , MQSCO_CELL                      , 0},
 { "MQSCO_Q_MGR"                      , MQSCO_Q_MGR                     , 0},
 { "MQSP_AVAILABLE"                   , MQSP_AVAILABLE                  , 0},
 { "MQSP_NOT_AVAILABLE"               , MQSP_NOT_AVAILABLE              , 0},
 { "MQTC_OFF"                         , MQTC_OFF                        , 0},
 { "MQTC_ON"                          , MQTC_ON                         , 0},
 { "MQTM_VERSION_1"                   , MQTM_VERSION_1                  , 0},
 { "MQTT_DEPTH"                       , MQTT_DEPTH                      , 0},
 { "MQTT_EVERY"                       , MQTT_EVERY                      , 0},
 { "MQTT_FIRST"                       , MQTT_FIRST                      , 0},
 { "MQTT_NONE"                        , MQTT_NONE                       , 0},
 { "MQUS_NORMAL"                      , MQUS_NORMAL                     , 0},
 { "MQUS_TRANSMISSION"                , MQUS_TRANSMISSION               , 0},
 { "MQWI_UNLIMITED"                   , MQWI_UNLIMITED                  , 0},
 { "MQXQH_VERSION_1"                  , MQXQH_VERSION_1                 , 0},

   // MQ Rexx Return Codes
 { "MQRC_OK"                          , MQCC_OK                         , 1},
 { "MQRC_WARNING"                     , MQCC_WARNING                    , 1},
 { "MQRC_UNKNOWN"                     , MQCC_UNKNOWN                    , 1},
 { "MQRC_FAILED"                      , MQCC_FAILED                     , 1},
 { "MQRC_ADAPTER_CONN_LOAD_ERROR"     , MQRC_ADAPTER_CONN_LOAD_ERROR    , 1},
 { "MQRC_ADAPTER_DEFS_ERROR"          , MQRC_ADAPTER_DEFS_ERROR         , 1},
 { "MQRC_ADAPTER_DEFS_LOAD_ERROR"     , MQRC_ADAPTER_DEFS_LOAD_ERROR    , 1},
 { "MQRC_ADAPTER_DISC_LOAD_ERROR"     , MQRC_ADAPTER_DISC_LOAD_ERROR    , 1},
 { "MQRC_ADAPTER_NOT_AVAILABLE"       , MQRC_ADAPTER_NOT_AVAILABLE      , 1},
 { "MQRC_ADAPTER_SERV_LOAD_ERROR"     , MQRC_ADAPTER_SERV_LOAD_ERROR    , 1},
 { "MQRC_ADAPTER_STORAGE_SHORTAGE"    , MQRC_ADAPTER_STORAGE_SHORTAGE   , 1},
 { "MQRC_ALIAS_BASE_Q_TYPE_ERROR"     , MQRC_ALIAS_BASE_Q_TYPE_ERROR    , 1},
 { "MQRC_ALREADY_CONNECTED"           , MQRC_ALREADY_CONNECTED          , 1},
 { "MQRC_ANOTHER_Q_MGR_CONNECTED"     , MQRC_ANOTHER_Q_MGR_CONNECTED    , 1},
 { "MQRC_API_EXIT_LOAD_ERROR"         , MQRC_API_EXIT_LOAD_ERROR        , 1},
 { "MQRC_API_EXIT_NOT_FOUND"          , MQRC_API_EXIT_NOT_FOUND         , 1},
 { "MQRC_ASID_MISMATCH"               , MQRC_ASID_MISMATCH              , 1},
 { "MQRC_BACKED_OUT"                  , MQRC_BACKED_OUT                 , 1},
 { "MQRC_BUFFER_ERROR"                , MQRC_BUFFER_ERROR               , 1},
 { "MQRC_BUFFER_LENGTH_ERROR"         , MQRC_BUFFER_LENGTH_ERROR        , 1},
 { "MQRC_CALL_IN_PROGRESS"            , MQRC_CALL_IN_PROGRESS           , 1},
 { "MQRC_CHANNEL_ACTIVATED"           , MQRC_CHANNEL_ACTIVATED          , 1},
 { "MQRC_CHANNEL_CONV_ERROR"          , MQRC_CHANNEL_CONV_ERROR         , 1},
 { "MQRC_CHANNEL_NOT_ACTIVATED"       , MQRC_CHANNEL_NOT_ACTIVATED      , 1},
 { "MQRC_CHANNEL_STARTED"             , MQRC_CHANNEL_STARTED            , 1},
 { "MQRC_CHANNEL_STOPPED"             , MQRC_CHANNEL_STOPPED            , 1},
 { "MQRC_CHAR_ATTR_LENGTH_ERROR"      , MQRC_CHAR_ATTR_LENGTH_ERROR     , 1},
 { "MQRC_CHAR_ATTRS_ERROR"            , MQRC_CHAR_ATTRS_ERROR           , 1},
 { "MQRC_CHAR_ATTRS_TOO_SHORT"        , MQRC_CHAR_ATTRS_TOO_SHORT       , 1},
 { "MQRC_CICS_WAIT_FAILED"            , MQRC_CICS_WAIT_FAILED           , 1},
 { "MQRC_CONN_ID_IN_USE"              , MQRC_CONN_ID_IN_USE             , 1},
 { "MQRC_CONNECTION_BROKEN"           , MQRC_CONNECTION_BROKEN          , 1},
 { "MQRC_CONNECTION_NOT_AUTHORIZED"   , MQRC_CONNECTION_NOT_AUTHORIZED  , 1},
 { "MQRC_CONNECTION_QUIESCING"        , MQRC_CONNECTION_QUIESCING       , 1},
 { "MQRC_CONNECTION_STOPPING"         , MQRC_CONNECTION_STOPPING        , 1},
 { "MQRC_CONTEXT_HANDLE_ERROR"        , MQRC_CONTEXT_HANDLE_ERROR       , 1},
 { "MQRC_CONTEXT_NOT_AVAILABLE"       , MQRC_CONTEXT_NOT_AVAILABLE      , 1},
 { "MQRC_CONVERTED_MSG_TOO_BIG"       , MQRC_CONVERTED_MSG_TOO_BIG      , 1},
 { "MQRC_CORREL_ID_ERROR"             , MQRC_CORREL_ID_ERROR            , 1},
 { "MQRC_DATA_LENGTH_ERROR"           , MQRC_DATA_LENGTH_ERROR          , 1},
 { "MQRC_DBCS_ERROR"                  , MQRC_DBCS_ERROR                 , 1},
 { "MQRC_DEF_XMIT_Q_TYPE_ERROR"       , MQRC_DEF_XMIT_Q_TYPE_ERROR      , 1},
 { "MQRC_DEF_XMIT_Q_USAGE_ERROR"      , MQRC_DEF_XMIT_Q_USAGE_ERROR     , 1},
 { "MQRC_DUPLICATE_RECOV_COORD"       , MQRC_DUPLICATE_RECOV_COORD      , 1},
 { "MQRC_DYNAMIC_Q_NAME_ERROR"        , MQRC_DYNAMIC_Q_NAME_ERROR       , 1},
 { "MQRC_ENVIRONMENT_ERROR"           , MQRC_ENVIRONMENT_ERROR          , 1},
 { "MQRC_EXPIRY_ERROR"                , MQRC_EXPIRY_ERROR               , 1},
 { "MQRC_FEEDBACK_ERROR"              , MQRC_FEEDBACK_ERROR             , 1},
 { "MQRC_FILE_NOT_AUDITED"            , MQRC_FILE_NOT_AUDITED           , 1},
 { "MQRC_FILE_SYSTEM_ERROR"           , MQRC_FILE_SYSTEM_ERROR          , 1},
 { "MQRC_FORMAT_ERROR"                , MQRC_FORMAT_ERROR               , 1},
 { "MQRC_FUNCTION_ERROR"              , MQRC_FUNCTION_ERROR             , 1},
 { "MQRC_GET_INHIBITED"               , MQRC_GET_INHIBITED              , 1},
 { "MQRC_GMO_ERROR"                   , MQRC_GMO_ERROR                  , 1},
 { "MQRC_HANDLE_NOT_AVAILABLE"        , MQRC_HANDLE_NOT_AVAILABLE       , 1},
 { "MQRC_HCONFIG_ERROR"               , MQRC_HCONFIG_ERROR              , 1},
 { "MQRC_HCONN_ERROR"                 , MQRC_HCONN_ERROR                , 1},
 { "MQRC_HOBJ_ERROR"                  , MQRC_HOBJ_ERROR                 , 1},
 { "MQRC_INHIBIT_VALUE_ERROR"         , MQRC_INHIBIT_VALUE_ERROR        , 1},
 { "MQRC_INITIALIZATION_FAILED"       , MQRC_INITIALIZATION_FAILED      , 1},
 { "MQRC_INT_ATTR_COUNT_ERROR"        , MQRC_INT_ATTR_COUNT_ERROR       , 1},
 { "MQRC_INT_ATTR_COUNT_TOO_SMALL"    , MQRC_INT_ATTR_COUNT_TOO_SMALL   , 1},
 { "MQRC_INT_ATTRS_ARRAY_ERROR"       , MQRC_INT_ATTRS_ARRAY_ERROR      , 1},
 { "MQRC_MAX_CONNS_LIMIT_REACHED"     , MQRC_MAX_CONNS_LIMIT_REACHED    , 1},
 { "MQRC_MD_ERROR"                    , MQRC_MD_ERROR                   , 1},
 { "MQRC_MISSING_REPLY_TO_Q"          , MQRC_MISSING_REPLY_TO_Q         , 1},
 { "MQRC_MSG_ID_ERROR"                , MQRC_MSG_ID_ERROR               , 1},
 { "MQRC_MSG_TOO_BIG_FOR_CHANNEL"     , MQRC_MSG_TOO_BIG_FOR_CHANNEL    , 1},
 { "MQRC_MSG_TOO_BIG_FOR_Q_MGR"       , MQRC_MSG_TOO_BIG_FOR_Q_MGR      , 1},
 { "MQRC_MSG_TOO_BIG_FOR_Q"           , MQRC_MSG_TOO_BIG_FOR_Q          , 1},
 { "MQRC_MSG_TYPE_ERROR"              , MQRC_MSG_TYPE_ERROR             , 1},
 { "MQRC_NAME_IN_USE"                 , MQRC_NAME_IN_USE                , 1},
 { "MQRC_NAME_NOT_VALID_FOR_TYPE"     , MQRC_NAME_NOT_VALID_FOR_TYPE    , 1},
 { "MQRC_NO_MSG_AVAILABLE"            , MQRC_NO_MSG_AVAILABLE           , 1},
 { "MQRC_NO_MSG_LOCKED"               , MQRC_NO_MSG_LOCKED              , 1},
 { "MQRC_NO_MSG_UNDER_CURSOR"         , MQRC_NO_MSG_UNDER_CURSOR        , 1},
 { "MQRC_NONE"                        , MQRC_NONE                       , 1},
 { "MQRC_NOT_AUTHORIZED"              , MQRC_NOT_AUTHORIZED             , 1},
 { "MQRC_NOT_CONVERTED"               , MQRC_NOT_CONVERTED              , 1},
 { "MQRC_NOT_OPEN_FOR_BROWSE"         , MQRC_NOT_OPEN_FOR_BROWSE        , 1},
 { "MQRC_NOT_OPEN_FOR_INPUT"          , MQRC_NOT_OPEN_FOR_INPUT         , 1},
 { "MQRC_NOT_OPEN_FOR_INQUIRE"        , MQRC_NOT_OPEN_FOR_INQUIRE       , 1},
 { "MQRC_NOT_OPEN_FOR_OUTPUT"         , MQRC_NOT_OPEN_FOR_OUTPUT        , 1},
 { "MQRC_NOT_OPEN_FOR_PASS_ALL"       , MQRC_NOT_OPEN_FOR_PASS_ALL      , 1},
 { "MQRC_NOT_OPEN_FOR_PASS_IDENT"     , MQRC_NOT_OPEN_FOR_PASS_IDENT    , 1},
 { "MQRC_NOT_OPEN_FOR_SET_ALL"        , MQRC_NOT_OPEN_FOR_SET_ALL       , 1},
 { "MQRC_NOT_OPEN_FOR_SET_IDENT"      , MQRC_NOT_OPEN_FOR_SET_IDENT     , 1},
 { "MQRC_NOT_OPEN_FOR_SET"            , MQRC_NOT_OPEN_FOR_SET           , 1},
 { "MQRC_OBJECT_ALREADY_EXISTS"       , MQRC_OBJECT_ALREADY_EXISTS      , 1},
 { "MQRC_OBJECT_CHANGED"              , MQRC_OBJECT_CHANGED             , 1},
 { "MQRC_OBJECT_DAMAGED"              , MQRC_OBJECT_DAMAGED             , 1},
 { "MQRC_OBJECT_IN_USE"               , MQRC_OBJECT_IN_USE              , 1},
 { "MQRC_OBJECT_TYPE_ERROR"           , MQRC_OBJECT_TYPE_ERROR          , 1},
 { "MQRC_OD_ERROR"                    , MQRC_OD_ERROR                   , 1},
 { "MQRC_OPTION_NOT_VALID_FOR_TYPE"   , MQRC_OPTION_NOT_VALID_FOR_TYPE  , 1},
 { "MQRC_OPTIONS_ERROR"               , MQRC_OPTIONS_ERROR              , 1},
 { "MQRC_PAGESET_ERROR"               , MQRC_PAGESET_ERROR              , 1},
 { "MQRC_PAGESET_FULL"                , MQRC_PAGESET_FULL               , 1},
 { "MQRC_PERSISTENCE_ERROR"           , MQRC_PERSISTENCE_ERROR          , 1},
 { "MQRC_PERSISTENT_NOT_ALLOWED"      , MQRC_PERSISTENT_NOT_ALLOWED     , 1},
 { "MQRC_PMO_ERROR"                   , MQRC_PMO_ERROR                  , 1},
 { "MQRC_PRIORITY_ERROR"              , MQRC_PRIORITY_ERROR             , 1},
 { "MQRC_PRIORITY_EXCEEDS_MAXIMUM"    , MQRC_PRIORITY_EXCEEDS_MAXIMUM   , 1},
 { "MQRC_PUT_INHIBITED"               , MQRC_PUT_INHIBITED              , 1},
 { "MQRC_Q_ALREADY_EXISTS"            , MQRC_Q_ALREADY_EXISTS           , 1},
 { "MQRC_Q_DELETED"                   , MQRC_Q_DELETED                  , 1},
 { "MQRC_Q_DEPTH_HIGH"                , MQRC_Q_DEPTH_HIGH               , 1},
 { "MQRC_Q_DEPTH_LOW"                 , MQRC_Q_DEPTH_LOW                , 1},
 { "MQRC_Q_FULL"                      , MQRC_Q_FULL                     , 1},
 { "MQRC_Q_MGR_ACTIVE"                , MQRC_Q_MGR_ACTIVE               , 1},
 { "MQRC_Q_MGR_NAME_ERROR"            , MQRC_Q_MGR_NAME_ERROR           , 1},
 { "MQRC_Q_MGR_NOT_ACTIVE"            , MQRC_Q_MGR_NOT_ACTIVE           , 1},
 { "MQRC_Q_MGR_NOT_AVAILABLE"         , MQRC_Q_MGR_NOT_AVAILABLE        , 1},
 { "MQRC_Q_MGR_QUIESCING"             , MQRC_Q_MGR_QUIESCING            , 1},
 { "MQRC_Q_MGR_STOPPING"              , MQRC_Q_MGR_STOPPING             , 1},
 { "MQRC_Q_NOT_EMPTY"                 , MQRC_Q_NOT_EMPTY                , 1},
 { "MQRC_Q_SERVICE_INTERVAL_HIGH"     , MQRC_Q_SERVICE_INTERVAL_HIGH    , 1},
 { "MQRC_Q_SERVICE_INTERVAL_OK"       , MQRC_Q_SERVICE_INTERVAL_OK      , 1},
 { "MQRC_Q_SPACE_NOT_AVAILABLE"       , MQRC_Q_SPACE_NOT_AVAILABLE      , 1},
 { "MQRC_Q_TYPE_ERROR"                , MQRC_Q_TYPE_ERROR               , 1},
 { "MQRC_REMOTE_Q_NAME_ERROR"         , MQRC_REMOTE_Q_NAME_ERROR        , 1},
 { "MQRC_REPORT_OPTIONS_ERROR"        , MQRC_REPORT_OPTIONS_ERROR       , 1},
 { "MQRC_RESOURCE_PROBLEM"            , MQRC_RESOURCE_PROBLEM           , 1},
 { "MQRC_SECOND_MARK_NOT_ALLOWED"     , MQRC_SECOND_MARK_NOT_ALLOWED    , 1},
 { "MQRC_SECURITY_ERROR"              , MQRC_SECURITY_ERROR             , 1},
 { "MQRC_SELECTOR_COUNT_ERROR"        , MQRC_SELECTOR_COUNT_ERROR       , 1},
 { "MQRC_SELECTOR_ERROR"              , MQRC_SELECTOR_ERROR             , 1},
 { "MQRC_SELECTOR_LIMIT_EXCEEDED"     , MQRC_SELECTOR_LIMIT_EXCEEDED    , 1},
 { "MQRC_SELECTOR_NOT_FOR_TYPE"       , MQRC_SELECTOR_NOT_FOR_TYPE      , 1},
 { "MQRC_SERVICE_ERROR"               , MQRC_SERVICE_ERROR              , 1},
 { "MQRC_SERVICE_NOT_AVAILABLE"       , MQRC_SERVICE_NOT_AVAILABLE      , 1},
 { "MQRC_SIGNAL_OUTSTANDING"          , MQRC_SIGNAL_OUTSTANDING         , 1},
 { "MQRC_SIGNAL_REQUEST_ACCEPTED"     , MQRC_SIGNAL_REQUEST_ACCEPTED    , 1},
 { "MQRC_SIGNAL1_ERROR"               , MQRC_SIGNAL1_ERROR              , 1},
 { "MQRC_SOURCE_BUFFER_ERROR"         , MQRC_SOURCE_BUFFER_ERROR        , 1},
 { "MQRC_SOURCE_CCSID_ERROR"          , MQRC_SOURCE_CCSID_ERROR         , 1},
 { "MQRC_SOURCE_DECIMAL_ENC_ERROR"    , MQRC_SOURCE_DECIMAL_ENC_ERROR   , 1},
 { "MQRC_SOURCE_FLOAT_ENC_ERROR"      , MQRC_SOURCE_FLOAT_ENC_ERROR     , 1},
 { "MQRC_SOURCE_INTEGER_ENC_ERROR"    , MQRC_SOURCE_INTEGER_ENC_ERROR   , 1},
 { "MQRC_SOURCE_LENGTH_ERROR"         , MQRC_SOURCE_LENGTH_ERROR        , 1},
 { "MQRC_STORAGE_NOT_AVAILABLE"       , MQRC_STORAGE_NOT_AVAILABLE      , 1},
 { "MQRC_SUPPRESSED_BY_EXIT"          , MQRC_SUPPRESSED_BY_EXIT         , 1},
 { "MQRC_SYNCPOINT_LIMIT_REACHED"     , MQRC_SYNCPOINT_LIMIT_REACHED    , 1},
 { "MQRC_SYNCPOINT_NOT_AVAILABLE"     , MQRC_SYNCPOINT_NOT_AVAILABLE    , 1},
 { "MQRC_TARGET_BUFFER_ERROR"         , MQRC_TARGET_BUFFER_ERROR        , 1},
 { "MQRC_TARGET_CCSID_ERROR"          , MQRC_TARGET_CCSID_ERROR         , 1},
 { "MQRC_TARGET_DECIMAL_ENC_ERROR"    , MQRC_TARGET_DECIMAL_ENC_ERROR   , 1},
 { "MQRC_TARGET_FLOAT_ENC_ERROR"      , MQRC_TARGET_FLOAT_ENC_ERROR     , 1},
 { "MQRC_TARGET_INTEGER_ENC_ERROR"    , MQRC_TARGET_INTEGER_ENC_ERROR   , 1},
 { "MQRC_TARGET_LENGTH_ERROR"         , MQRC_TARGET_LENGTH_ERROR        , 1},
 { "MQRC_TERMINATION_FAILED"          , MQRC_TERMINATION_FAILED         , 1},
 { "MQRC_TRIGGER_CONTROL_ERROR"       , MQRC_TRIGGER_CONTROL_ERROR      , 1},
 { "MQRC_TRIGGER_DEPTH_ERROR"         , MQRC_TRIGGER_DEPTH_ERROR        , 1},
 { "MQRC_TRIGGER_MSG_PRIORITY_ERR"    , MQRC_TRIGGER_MSG_PRIORITY_ERR   , 1},
 { "MQRC_TRIGGER_TYPE_ERROR"          , MQRC_TRIGGER_TYPE_ERROR         , 1},
 { "MQRC_TRUNCATED_MSG_ACCEPTED"      , MQRC_TRUNCATED_MSG_ACCEPTED     , 1},
 { "MQRC_TRUNCATED_MSG_FAILED"        , MQRC_TRUNCATED_MSG_FAILED       , 1},
 { "MQRC_TRUNCATED"                   , MQRC_TRUNCATED                  , 1},
 { "MQRC_UNEXPECTED_ERROR"            , MQRC_UNEXPECTED_ERROR           , 1},
 { "MQRC_UNKNOWN_ALIAS_BASE_Q"        , MQRC_UNKNOWN_ALIAS_BASE_Q       , 1},
 { "MQRC_UNKNOWN_AUTH_ENTITY"         , MQRC_UNKNOWN_AUTH_ENTITY        , 1},
 { "MQRC_UNKNOWN_DEF_XMIT_Q"          , MQRC_UNKNOWN_DEF_XMIT_Q         , 1},
 { "MQRC_UNKNOWN_ENTITY"              , MQRC_UNKNOWN_ENTITY             , 1},
 { "MQRC_UNKNOWN_OBJECT_NAME"         , MQRC_UNKNOWN_OBJECT_NAME        , 1},
 { "MQRC_UNKNOWN_OBJECT_Q_MGR"        , MQRC_UNKNOWN_OBJECT_Q_MGR       , 1},
 { "MQRC_UNKNOWN_Q_NAME"              , MQRC_UNKNOWN_Q_NAME             , 1},
 { "MQRC_UNKNOWN_REF_OBJECT"          , MQRC_UNKNOWN_REF_OBJECT         , 1},
 { "MQRC_UNKNOWN_REMOTE_Q_MGR"        , MQRC_UNKNOWN_REMOTE_Q_MGR       , 1},
 { "MQRC_UNKNOWN_REPORT_OPTION"       , MQRC_UNKNOWN_REPORT_OPTION      , 1},
 { "MQRC_UNKNOWN_XMIT_Q"              , MQRC_UNKNOWN_XMIT_Q             , 1},
 { "MQRC_USER_ID_NOT_AVAILABLE"       , MQRC_USER_ID_NOT_AVAILABLE      , 1},
 { "MQRC_WAIT_INTERVAL_ERROR"         , MQRC_WAIT_INTERVAL_ERROR        , 1},
 { "MQRC_XMIT_Q_TYPE_ERROR"           , MQRC_XMIT_Q_TYPE_ERROR          , 1},
 { "MQRC_XMIT_Q_USAGE_ERROR"          , MQRC_XMIT_Q_USAGE_ERROR         , 1},
 { "MQRCCF_ALLOCATE_FAILED"           , MQRCCF_ALLOCATE_FAILED          , 1},
 { "MQRCCF_ATTR_VALUE_ERROR"          , MQRCCF_ATTR_VALUE_ERROR         , 1},
 { "MQRCCF_BATCH_SIZE_ERROR"          , MQRCCF_BATCH_SIZE_ERROR         , 1},
 { "MQRCCF_BIND_FAILED"               , MQRCCF_BIND_FAILED              , 1},
 { "MQRCCF_CCSID_ERROR"               , MQRCCF_CCSID_ERROR              , 1},
 { "MQRCCF_CELL_DIR_NOT_AVAILABLE"    , MQRCCF_CELL_DIR_NOT_AVAILABLE   , 1},
 { "MQRCCF_CFH_COMMAND_ERROR"         , MQRCCF_CFH_COMMAND_ERROR        , 1},
 { "MQRCCF_CFH_CONTROL_ERROR"         , MQRCCF_CFH_CONTROL_ERROR        , 1},
 { "MQRCCF_CFH_LENGTH_ERROR"          , MQRCCF_CFH_LENGTH_ERROR         , 1},
 { "MQRCCF_CFH_MSG_SEQ_NUMBER_ERR"    , MQRCCF_CFH_MSG_SEQ_NUMBER_ERR   , 1},
 { "MQRCCF_CFH_PARM_COUNT_ERROR"      , MQRCCF_CFH_PARM_COUNT_ERROR     , 1},
 { "MQRCCF_CFH_TYPE_ERROR"            , MQRCCF_CFH_TYPE_ERROR           , 1},
 { "MQRCCF_CFH_VERSION_ERROR"         , MQRCCF_CFH_VERSION_ERROR        , 1},
 { "MQRCCF_CFIL_COUNT_ERROR"          , MQRCCF_CFIL_COUNT_ERROR         , 1},
 { "MQRCCF_CFIL_DUPLICATE_VALUE"      , MQRCCF_CFIL_DUPLICATE_VALUE     , 1},
 { "MQRCCF_CFIL_LENGTH_ERROR"         , MQRCCF_CFIL_LENGTH_ERROR        , 1},
 { "MQRCCF_CFIL_PARM_ID_ERROR"        , MQRCCF_CFIL_PARM_ID_ERROR       , 1},
 { "MQRCCF_CFIN_DUPLICATE_PARM"       , MQRCCF_CFIN_DUPLICATE_PARM      , 1},
 { "MQRCCF_CFIN_LENGTH_ERROR"         , MQRCCF_CFIN_LENGTH_ERROR        , 1},
 { "MQRCCF_CFIN_PARM_ID_ERROR"        , MQRCCF_CFIN_PARM_ID_ERROR       , 1},
 { "MQRCCF_CFST_DUPLICATE_PARM"       , MQRCCF_CFST_DUPLICATE_PARM      , 1},
 { "MQRCCF_CFST_LENGTH_ERROR"         , MQRCCF_CFST_LENGTH_ERROR        , 1},
 { "MQRCCF_CFST_PARM_ID_ERROR"        , MQRCCF_CFST_PARM_ID_ERROR       , 1},
 { "MQRCCF_CFST_STRING_LENGTH_ERR"    , MQRCCF_CFST_STRING_LENGTH_ERR   , 1},
 { "MQRCCF_CHANNEL_ALREADY_EXISTS"    , MQRCCF_CHANNEL_ALREADY_EXISTS   , 1},
 { "MQRCCF_CHANNEL_DISABLED"          , MQRCCF_CHANNEL_DISABLED         , 1},
 { "MQRCCF_CHANNEL_IN_USE"            , MQRCCF_CHANNEL_IN_USE           , 1},
 { "MQRCCF_CHANNEL_INDOUBT"           , MQRCCF_CHANNEL_INDOUBT          , 1},
 { "MQRCCF_CHANNEL_NAME_ERROR"        , MQRCCF_CHANNEL_NAME_ERROR       , 1},
 { "MQRCCF_CHANNEL_NOT_ACTIVE"        , MQRCCF_CHANNEL_NOT_ACTIVE       , 1},
 { "MQRCCF_CHANNEL_NOT_FOUND"         , MQRCCF_CHANNEL_NOT_FOUND        , 1},
 { "MQRCCF_CHANNEL_TABLE_ERROR"       , MQRCCF_CHANNEL_TABLE_ERROR      , 1},
 { "MQRCCF_CHANNEL_TYPE_ERROR"        , MQRCCF_CHANNEL_TYPE_ERROR       , 1},
 { "MQRCCF_CHL_INST_TYPE_ERROR"       , MQRCCF_CHL_INST_TYPE_ERROR      , 1},
 { "MQRCCF_CHL_STATUS_NOT_FOUND"      , MQRCCF_CHL_STATUS_NOT_FOUND     , 1},
 { "MQRCCF_COMMAND_FAILED"            , MQRCCF_COMMAND_FAILED           , 1},
 { "MQRCCF_COMMIT_FAILED"             , MQRCCF_COMMIT_FAILED            , 1},
 { "MQRCCF_CONFIGURATION_ERROR"       , MQRCCF_CONFIGURATION_ERROR      , 1},
 { "MQRCCF_CONN_NAME_ERROR"           , MQRCCF_CONN_NAME_ERROR          , 1},
 { "MQRCCF_CONNECTION_CLOSED"         , MQRCCF_CONNECTION_CLOSED        , 1},
 { "MQRCCF_CONNECTION_REFUSED"        , MQRCCF_CONNECTION_REFUSED       , 1},
 { "MQRCCF_DATA_CONV_VALUE_ERROR"     , MQRCCF_DATA_CONV_VALUE_ERROR    , 1},
 { "MQRCCF_DATA_TOO_LARGE"            , MQRCCF_DATA_TOO_LARGE           , 1},
 { "MQRCCF_DISC_INT_ERROR"            , MQRCCF_DISC_INT_ERROR           , 1},
 { "MQRCCF_DISC_INT_WRONG_TYPE"       , MQRCCF_DISC_INT_WRONG_TYPE      , 1},
 { "MQRCCF_DYNAMIC_Q_SCOPE_ERROR"     , MQRCCF_DYNAMIC_Q_SCOPE_ERROR    , 1},
 { "MQRCCF_ENCODING_ERROR"            , MQRCCF_ENCODING_ERROR           , 1},
 { "MQRCCF_ENTRY_ERROR"               , MQRCCF_ENTRY_ERROR              , 1},
 { "MQRCCF_ESCAPE_TYPE_ERROR"         , MQRCCF_ESCAPE_TYPE_ERROR        , 1},
 { "MQRCCF_FORCE_VALUE_ERROR"         , MQRCCF_FORCE_VALUE_ERROR        , 1},
 { "MQRCCF_HOST_NOT_AVAILABLE"        , MQRCCF_HOST_NOT_AVAILABLE       , 1},
 { "MQRCCF_INDOUBT_VALUE_ERROR"       , MQRCCF_INDOUBT_VALUE_ERROR      , 1},
 { "MQRCCF_LIKE_OBJECT_WRONG_TYPE"    , MQRCCF_LIKE_OBJECT_WRONG_TYPE   , 1},
 { "MQRCCF_LISTENER_NOT_STARTED"      , MQRCCF_LISTENER_NOT_STARTED     , 1},
 { "MQRCCF_LONG_RETRY_ERROR"          , MQRCCF_LONG_RETRY_ERROR         , 1},
 { "MQRCCF_LONG_RETRY_WRONG_TYPE"     , MQRCCF_LONG_RETRY_WRONG_TYPE    , 1},
 { "MQRCCF_LONG_TIMER_ERROR"          , MQRCCF_LONG_TIMER_ERROR         , 1},
 { "MQRCCF_LONG_TIMER_WRONG_TYPE"     , MQRCCF_LONG_TIMER_WRONG_TYPE    , 1},
 { "MQRCCF_MAX_MSG_LENGTH_ERROR"      , MQRCCF_MAX_MSG_LENGTH_ERROR     , 1},
 { "MQRCCF_MCA_NAME_ERROR"            , MQRCCF_MCA_NAME_ERROR           , 1},
 { "MQRCCF_MCA_NAME_WRONG_TYPE"       , MQRCCF_MCA_NAME_WRONG_TYPE      , 1},
 { "MQRCCF_MCA_TYPE_ERROR"            , MQRCCF_MCA_TYPE_ERROR           , 1},
 { "MQRCCF_MD_FORMAT_ERROR"           , MQRCCF_MD_FORMAT_ERROR          , 1},
 { "MQRCCF_MISSING_CONN_NAME"         , MQRCCF_MISSING_CONN_NAME        , 1},
 { "MQRCCF_MQCONN_FAILED"             , MQRCCF_MQCONN_FAILED            , 1},
 { "MQRCCF_MQGET_FAILED"              , MQRCCF_MQGET_FAILED             , 1},
 { "MQRCCF_MQINQ_FAILED"              , MQRCCF_MQINQ_FAILED             , 1},
 { "MQRCCF_MQOPEN_FAILED"             , MQRCCF_MQOPEN_FAILED            , 1},
 { "MQRCCF_MQPUT_FAILED"              , MQRCCF_MQPUT_FAILED             , 1},
 { "MQRCCF_MQSET_FAILED"              , MQRCCF_MQSET_FAILED             , 1},
 { "MQRCCF_MR_COUNT_ERROR"            , MQRCCF_MR_COUNT_ERROR           , 1},
 { "MQRCCF_MR_COUNT_WRONG_TYPE"       , MQRCCF_MR_COUNT_WRONG_TYPE      , 1},
 { "MQRCCF_MR_EXIT_NAME_ERROR"        , MQRCCF_MR_EXIT_NAME_ERROR       , 1},
 { "MQRCCF_MR_EXIT_NAME_WRONG_TYPE"   , MQRCCF_MR_EXIT_NAME_WRONG_TYPE  , 1},
 { "MQRCCF_MR_INTERVAL_ERROR"         , MQRCCF_MR_INTERVAL_ERROR        , 1},
 { "MQRCCF_MR_INTERVAL_WRONG_TYPE"    , MQRCCF_MR_INTERVAL_WRONG_TYPE   , 1},
 { "MQRCCF_MSG_EXIT_NAME_ERROR"       , MQRCCF_MSG_EXIT_NAME_ERROR      , 1},
 { "MQRCCF_MSG_LENGTH_ERROR"          , MQRCCF_MSG_LENGTH_ERROR         , 1},
 { "MQRCCF_MSG_SEQ_NUMBER_ERROR"      , MQRCCF_MSG_SEQ_NUMBER_ERROR     , 1},
 { "MQRCCF_MSG_TRUNCATED"             , MQRCCF_MSG_TRUNCATED            , 1},
 { "MQRCCF_NO_COMMS_MANAGER"          , MQRCCF_NO_COMMS_MANAGER         , 1},
 { "MQRCCF_NO_STORAGE"                , MQRCCF_NO_STORAGE               , 1},
 { "MQRCCF_NOT_XMIT_Q"                , MQRCCF_NOT_XMIT_Q               , 1},
 { "MQRCCF_OBJECT_ALREADY_EXISTS"     , MQRCCF_OBJECT_ALREADY_EXISTS    , 1},
 { "MQRCCF_OBJECT_NAME_ERROR"         , MQRCCF_OBJECT_NAME_ERROR        , 1},
 { "MQRCCF_OBJECT_OPEN"               , MQRCCF_OBJECT_OPEN              , 1},
 { "MQRCCF_OBJECT_WRONG_TYPE"         , MQRCCF_OBJECT_WRONG_TYPE        , 1},
 { "MQRCCF_PARM_COUNT_TOO_BIG"        , MQRCCF_PARM_COUNT_TOO_BIG       , 1},
 { "MQRCCF_PARM_COUNT_TOO_SMALL"      , MQRCCF_PARM_COUNT_TOO_SMALL     , 1},
 { "MQRCCF_PARM_SEQUENCE_ERROR"       , MQRCCF_PARM_SEQUENCE_ERROR      , 1},
 { "MQRCCF_PING_DATA_COMPARE_ERROR"   , MQRCCF_PING_DATA_COMPARE_ERROR  , 1},
 { "MQRCCF_PING_DATA_COUNT_ERROR"     , MQRCCF_PING_DATA_COUNT_ERROR    , 1},
 { "MQRCCF_PING_ERROR"                , MQRCCF_PING_ERROR               , 1},
 { "MQRCCF_PURGE_VALUE_ERROR"         , MQRCCF_PURGE_VALUE_ERROR        , 1},
 { "MQRCCF_PUT_AUTH_ERROR"            , MQRCCF_PUT_AUTH_ERROR           , 1},
 { "MQRCCF_PUT_AUTH_WRONG_TYPE"       , MQRCCF_PUT_AUTH_WRONG_TYPE      , 1},
 { "MQRCCF_Q_ALREADY_IN_CELL"         , MQRCCF_Q_ALREADY_IN_CELL        , 1},
 { "MQRCCF_Q_TYPE_ERROR"              , MQRCCF_Q_TYPE_ERROR             , 1},
 { "MQRCCF_Q_WRONG_TYPE"              , MQRCCF_Q_WRONG_TYPE             , 1},
 { "MQRCCF_QUIESCE_VALUE_ERROR"       , MQRCCF_QUIESCE_VALUE_ERROR      , 1},
 { "MQRCCF_RCV_EXIT_NAME_ERROR"       , MQRCCF_RCV_EXIT_NAME_ERROR      , 1},
 { "MQRCCF_RECEIVE_FAILED"            , MQRCCF_RECEIVE_FAILED           , 1},
 { "MQRCCF_RECEIVED_DATA_ERROR"       , MQRCCF_RECEIVED_DATA_ERROR      , 1},
 { "MQRCCF_REMOTE_QM_TERMINATING"     , MQRCCF_REMOTE_QM_TERMINATING    , 1},
 { "MQRCCF_REMOTE_QM_UNAVAILABLE"     , MQRCCF_REMOTE_QM_UNAVAILABLE    , 1},
 { "MQRCCF_REPLACE_VALUE_ERROR"       , MQRCCF_REPLACE_VALUE_ERROR      , 1},
 { "MQRCCF_SEC_EXIT_NAME_ERROR"       , MQRCCF_SEC_EXIT_NAME_ERROR      , 1},
 { "MQRCCF_SEND_EXIT_NAME_ERROR"      , MQRCCF_SEND_EXIT_NAME_ERROR     , 1},
 { "MQRCCF_SEND_FAILED"               , MQRCCF_SEND_FAILED              , 1},
 { "MQRCCF_SEQ_NUMBER_WRAP_ERROR"     , MQRCCF_SEQ_NUMBER_WRAP_ERROR    , 1},
 { "MQRCCF_SHORT_RETRY_ERROR"         , MQRCCF_SHORT_RETRY_ERROR        , 1},
 { "MQRCCF_SHORT_RETRY_WRONG_TYPE"    , MQRCCF_SHORT_RETRY_WRONG_TYPE   , 1},
 { "MQRCCF_SHORT_TIMER_ERROR"         , MQRCCF_SHORT_TIMER_ERROR        , 1},
 { "MQRCCF_SHORT_TIMER_WRONG_TYPE"    , MQRCCF_SHORT_TIMER_WRONG_TYPE   , 1},
 { "MQRCCF_STRUCTURE_TYPE_ERROR"      , MQRCCF_STRUCTURE_TYPE_ERROR     , 1},
 { "MQRCCF_TERMINATED_BY_SEC_EXIT"    , MQRCCF_TERMINATED_BY_SEC_EXIT   , 1},
 { "MQRCCF_UNKNOWN_Q_MGR"             , MQRCCF_UNKNOWN_Q_MGR            , 1},
 { "MQRCCF_UNKNOWN_REMOTE_CHANNEL"    , MQRCCF_UNKNOWN_REMOTE_CHANNEL   , 1},
 { "MQRCCF_USER_EXIT_NOT_AVAILABLE"   , MQRCCF_USER_EXIT_NOT_AVAILABLE  , 1},
 { "MQRCCF_XMIT_PROTOCOL_TYPE_ERR"    , MQRCCF_XMIT_PROTOCOL_TYPE_ERR   , 1},
 { "MQRCCF_XMIT_Q_NAME_ERROR"         , MQRCCF_XMIT_Q_NAME_ERROR        , 1},
 { "MQRCCF_XMIT_Q_NAME_WRONG_TYPE"    , MQRCCF_XMIT_Q_NAME_WRONG_TYPE   , 1},

   // MQ Rexx Null Variables
 { "MQAT_NONE"                        , 16                              , 2},
 { "MQCI_NONE"                        , 24                              , 2},
 { "MQMI_NONE"                        , 24                              , 2},

   // MQ Rexx Info variables
 { "RXMQR.LASTMSG"                    , 0                               , 0},
 { "RXMQR.LASTRC"                     , 0                               , 0},
 { "RXMQR.LASTCC"                     , 0                               , 0},
 { "RXMQR.LASTAC"                     , 0                               , 0},
 { "RXMQR.LASTOP"                     , 0                               , 0},
 { "?"                                , 0                               , 0}
 } ;

 DEBUG ("INIT", "Entering setcons" ) ;

 memset(nulls, 0, sizeof(nulls) ) ;

 //
 // Initialise all the constants required.
 //
 // This is done by looping through the table of names and
 //      settings, invoking the RexxVariable interface to set up
 //      each literal.
 //
 // Additionally, if the constant represents Reason Code (MQRC_)
 //               then create an number->name entry in the
 //               RXMQR.RCMAP. stem variable
 //

 for (i=0; ; i++)
   {
    pi = &define_mq_ints??(i??)                           ;
    if ( pi->s_define_mq_ints_name??(0??) == '?' )  break ;

    sprintf((char *)intvalc,"%lu",pi->s_define_mq_ints_value)           ;

    intname.strlength = strlen((const char *)pi->s_define_mq_ints_name) ;
    intname.strptr    =      &(pi->s_define_mq_ints_name??(0??))        ;

    intvalu.strlength = strlen((const char *)intvalc)  ;
    intvalu.strptr     = (char *)intvalc               ;

    sv1.shvnext     = 0                            ;
    memcpy(&sv1.shvname ,&intname,sizeof(intname)) ;
    memcpy(&sv1.shvvalue,&intvalu,sizeof(intvalu)) ;
    sv1.shvnamelen  = intname.strlength            ;
    sv1.shvvaluelen = intvalu.strlength            ;
    sv1.shvcode     = RXSHV_SET                    ;
    sv1.shvret      = 0                            ;

    sv1rc = RexxVariablePool(&sv1)                 ;

    if ( pi->s_define_mq_ints_type == 1 )    //Only for MQRC_ stuff
     {
       sprintf((char *)intvalc,"RXMQR.RCMAP.%lu",
                               pi->s_define_mq_ints_value)           ;
       intname.strptr    = (char *)intvalc                           ;
       intname.strlength = strlen((const char *)intname.strptr)      ;

       intvalu.strptr    =      &(pi->s_define_mq_ints_name??(0??))  ;
       intvalu.strlength = strlen((const char *)intvalu.strptr)      ;

       sv1.shvnext     = 0                            ;
       memcpy(&sv1.shvname ,&intname,sizeof(intname)) ;
       memcpy(&sv1.shvvalue,&intvalu,sizeof(intvalu)) ;
       sv1.shvnamelen  = intname.strlength            ;
       sv1.shvvaluelen = intvalu.strlength            ;
       sv1.shvcode     = RXSHV_SET                    ;
       sv1.shvret      = 0                            ;

       sv1rc = RexxVariablePool(&sv1)                 ;

     }

    if ( pi->s_define_mq_ints_type == 2 )    //Only for Nulls
     {
       intname.strlength = strlen((const char *)pi->s_define_mq_ints_name) ;
       intname.strptr    =      &(pi->s_define_mq_ints_name??(0??))        ;

       intvalu.strlength = pi->s_define_mq_ints_value ;
       intvalu.strptr    = nulls                      ;

       sv1.shvnext     = 0                            ;
       memcpy(&sv1.shvname ,&intname,sizeof(intname)) ;
       memcpy(&sv1.shvvalue,&intvalu,sizeof(intvalu)) ;
       sv1.shvnamelen  = intname.strlength            ;
       sv1.shvvaluelen = intvalu.strlength            ;
       sv1.shvcode     = RXSHV_SET                    ;
       sv1.shvret      = 0                            ;

       sv1rc = RexxVariablePool(&sv1)                 ;
     }

   } // End of Integer initialisations Loop

 DEBUG("INIT", "Leaving setcons " ) ;

 return ;

 } // End of setcons function

//
// Routine geteventname - converts the NUMBER of an Event into
//                        a name (for use as a Component Variable)
//                        (the name is 10 bytes ie 11 long )
//
//         If the PCF field number is unknown, then the
//            component is set to the character version of the
//            PCF event number. This only works for EVENT PCFs!!!!
//
//         Tracing is via the EVENT setting
//
//

void geteventname(char * output, const MQLONG pcfnum )
 {
  DEBUG("EVENT","Entering geteventname for " << pcfnum << " " ) ;

  switch ( pcfnum )
   {
    case MQCA_APPL_ID                    : strcpy(output,"APPLID"    );break;
    case MQCA_BACKOUT_REQ_Q_NAME         : strcpy(output,"BREQQN"    );break;
    case MQCA_BASE_Q_NAME                : strcpy(output,"BQN"       );break;
    case MQCA_COMMAND_INPUT_Q_NAME       : strcpy(output,"CIQN"      );break;
    case MQCA_CREATION_DATE              : strcpy(output,"CREDATE"   );break;
    case MQCA_CREATION_TIME              : strcpy(output,"CRETIME"   );break;
    case MQCA_DEAD_LETTER_Q_NAME         : strcpy(output,"DLQQN"     );break;
    case MQCA_DEF_XMIT_Q_NAME            : strcpy(output,"DEFXQN"    );break;
    case MQCA_ENV_DATA                   : strcpy(output,"ENVDATA"   );break;
    case MQCA_INITIATION_Q_NAME          : strcpy(output,"IQN"       );break;
    case MQCA_NAMES                      : strcpy(output,"NAMES"     );break;
    case MQCA_PROCESS_DESC               : strcpy(output,"PDESC"     );break;
    case MQCA_PROCESS_NAME               : strcpy(output,"PN"        );break;
    case MQCA_Q_DESC                     : strcpy(output,"QDESC"     );break;
    case MQCA_Q_MGR_DESC                 : strcpy(output,"QMDESC"    );break;
    case MQCA_Q_MGR_NAME                 : strcpy(output,"QM"        );break;
    case MQCA_Q_NAME                     : strcpy(output,"QN"        );break;
    case MQCA_REMOTE_Q_MGR_NAME          : strcpy(output,"RQM"       );break;
    case MQCA_REMOTE_Q_NAME              : strcpy(output,"RQN"       );break;
    case MQCA_STORAGE_CLASS              : strcpy(output,"STGCLASS"  );break;
    case MQCA_TRIGGER_DATA               : strcpy(output,"TRIGDATA"  );break;
    case MQCA_USER_DATA                  : strcpy(output,"USERDATA"  );break;
    case MQCA_XMIT_Q_NAME                : strcpy(output,"XXQN"      );break;
    case MQCACF_ALIAS_Q_NAMES            : strcpy(output,"AQNS"      );break;
    case MQCACF_APPL_NAME                : strcpy(output,"AN"        );break;
    case MQCACF_AUX_ERROR_DATA_STR_1     : strcpy(output,"AEDS1"     );break;
    case MQCACF_AUX_ERROR_DATA_STR_2     : strcpy(output,"AEDS2"     );break;
    case MQCACF_AUX_ERROR_DATA_STR_3     : strcpy(output,"AEDS3"     );break;
    case MQCACF_ESCAPE_TEXT              : strcpy(output,"ESC"       );break;
    case MQCACF_FROM_CHANNEL_NAME        : strcpy(output,"FCN"       );break;
    case MQCACF_FROM_PROCESS_NAME        : strcpy(output,"FPN"       );break;
    case MQCACF_FROM_Q_NAME              : strcpy(output,"FQN"       );break;
    case MQCACF_LOCAL_Q_NAMES            : strcpy(output,"LQNS"      );break;
    case MQCACF_MODEL_Q_NAMES            : strcpy(output,"MQNS"      );break;
    case MQCACF_OBJECT_Q_MGR_NAME        : strcpy(output,"OQM"       );break;
    case MQCACF_PROCESS_NAMES            : strcpy(output,"PNS"       );break;
    case MQCACF_Q_NAMES                  : strcpy(output,"QNS"       );break;
    case MQCACF_RECEIVER_CHANNEL_NAMES   : strcpy(output,"RECCNS"    );break;
    case MQCACF_REMOTE_Q_NAMES           : strcpy(output,"REMQNS"    );break;
    case MQCACF_REQUESTER_CHANNEL_NAMES  : strcpy(output,"REQCNS"    );break;
    case MQCACF_SENDER_CHANNEL_NAMES     : strcpy(output,"SENDCNS"   );break;
    case MQCACF_SERVER_CHANNEL_NAMES     : strcpy(output,"SERVCNS"   );break;
    case MQCACF_TO_CHANNEL_NAME          : strcpy(output,"TCN"       );break;
    case MQCACF_TO_PROCESS_NAME          : strcpy(output,"TPN"       );break;
    case MQCACF_TO_Q_NAME                : strcpy(output,"TQN"       );break;
    case MQCACF_USER_IDENTIFIER          : strcpy(output,"UID"       );break;
    case MQCACH_CHANNEL_NAME             : strcpy(output,"CN"        );break;
    case MQCACH_CHANNEL_NAMES            : strcpy(output,"CNS"       );break;
    case MQCACH_CHANNEL_START_DATE       : strcpy(output,"CSD"       );break;
    case MQCACH_CHANNEL_START_TIME       : strcpy(output,"CST"       );break;
    case MQCACH_CONNECTION_NAME          : strcpy(output,"CONN"      );break;
    case MQCACH_CURRENT_LUWID            : strcpy(output,"CLUWID"    );break;
    case MQCACH_DESC                     : strcpy(output,"DESC"      );break;
    case MQCACH_FORMAT_NAME              : strcpy(output,"FORMAT"    );break;
    case MQCACH_LAST_LUWID               : strcpy(output,"LLUWID"    );break;
    case MQCACH_LAST_MSG_DATE            : strcpy(output,"LMSGD"     );break;
    case MQCACH_LAST_MSG_TIME            : strcpy(output,"LMSGT"     );break;
    case MQCACH_MCA_JOB_NAME             : strcpy(output,"MCAJN"     );break;
    case MQCACH_MCA_NAME                 : strcpy(output,"MCAN"      );break;
    case MQCACH_MCA_USER_ID              : strcpy(output,"MCAUID"    );break;
    case MQCACH_MODE_NAME                : strcpy(output,"MCAMODE"   );break;
    case MQCACH_MR_EXIT_NAME             : strcpy(output,"MRXITN"    );break;
    case MQCACH_MR_EXIT_USER_DATA        : strcpy(output,"MRXITUD"   );break;
    case MQCACH_MSG_EXIT_NAME            : strcpy(output,"MSGXN"     );break;
    case MQCACH_MSG_EXIT_USER_DATA       : strcpy(output,"MSGXUD"    );break;
    case MQCACH_PASSWORD                 : strcpy(output,"PASSWORD"  );break;
    case MQCACH_RCV_EXIT_NAME            : strcpy(output,"RCVXN"     );break;
    case MQCACH_RCV_EXIT_USER_DATA       : strcpy(output,"RCVXUD"    );break;
    case MQCACH_SEC_EXIT_NAME            : strcpy(output,"SECXN"     );break;
    case MQCACH_SEC_EXIT_USER_DATA       : strcpy(output,"SECXUD"    );break;
    case MQCACH_SEND_EXIT_NAME           : strcpy(output,"SENDXN"    );break;
    case MQCACH_SEND_EXIT_USER_DATA      : strcpy(output,"SENDXUD"   );break;
    case MQCACH_TP_NAME                  : strcpy(output,"TPNAME"    );break;
    case MQCACH_USER_ID                  : strcpy(output,"USERID"    );break;
    case MQCACH_XMIT_Q_NAME              : strcpy(output,"XQN"       );break;
    case MQIA_APPL_TYPE                  : strcpy(output,"AT"        );break;
    case MQIA_AUTHORITY_EVENT            : strcpy(output,"AE"        );break;
    case MQIA_BACKOUT_THRESHOLD          : strcpy(output,"BT"        );break;
    case MQIA_CODED_CHAR_SET_ID          : strcpy(output,"CCSID"     );break;
    case MQIA_COMMAND_LEVEL              : strcpy(output,"CLEV"      );break;
    case MQIA_CURRENT_Q_DEPTH            : strcpy(output,"CURDEPTH"  );break;
    case MQIA_DEF_INPUT_OPEN_OPTION      : strcpy(output,"DEFIO"     );break;
    case MQIA_DEF_PERSISTENCE            : strcpy(output,"DEFPER"    );break;
    case MQIA_DEF_PRIORITY               : strcpy(output,"DEFPRI"    );break;
    case MQIA_DEFINITION_TYPE            : strcpy(output,"DEFTYPE"   );break;
    case MQIA_HARDEN_GET_BACKOUT         : strcpy(output,"HARDENGB"  );break;
    case MQIA_HIGH_Q_DEPTH               : strcpy(output,"HQD"       );break;
    case MQIA_INHIBIT_EVENT              : strcpy(output,"IE"        );break;
    case MQIA_INHIBIT_GET                : strcpy(output,"IGET"      );break;
    case MQIA_INHIBIT_PUT                : strcpy(output,"IPUT"      );break;
    case MQIA_LOCAL_EVENT                : strcpy(output,"LOCEV"     );break;
    case MQIA_MAX_HANDLES                : strcpy(output,"MAXH"      );break;
    case MQIA_MAX_MSG_LENGTH             : strcpy(output,"MAXMSG"    );break;
    case MQIA_MAX_PRIORITY               : strcpy(output,"MAXPRI"    );break;
    case MQIA_MAX_Q_DEPTH                : strcpy(output,"MAXDEPTH"  );break;
    case MQIA_MAX_UNCOMMITTED_MSGS       : strcpy(output,"NUMUNC"    );break;
    case MQIA_MSG_DELIVERY_SEQUENCE      : strcpy(output,"DELSEQ"    );break;
    case MQIA_MSG_DEQ_COUNT              : strcpy(output,"MDC"       );break;
    case MQIA_MSG_ENQ_COUNT              : strcpy(output,"MEC"       );break;
    case MQIA_NAME_COUNT                 : strcpy(output,"NAMEC"     );break;
    case MQIA_OPEN_INPUT_COUNT           : strcpy(output,"IOC"       );break;
    case MQIA_OPEN_OUTPUT_COUNT          : strcpy(output,"OOC"       );break;
    case MQIA_PERFORMANCE_EVENT          : strcpy(output,"PEV"       );break;
    case MQIA_PLATFORM                   : strcpy(output,"PLATFORM"  );break;
    case MQIA_Q_DEPTH_HIGH_EVENT         : strcpy(output,"QDHEV"     );break;
    case MQIA_Q_DEPTH_HIGH_LIMIT         : strcpy(output,"QDHLIM"    );break;
    case MQIA_Q_DEPTH_LOW_EVENT          : strcpy(output,"QDLEV"     );break;
    case MQIA_Q_DEPTH_LOW_LIMIT          : strcpy(output,"QDLLIM"    );break;
    case MQIA_Q_DEPTH_MAX_EVENT          : strcpy(output,"QDMEV"     );break;
    case MQIA_Q_SERVICE_INTERVAL         : strcpy(output,"SI"        );break;
    case MQIA_Q_SERVICE_INTERVAL_EVENT   : strcpy(output,"SIEV"      );break;
    case MQIA_Q_TYPE                     : strcpy(output,"QT"        );break;
    case MQIA_REMOTE_EVENT               : strcpy(output,"REV"       );break;
    case MQIA_RETENTION_INTERVAL         : strcpy(output,"RETINT"    );break;
    case MQIA_SCOPE                      : strcpy(output,"SCOPE"     );break;
    case MQIA_SHAREABILITY               : strcpy(output,"SHARE"     );break;
    case MQIA_START_STOP_EVENT           : strcpy(output,"SSEV"      );break;
    case MQIA_SYNCPOINT                  : strcpy(output,"SYNC"      );break;
    case MQIA_TIME_SINCE_RESET           : strcpy(output,"TSR"       );break;
    case MQIA_TRIGGER_CONTROL            : strcpy(output,"TRIGC"     );break;
    case MQIA_TRIGGER_DEPTH              : strcpy(output,"TRIGD"     );break;
    case MQIA_TRIGGER_INTERVAL           : strcpy(output,"TRIGI"     );break;
    case MQIA_TRIGGER_MSG_PRIORITY       : strcpy(output,"MSGPRI"    );break;
    case MQIA_TRIGGER_TYPE               : strcpy(output,"TRIT"      );break;
    case MQIA_USAGE                      : strcpy(output,"USAGE"     );break;
    case MQIACF_ALL                      : strcpy(output,"ALL"       );break;
    case MQIACF_AUX_ERROR_DATA_INT_1     : strcpy(output,"AED1"      );break;
    case MQIACF_AUX_ERROR_DATA_INT_2     : strcpy(output,"AED2"      );break;
    case MQIACF_CHANNEL_ATTRS            : strcpy(output,"CATTR"     );break;
    case MQIACF_COMMAND                  : strcpy(output,"COM"       );break;
    case MQIACF_CONV_REASON_CODE         : strcpy(output,"CONVRC"    );break;
    case MQIACF_ERROR_ID                 : strcpy(output,"EID"       );break;
//  case MQIACF_ERROR_IDENTIFIER         : strcpy(output,"ERRORID"   );break;
    case MQIACF_ERROR_OFFSET             : strcpy(output,"ERROROF"   );break;
    case MQIACF_ESCAPE_TYPE              : strcpy(output,"ESCTYPE"   );break;
    case MQIACF_FORCE                    : strcpy(output,"FORCE"     );break;
    case MQIACF_OPEN_OPTIONS             : strcpy(output,"OOPTS"     );break;
    case MQIACF_PARAMETER_ID             : strcpy(output,"PARMID"    );break;
    case MQIACF_PROCESS_ATTRS            : strcpy(output,"PROCATTR"  );break;
    case MQIACF_PURGE                    : strcpy(output,"PURGE"     );break;
    case MQIACF_Q_ATTRS                  : strcpy(output,"QATTR"     );break;
    case MQIACF_Q_MGR_ATTRS              : strcpy(output,"QMATTR"    );break;
    case MQIACF_QUIESCE                  : strcpy(output,"QUIESCE"   );break;
    case MQIACF_REASON_QUALIFIER         : strcpy(output,"RQUAL"     );break;
    case MQIACF_REPLACE                  : strcpy(output,"REPLACE"   );break;
    case MQIACF_SELECTOR                 : strcpy(output,"SELECTOR"  );break;
    case MQIACH_BATCH_SIZE               : strcpy(output,"BATCHSZ"   );break;
    case MQIACH_BATCHES                  : strcpy(output,"BATCHES"   );break;
    case MQIACH_BUFFERS_RECEIVED         : strcpy(output,"NBUFSENT"  );break;
    case MQIACH_BUFFERS_SENT             : strcpy(output,"NBUFRECV"  );break;
    case MQIACH_BYTES_RECEIVED           : strcpy(output,"NBYTESENT" );break;
    case MQIACH_BYTES_SENT               : strcpy(output,"NBYTERECV" );break;
    case MQIACH_CHANNEL_ERROR_DATA       : strcpy(output,"CED"       );break;
    case MQIACH_CHANNEL_INSTANCE_ATTRS   : strcpy(output,"CIATTR"    );break;
    case MQIACH_CHANNEL_INSTANCE_TYPE    : strcpy(output,"CITYPE"    );break;
    case MQIACH_CHANNEL_STATUS           : strcpy(output,"CSTATUS"   );break;
    case MQIACH_CHANNEL_TABLE            : strcpy(output,"CTABLE"    );break;
    case MQIACH_CHANNEL_TYPE             : strcpy(output,"CTYPE"     );break;
    case MQIACH_CURRENT_MSGS             : strcpy(output,"CURMSG"    );break;
    case MQIACH_CURRENT_SEQUENCE_NUMBER  : strcpy(output,"CURSEQ"    );break;
    case MQIACH_DATA_CONVERSION          : strcpy(output,"CURDATACV" );break;
    case MQIACH_DATA_COUNT               : strcpy(output,"CURDATAIN" );break;
    case MQIACH_DISC_INTERVAL            : strcpy(output,"CURDISCIN" );break;
    case MQIACH_IN_DOUBT                 : strcpy(output,"IND"       );break;
    case MQIACH_INDOUBT_STATUS           : strcpy(output,"INDSEQ"    );break;
    case MQIACH_LAST_SEQUENCE_NUMBER     : strcpy(output,"LASTSEQ"   );break;
    case MQIACH_LONG_RETRIES_LEFT        : strcpy(output,"LRTRYL"    );break;
    case MQIACH_LONG_RETRY               : strcpy(output,"LTRY"      );break;
    case MQIACH_LONG_TIMER               : strcpy(output,"LTIME"     );break;
    case MQIACH_MAX_MSG_LENGTH           : strcpy(output,"MAXMSGL"   );break;
    case MQIACH_MCA_STATUS               : strcpy(output,"MCAST"     );break;
    case MQIACH_MCA_TYPE                 : strcpy(output,"MCATYPE"   );break;
    case MQIACH_MR_COUNT                 : strcpy(output,"MRCOUNT"   );break;
    case MQIACH_MR_INTERVAL              : strcpy(output,"MRINT"     );break;
    case MQIACH_MSG_SEQUENCE_NUMBER      : strcpy(output,"MSGSEQNO"  );break;
    case MQIACH_MSGS                     : strcpy(output,"MSGS"      );break;
    case MQIACH_PUT_AUTHORITY            : strcpy(output,"PUTAUTH"   );break;
    case MQIACH_SEQUENCE_NUMBER_WRAP     : strcpy(output,"SEQNWRAP"  );break;
    case MQIACH_SHORT_RETRIES_LEFT       : strcpy(output,"SRTRYL"    );break;
    case MQIACH_SHORT_RETRY              : strcpy(output,"SRTRY"     );break;
    case MQIACH_SHORT_TIMER              : strcpy(output,"STIME"     );break;
    case MQIACH_STOP_REQUESTED           : strcpy(output,"STOPREQ"   );break;
    case MQIACH_XMIT_PROTOCOL_TYPE       : strcpy(output,"XPT"       );break;
    case MQIAV_NOT_APPLICABLE            : strcpy(output,"NA"        );break;

    default : sprintf(output,"%lu",pcfnum) ; break ;

   }

  DEBUG("EVENT","Converted " << pcfnum << " into " << output << " " ) ;

  DEBUG("EVENT","Leaving geteventname ") ;

  return ;

 } // End of geteventname function


//
// iscurrthread - tests to see if the Thread the MQCONN was issued upon
//                is the one currently executing.
//
//                Takes a parm of the threadid of the RAHMQOTR object
//                      or a pointer to the RAHMQOTR object itself
//                and returns TRUE or FALSE accordingly
//
//                Traced by the TR setting
//


int iscurrthread ( ULONG mqthread )
 {
  PTIB      ptinfo         ;
  PPIB      ppinfo         ;
  int       rc     = FALSE ;

  DEBUG ("TR", "Entering iscurrthread, testing thread " << mqthread << " " ) ;

  DosGetInfoBlocks(&ptinfo,&ppinfo) ;
  DEBUG("TR", "DosGetInfoBlocks on thread " << ptinfo->tib_ordinal << " " ) ;

  if ( mqthread == ptinfo->tib_ordinal ) rc = TRUE ;

  DEBUG("TR", "Leaving iscurrthread "
               << ((rc == TRUE) ? "Threads match " : "Thread mismatch " ) )  ;

  return rc ;

 } // End of iscurrthread (thread) function


int iscurrthread ( rahmqotr * mqotr )
 {
  PTIB      ptinfo         ;
  PPIB      ppinfo         ;
  int       rc     = FALSE ;

  DEBUG ("TR", "Entering iscurrthread, testing object thread " << mqotr->threadid << " " ) ;

  DosGetInfoBlocks(&ptinfo,&ppinfo) ;
  DEBUG("TR", "DosGetInfoBlocks on thread " << ptinfo->tib_ordinal << " " ) ;

  if ( mqotr->threadid == ptinfo->tib_ordinal ) rc = TRUE ;

  DEBUG("TR", "Leaving iscurrthread "
               << ((rc == TRUE) ? "Threads match " : "Thread mismatch " ) )  ;

  return rc ;

 } // End of iscurrthread (object) function


//
// getthreadid  - returns the current thread number
//
//                Traced by the TR setting
//


ULONG getthreadid ( void )
 {
  PTIB      ptinfo         ;
  PPIB      ppinfo         ;
  ULONG     rc             ;

  DEBUG ("TR", "Entering getthreadid " ) ;

  DosGetInfoBlocks(&ptinfo,&ppinfo)  ;
  rc = ptinfo->tib_ordinal           ;
  DEBUG("TR", "DosGetInfoBlocks rc on thread " << rc << " " ) ;

  DEBUG("TR", "Leaving iscurrthread " ) ;

  return rc ;

 } // End of getthreadid function


//
// isthreadconn - tests to see if the Thread is connected to a Queue Manager
//                and returns FALSE if not, or the handle of the QM to which
//                the thread is connected.
//
//                Traced by the TR setting
//


MQHCONN isthreadconn ( void )
 {
  long      i              ;
  PTIB      ptinfo         ;
  PPIB      ppinfo         ;
  MQHCONN   rc     = FALSE ;

  DEBUG ("TR", "Entering isthreadconn " ) ;

  DosGetInfoBlocks(&ptinfo,&ppinfo) ;
  DEBUG("TR", "DosGetInfoBlocks on thread " << ptinfo->tib_ordinal << " " ) ;

  for ( i=MINQS ; i <= MAXQS ; i++ )
      {
       if ( ptinfo->tib_ordinal == gmqo??(i??).threadid )
          {
            rc = gmqo??(i??).qmh  ;
            DEBUG("TR", "Match found on " << i <<  " " ) ;
            break                 ;
          }
      }

  DEBUG("TR", "Leaving isthreadconn, Thread "
               << ((rc == FALSE) ? "not connected "
                                 : "connected " ) )  ;

  return rc ;

 } // End of isthreadconn function


//
// External Functions, callable from Rexx
//

//
// Initialise the interface - RXMQRINIT
//
//    This function installs the other functions of the DLL.
//    It MUST be called before any of the other Rexx/MQ functions are
//    available for usage.
//
//    This is done by a RxFuncAdd('RXMQRINIT','RXCMQCI',RXMQRINIT')
//         call to let Rexx know about the DLL, and then a
//         rcc = RXMQRINIT() to call the initialisation function.
//
//    The following things are done:
//
//        * Create Rexx variables for ALL known MQ Constants
//        * Create number->name translation variables for MQ ACs
//        * Initialise the DLL Global variables
//            (including obtaining the Mutex for Conn/Open/Close processing)
//        * Make the RXMQR functions known to Rexx
//
//
//

LONG _Export _System  rxmqrinit
                      ( PSZ       afuncname   ,  //Rexx Func name
                        LONG      aargc       ,  //     Number of parms
                        RXSTRING  aargv??(??) ,  //     Parms
                        PSZ       aqname      ,  //     Cur Queue
                        PRXSTRING aretstr     )  //     Return String
 {

 ULONG                    i                ;  // Looper
 unsigned char            returnc??(600??) ;  // Return String
 ULONG                    rc    = 0        ;  // Function interface
 APIRET                   dosrc            ;  // Mutex RC

 //
 // This is a table of functions to be registered with Rexx
 //

 static PCSZ   funcs??(??) = { "RXMQRCONN"   ,  "RXMQRDISC"   ,
                               "RXMQROPEN"   ,  "RXMQRCLOSE"  ,
                               "RXMQRGET"    ,  "RXMQRPUT"    ,
                               "RXMQRINQ"    ,  "RXMQRSET"    ,
                               "RXMQRCMIT"   ,  "RXMQRBACK"   ,
                               "RXMQRBROWSE" ,  "RXMQRHXT"    ,
                               "RXMQRCONS"   ,  "RXMQREVENT"  ,
                               "RXMQRTM"     ,
                               "RXMQRTERM"                    } ;

 static unsigned long numfuncs = sizeof(funcs)/sizeof(char *) ;

 DEBUG ("INIT", "Entering rxmqrinit" ) ;

 //
 // Firstly, initialise all the constants required by calling setcons.
 //
 // Additionally, if the constant represents Reason Code (MQRC_)
 //               then create an number->name entry in the
 //               RXMQR.RCMAP. stem variable
 //

 setcons() ;

 //
 // Initialise any required Globals
 //         -> Create the Mutex to stop multiple access on gmqo
 //

 DEBUG("INIT","Creating the Mutex ") ;

 dosrc = DosCreateMutexSem(0, &mutex, 0, 0 ) ;
 DEBUG("INIT", "Mutex creation rc " << dosrc
               << " Mutex Handle is " << mutex << " " ) ;

 if ( dosrc != 0 )
    {
      sprintf((char *)returnc,"-1 0 0 RXMQRINIT Mutex creation failure rc %lu " , dosrc ) ;
      rc = -1 ;
    }


 // Register the DLL functions

 if ( rc == 0 )
    {
      for (i= 0; i< numfuncs ; i++ )
          {
           rc = RexxRegisterFunctionDll(        funcs??(i??),
                                         (PCSZ) THISDLL,
                                                funcs??(i??)) ;

           DEBUG ("INIT", "Registration of " << funcs??(i??) << " RC " << rc ) ;

          } ;

      strcpy((char *)returnc,"0 0 0 RXMQRINIT OK "
                             "Written by Robert Harris. Version 3.0 "
                             "Copyright IBM UK Labs Ltd., 1995. " ) ;

    } ;


 // Return a (good!) return Code

 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

 DEBUG("INIT", "Leaving rxmqrinit " ) ;
 return 0 ;

 } // End of RXMQRINIT function

//
// RXMQRCONS will simply setp the MQ mappings in the Rexx Workspace.
//
//           This is useful in a thread based environment where RXMQRINIT
//                has been done elsewhere (so setting the mappings elsewhere)
//                to act, essentially, as the initialisation for the thread
//

LONG _Export _System  rxmqrcons
                      ( PSZ       afuncname   ,  //Rexx Func name
                        LONG      aargc       ,  //     Number of parms
                        RXSTRING  aargv??(??) ,  //     Parms
                        PSZ       aqname      ,  //     Cur Queue
                        PRXSTRING aretstr     )  //     Return String
 {

 unsigned char            returnc??(600??) ;  // Return String
 ULONG                    rc               ;  // Function interface

 DEBUG ("INIT", "Entering rxmqrcons" ) ;

 //
 // Just initialise all the constants etc. required by calling setcons.
 //

 setcons() ;

 // Return a good return Code

 strcpy((char *)returnc,"0 0 0 RXMQRCONS OK " ) ;

 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

 DEBUG("INIT", "Leaving rxmqrcons " ) ;
 return 0 ;

 } // End of RXMQRCONS function

//
// Terminate the interface - RXMQRTERM
//
//    This function removes access to the functions of the DLL.
//         After calling, no other Rexx/MQ functions are available.
//
//    However, all the REXX name bindings are left in the
//         Workspace; this is done to provide a method of assembling
//         stem variable with the correct settings.
//
//    NO end of process actions are taken. If a prior Syncpoint is
//         not taken, or the Queue Closed, or the QM Disconnected
//         before this routine is invoked, then the usual MQ
//         End of Process action will occur at that time.
//
//

LONG _Export _System  rxmqrterm
                      ( PSZ       afuncname   ,  //Rexx Func name
                        LONG      aargc       ,  //     Number of parms
                        RXSTRING  aargv??(??) ,  //     Parms
                        PSZ       aqname      ,  //     Cur Queue
                        PRXSTRING aretstr     )  //     Return String
 {

 ULONG                    i                ;  // Looper
 unsigned char            returnc??(600??) ;  // Return String
 ULONG                    rc               ;  // Function interface
 APIRET                   dosrc            ;  // Mutex RC


 //
 // This is a table of functions to be deregistered from Rexx
 //

 static PCSZ   funcs??(??) = { "RXMQRCONN"   ,  "RXMQRDISC"   ,
                               "RXMQROPEN"   ,  "RXMQRCLOSE"  ,
                               "RXMQRGET"    ,  "RXMQRPUT"    ,
                               "RXMQRINQ"    ,  "RXMQRSET"    ,
                               "RXMQRCMIT"   ,  "RXMQRBACK"   ,
                               "RXMQRBROWSE" ,  "RXMQRHXT"    ,
                               "RXMQRCONS"   ,  "RXMQREVENT"  ,
                               "RXMQRTERM"   ,  "RXMQRINIT"   } ;

 static unsigned long numfuncs = sizeof(funcs)/sizeof(char *) ;


 DEBUG ("TERM", "Entering rxmqrterm" ) ;

 // Deregister the DLL functions

 for (i= 0; i< numfuncs ; i++ )
     {
      rc = RexxDeregisterFunction(funcs??(i??)) ;

      DEBUG ("TERM", "Deregistration of " << funcs??(i??) << " RC " << rc ) ;

     } ;


 DEBUG("INIT","Deleting the Mutex ") ;
 dosrc = DosCloseMutexSem(mutex) ;
 DEBUG("INIT", " Mutex deletion rc " << dosrc << " " ) ;
 mutex = 0 ;

 // Return a good return Code

 strcpy((char *)returnc,"0 0 0 RXMQRTERM OK "
                        "Written by Robert Harris. "
                        "Rexx MQ Functions are no longer available " ) ;

 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

 DEBUG("TERM", "Leaving rxmqrterm " ) ;
 return 0 ;

 } // End of RXMQRTERM function

//
// Do a Connect MQCONN
//
//   Call:   rc = rxmqrconn(qmname)
//

LONG _Export _System rxmqrconn
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 LONG                    i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC

 LONG                    reqobj = -1      ;  // Connection Object
 APIRET                  dosrc            ;  // Mutex RC
 HMTX                    gotmutex = 0     ;  // Mutex ownership

 char                    qm??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name

 rc   = 0                    ;
 mqrc = 0                    ;
 mqac = 0                    ;
 memset(qm, 0, sizeof(qm) )  ;

 strcpy((char *)returnc,"-99 0 0 RXMQRCONN UNKNOWN FAILURE" ) ;

 DEBUG("CONN", "Entering rxmqrconn " ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 1 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRCONN Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQRCONN Null QM name" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQRCONN Zero length QM name" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && ( strlen( (const char *)(aargv??(0??).strptr))
                             > MQ_Q_MGR_NAME_LENGTH ) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQRCONN QM name too long" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && ( isthreadconn() != FALSE) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRCONN"
                           " Already Connected to a QM " ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

//
// Now the parms are correct, get them
//

 if (rc == 0)
    {
      memcpy(qm,aargv??(0??).strptr,aargv??(0??).strlength)  ;
      DEBUG("CONN", "Requested Contact with " << qm << " " ) ;
    }

//
// During the Scan and Connect, the Queue Objects are locked
//       Wait for 30 seconds before timing out. This lock lasts OVER
//       the MQCONN call!
//

if ( rc == 0 )
   {
    DEBUG("CONN", "Acquiring the Mutex " << mutex << " " ) ;
    dosrc = DosRequestMutexSem(mutex, MUTEXWAIT) ;
    DEBUG("CONN", "Mutex Acquisition rc " << dosrc << " " ) ;

    if ( dosrc == 0 )
       {
         gotmutex = mutex ;
         DEBUG("CONN", "Now in Critical Section ") ;
       } else
       {
         sprintf((char *)returnc,"-96 0 0 RXMQRCONN Mutex %lu Acquisition failed rc %lu " ,
                         mutex, dosrc ) ;
         rc   = -96 ;
         mqrc =  0  ;
         mqac =  0  ;
       }
   }

//
//
// Find a suitable object to do the Connect upon (ie: unconnected as yet)
//

if ( rc == 0 )
   {
     DEBUG("CONN", "Finding an unused Object ") ;

     for ( i=MINQS; i <= MAXQS ; i++ )
         {
           if ( gmqo??(i??).qmh == 0 )
              {
                DEBUG("CONN", "Empty object at " << i << " located" ) ;
                reqobj = i ;
                break      ;
              }
         }

      if ( reqobj == -1 )
         {
          strcpy((char *)returnc,"-5 0 0 RXMQRCONN"
                                " No available Q Objects" ) ;
          rc   = -5  ;
          mqrc =  0  ;
          mqac =  0  ;
         } ;
   }

//
// Do the MQCONN
//

 if (rc == 0)
    {
      ac   = gmqo??(reqobj??).conn(qm)   ;
      mqrc = gmqo??(reqobj??).rc         ;
      mqac = gmqo??(reqobj??).ac         ;
      rc   = ac                          ;

      sprintf((char *)returnc,"%lu %lu %lu RXMQRCONN %s %s (thread %lu) ",
                      ac, mqrc, mqac, qm,
                      ( (mqrc == MQCC_OK      ) ? "OK"      :
                        (mqrc == MQCC_WARNING ) ? "WARNING" :
                        (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                  "UNKNOWN" ) ,
                      getthreadid()              ) ;

      DEBUG("CONN", "MQCONN " << reqobj
                    << " Handle is "       << gmqo??(reqobj??).qmh
                    << " Threadid is "     << gmqo??(reqobj??).threadid
                    << " QM is "           << gmqo??(reqobj??).qm
                    << " " ) ;

    }

//
// Now Exit the critical section, if within it
//

 if ( gotmutex != 0 )
    {
      DEBUG("CONN", "Releasing the Mutex " << gotmutex << " " ) ;
      dosrc = DosReleaseMutexSem(gotmutex) ;
      DEBUG("CONN", "Mutex Release rc " << dosrc << " " ) ;

      if ( dosrc == 0 )
         {
           DEBUG("CONN", "Now left Critical Section ") ;
         } else
         {
           sprintf((char *)returnc,"-95 0 0 RXMQRCONN Mutex %lu Release failed rc %lu " ,
                           mutex, dosrc ) ;
           rc   = -95 ;
           mqrc =  0  ;
           mqac =  0  ;
         }
    }

//
// Set the LAST variables, and the function return string
//

 stack_and_set(rc,mqrc,mqac,"RXMQRCONN",returnc)       ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("CONN", "Leaving rxmqrconn " ) ;

 return 0 ;

 } // End of RXMQRCONN function


//
// Do an Open   MQOPEN
//
//   Call:   rc = rxmqropen(iodesc,opts,handle,oodesc)
//

LONG _Export _System rxmqropen
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC

 RXSTRING                viodesc          ;  // Stem Var Obj Desc Input
 RXSTRING                voodesc          ;  // Stem Var Obj Desc Output
 RXSTRING                dopts            ;  // Data     Options
 RXSTRING                vhandle          ;  //      Var Obj Handle

 APIRET                  dosrc            ;  // Mutex RC
 HMTX                    gotmutex = 0     ;  // Mutex ownership

 MQHCONN                 qmh              ;  //Connected Queue Manager
 MQOD                    odopen           ;  //MQ object desc
 MQLONG                  options          ;  //MQ open options
 LONG                    theobj     = -1  ;  //gmqo object to use
 unsigned char           theobjc??(600??) ;  //     in char form
 RXSTRING                theobjx          ;  //     as a RXSTRING

 DEBUG("OPEN", "Entering rxmqropen " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;
 memset(&viodesc ,0,sizeof(viodesc) ) ;
 memset(&voodesc ,0,sizeof(voodesc) ) ;
 memset(&dopts   ,0,sizeof(dopts  ) ) ;
 memset(&vhandle ,0,sizeof(vhandle) ) ;


 strcpy((char *)returnc,"-99 0 0 RXMQROPEN UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 4 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQROPEN Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQROPEN Null Input OD/Qname" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQROPEN Zero length Input OD/Qname" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQROPEN Null options" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQROPEN Zero length options" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-6 0 0 RXMQROPEN Null handle name" ) ;
     rc   = -6 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-7 0 0 RXMQROPEN Zero length handle name" ) ;
     rc   = -7 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(3??)) )
    {
     strcpy((char *)returnc,"-8 0 0 RXMQROPEN Null Output OD" ) ;
     rc   = -8 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(3??)) )
    {
     strcpy((char *)returnc,"-9 0 0 RXMQROPEN Zero length Output OD" ) ;
     rc   = -9 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && ( isthreadconn() == FALSE ) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQROPEN "
                           "Not Connected to a QM" ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;


//
// Now the parms are correct, get them
//

 if (rc == 0)
    {
      memcpy(&viodesc ,&aargv??(0??),sizeof(viodesc)) ;
      memcpy(&dopts   ,&aargv??(1??),sizeof(dopts  )) ;
      memcpy(&vhandle ,&aargv??(2??),sizeof(vhandle)) ;
      memcpy(&voodesc ,&aargv??(3??),sizeof(voodesc)) ;

      DEBUG("OPEN", "viodesc " << viodesc.strptr  )   ;
      DEBUG("OPEN", "dopts   " << dopts.strptr    )   ;
      DEBUG("OPEN", "vhandle " << vhandle.strptr  )   ;
      DEBUG("OPEN", "voodesc " << viodesc.strptr  )   ;

      make_od_from_stem(&odopen,viodesc)              ;
      options = atol((const char *)dopts.strptr)      ;

    }

//
// During the Scan and Open, the Queue Objects are locked
//       Wait for 30 seconds before timing out. This lock lasts OVER
//       the MQOPEN call!
//

if ( rc == 0 )
   {
    DEBUG("OPEN", "Acquiring the Mutex " << mutex << " " ) ;
    dosrc = DosRequestMutexSem(mutex, MUTEXWAIT) ;
    DEBUG("OPEN", "Mutex Acquisition rc " << dosrc << " " ) ;

    if ( dosrc == 0 )
       {
         gotmutex = mutex ;
         DEBUG("OPEN", "Now in Critical Section ") ;
       } else
       {
         sprintf((char *)returnc,"-96 0 0 RXMQROPEN Mutex %lu Acquisition failed rc %lu " ,
                         mutex, dosrc ) ;
         rc   = -96 ;
         mqrc =  0  ;
         mqac =  0  ;
       }
   }

//
// Get hold of the QMH for the thread
//          (prior call shows that there is one)
//

 DEBUG("OPEN", "Getting the QMH ")  ;
 qmh = isthreadconn() ;
 DEBUG("OPEN", "Obtained the QMH ") ;

//
// Select the handle (ie: the gmqo to use)
//        Use one with a QMH but no QH, or else an unused one
//

 if (rc == 0)
    {
     for ( i=MINQS ; i <= MAXQS ; i++ )
         {
           if ( ( gmqo??(i??).qmh == qmh ) && ( gmqo??(i??).qh == 0 ) )
              {
                theobj = i ;
                sprintf((char *)theobjc,"%lu",theobj) ;
                DEBUG("OPEN", "Selected QH Object " << theobjc << " " ) ;
                MAKERXSTRING(theobjx,theobjc,
                                     strlen((const char *)theobjc)) ;
                break      ;
              }
         }

     if ( theobj == -1 ) for ( i=MINQS ; i <= MAXQS ; i++ )
         {
           if ( gmqo??(i??).qmh == 0 )
              {
                theobj = i ;
                sprintf((char *)theobjc,"%lu",theobj) ;
                DEBUG("OPEN", "Selected Empty Object " << theobjc << " " ) ;
                MAKERXSTRING(theobjx,theobjc,
                                     strlen((const char *)theobjc)) ;
                break      ;
              }
         }

     if ( (theobj == -1) )
        {
         strcpy((char *)returnc,"-10 0 0 RXMQROPEN "
                               "No available Q objects" ) ;
         rc   = -10 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;

    }

//
// If the Object does not have a QMH, then setup the various entries via a copy
//

 if ( ( rc == 0 ) && ( gmqo??(theobj??).qmh == 0 ) )
    {
      gmqo??(theobj??).qmh      = qmh           ;
      gmqo??(theobj??).threadid = getthreadid() ;
    }

//
// Do the MQOPEN on the obtained Queue object
//

 if (rc == 0)
    {
      ac   = gmqo??(theobj??).open(&odopen,options) ;
      mqrc = gmqo??(theobj??).rc                    ;
      mqac = gmqo??(theobj??).ac                    ;
      rc   = ac                                     ;

      sprintf((char *)returnc,"%lu %lu %lu RXMQROPEN %s",
                      ac, mqrc, mqac,
                      ( (mqrc == MQCC_OK      ) ? "OK"      :
                        (mqrc == MQCC_WARNING ) ? "WARNING" :
                        (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                  "UNKNOWN" ) ) ;

      if ( gmqo??(theobj??).qh != 0 )   //If the Connect worked,
         {                               //then .....
          put_var(vhandle,theobjx) ;     //return the theobj handle num
          make_stem_from_od(&odopen,voodesc) ; //and update the OD
         }

    }

//
// Now Exit the critical section, if within it
//

 if ( gotmutex != 0 )
    {
      DEBUG("OPEN", "Releasing the Mutex " << gotmutex << " " ) ;
      dosrc = DosReleaseMutexSem(gotmutex) ;
      DEBUG("OPEN", "Mutex Release rc " << dosrc << " " ) ;

      if ( dosrc == 0 )
         {
           DEBUG("OPEN", "Now left Critical Section ") ;
         } else
         {
           sprintf((char *)returnc,"-95 0 0 RXMQROPEN Mutex %lu Release failed rc %lu " ,
                           mutex, dosrc ) ;
           rc   = -95 ;
           mqrc =  0  ;
           mqac =  0  ;
         }
    }

//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQROPEN",returnc)       ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("OPEN", "Leaving rxmqropen " ) ;

 return 0 ;

 } // End of RXMQROPEN function

//
// Do a Close   MQCLOSE
//
//   Call:   rc = rxmqrclose(handle,opts)
//

LONG _Export _System rxmqrclose
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC

 RXSTRING                dopts            ;  // Data     Options
 RXSTRING                dhandle          ;  // Data     Obj Handle

 MQLONG                  options          ;  //MQ close options
 MQLONG                  handle           ;  //MQ object number

 APIRET                  dosrc            ;  // Mutex RC
 HMTX                    gotmutex = 0     ;  // Mutex ownership

 DEBUG("CLOSE", "Entering rxmqrclose " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;
 memset(&dopts  ,0,sizeof(dopts  ) ) ;
 memset(&dhandle,0,sizeof(dhandle) ) ;


 strcpy((char *)returnc,"-99 0 0 RXMQRCLOSE UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 2 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRCLOSE Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQRCLOSE Null handle" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQRCLOSE Zero length handle" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQRCLOSE Null options" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQRCLOSE Zero length options" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && ( isthreadconn() == FALSE ) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRCLOSE "
                           "Not Connected to a QM" ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

//
// Now the parms are correct, get them
//

 if (rc == 0)
    {
      memcpy(&dhandle,&aargv??(0??),sizeof(dhandle)) ;
      memcpy(&dopts  ,&aargv??(1??),sizeof(dopts  )) ;

      DEBUG("CLOSE",  "dopts   " << dopts.strptr   ) ;
      DEBUG("CLOSE",  "dhandle " << dhandle.strptr ) ;

      options = atol((const char *)dopts.strptr)     ;
      handle  = atol((const char *)dhandle.strptr)   ;

    }


//
// See if the handle is valid (ie: the gmqo to use)
//

 if (rc == 0)
    {
     if ( ( handle > MAXQS ) || ( handle <= 0 ) )
        {
         strcpy((char *)returnc,"-6 0 0 RXMQRCLOSE "
                               "Handle out of range" ) ;
         rc   = -6 ;
         mqrc =  0 ;
         mqac =  0 ;
        } ;
    }

 if (rc == 0)
    {
     if ( gmqo??(handle??).qh == 0 )
        {
         strcpy((char *)returnc,"-7 0 0 RXMQRCLOSE "
                               "Invalid handle" ) ;
         rc   = -7 ;
         mqrc =  0 ;
         mqac =  0 ;
        } ;
    }

//
// The Handle points to a valid object, see if it is owned by the thread
//

if (rc == 0)
    {
     if ( iscurrthread(&gmqo??(handle??)) == FALSE )
        {
         strcpy((char *)returnc,"-97 0 0 RXMQRCLOSE "
                               "Handle not owned by current thread " ) ;
         rc   = -97 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

//
// During the Close, the Queue Objects are locked
//       Wait for 30 seconds before timing out. This lock lasts OVER
//       the MQCLOSE call!
//

if ( rc == 0 )
   {
    DEBUG("CLOSE", "Acquiring the Mutex " << mutex << " " ) ;
    dosrc = DosRequestMutexSem(mutex, MUTEXWAIT) ;
    DEBUG("CLOSE", "Mutex Acquisition rc " << dosrc << " " ) ;

    if ( dosrc == 0 )
       {
         gotmutex = mutex ;
         DEBUG("CLOSE", "Now in Critical Section ") ;
       } else
       {
         sprintf((char *)returnc,"-96 0 0 RXMQRCLOSE Mutex %lu Acquisition failed rc %lu " ,
                         mutex, dosrc ) ;
         rc   = -96 ;
         mqrc =  0  ;
         mqac =  0  ;
       }
   }

//
// Now close the object
//

 if (rc == 0)
    {
      ac   = gmqo??(handle??).close(options)        ;
      mqrc = gmqo??(handle??).rc                    ;
      mqac = gmqo??(handle??).ac                    ;
      rc   = ac                                     ;

      sprintf((char *)returnc,"%lu %lu %lu RXMQRCLOSE %s",
                      ac, mqrc, mqac,
                      ( (mqrc == MQCC_OK      ) ? "OK"      :
                        (mqrc == MQCC_WARNING ) ? "WARNING" :
                        (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                  "UNKNOWN" ) ) ;

      if ( mqac == 0 )                   //If the Close worked,
         {                               //then .....
          gmqo??(handle??).qh = 0 ;      //loose the MQ object
         }

    }

//
// Now Exit the critical section, if within it
//

 if ( gotmutex != 0 )
    {
      DEBUG("CLOSE", "Releasing the Mutex " << gotmutex << " " ) ;
      dosrc = DosReleaseMutexSem(gotmutex) ;
      DEBUG("CLOSE", "Mutex Release rc " << dosrc << " " ) ;

      if ( dosrc == 0 )
         {
           DEBUG("CLOSE", "Now left Critical Section ") ;
         } else
         {
           sprintf((char *)returnc,"-95 0 0 RXMQRCLOSE Mutex %lu Release failed rc %lu " ,
                           mutex, dosrc ) ;
           rc   = -95 ;
           mqrc =  0  ;
           mqac =  0  ;
         }
    }

//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRCLOSE",returnc)      ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("CLOSE", "Leaving rxmqrclose " ) ;

 return 0 ;

 } // End of RXMQRCLOSE function

//
// Do a Disconnect   MQDISC
//
//   Call:   rc = rxmqrdisc()
//

LONG _Export _System rxmqrdisc
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC
 ULONG                   firstdisc = 0    ;  // First time disc flag

 APIRET                  dosrc            ;  // Mutex RC
 HMTX                    gotmutex = 0     ;  // Mutex ownership

 DEBUG("DISC", "Entering rxmqrdisc " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;


 strcpy((char *)returnc,"-99 0 0 RXMQRDISC UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 0 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRDISC Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && ( isthreadconn() == FALSE ) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRDISC "
                           "Not Connected to a QM" ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

//
// During the Close & Disc, the Queue Objects are locked
//       Wait for 30 seconds before timing out. This lock lasts OVER
//       the MQCLOSE & MQDISC calls!
//

if ( rc == 0 )
   {
    DEBUG("DISC", "Acquiring the Mutex " << mutex << " " ) ;
    dosrc = DosRequestMutexSem(mutex, MUTEXWAIT) ;
    DEBUG("DISC", "Mutex Acquisition rc " << dosrc << " " ) ;

    if ( dosrc == 0 )
       {
         gotmutex = mutex ;
         DEBUG("DISC", "Now in Critical Section ") ;
       } else
       {
         sprintf((char *)returnc,"-96 0 0 RXMQRDISC Mutex %lu Acquisition failed rc %lu " ,
                         mutex, dosrc ) ;
         rc   = -96 ;
         mqrc =  0  ;
         mqac =  0  ;
       }
   }

//
// Just to be nice, loop through all the open MQ objects and
//         close them before doing the DISC. Do not take any
//         notice of these Return Codes, as they are all supposed
//         to be shut anyway.
//

 if (rc == 0)
    {
     for ( i=MINQS ; i <= MAXQS ; i++ ) // Close Phase
         {
           if ( (gmqo??(i??).qh != 0 ) && ( iscurrthread(&gmqo??(i??)) == TRUE ) )
              {
                gmqo??(i??).close(MQCO_NONE) ;
                gmqo??(i??).qh = 0           ;
              }
         }

     for ( i=MINQS ; i <= MAXQS ; i++ )  // Disc Phase
         {
           if ( (gmqo??(i??).qmh != 0 ) && ( iscurrthread(&gmqo??(i??)) == TRUE ) )
              {
                gmqo??(i??).disc()         ;
                gmqo??(i??).qmh = 0        ;
                gmqo??(i??).threadid = 0   ;

                if ( firstdisc == 0 )
                   {
                     firstdisc = i         ;
                     mqrc = gmqo??(i??).rc ;
                     mqac = gmqo??(i??).ac ;
                     ac   = mqac           ;
                     rc   = ac             ;

                     sprintf((char *)returnc,"%lu %lu %lu RXMQRDISC %s",
                                     ac, mqrc, mqac,
                                     ( (mqrc == MQCC_OK      ) ? "OK"      :
                                       (mqrc == MQCC_WARNING ) ? "WARNING" :
                                       (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                                 "UNKNOWN" ) ) ;
                   }

              }
         }

     for ( i=MINQS ; i <= MAXQS ; i++ ) // Zap phase
         {
           if ( (gmqo??(i??).qmh != 0 ) && ( iscurrthread(&gmqo??(i??)) == TRUE ) )
              {
                gmqo??(i??).qmh = 0        ;
                gmqo??(i??).threadid = 0   ;
              }
         }
    }

//
// Now Exit the critical section, if within it
//

 if ( gotmutex != 0 )
    {
      DEBUG("DISC", "Releasing the Mutex " << gotmutex << " " ) ;
      dosrc = DosReleaseMutexSem(gotmutex) ;
      DEBUG("DISC", "Mutex Release rc " << dosrc << " " ) ;

      if ( dosrc == 0 )
         {
           DEBUG("DISC", "Now left Critical Section ") ;
         } else
         {
           sprintf((char *)returnc,"-95 0 0 RXMQRDISC Mutex %lu Release failed rc %lu " ,
                           mutex, dosrc ) ;
           rc   = -95 ;
           mqrc =  0  ;
           mqac =  0  ;
         }
    }

//
// Set the LAST variables, and the function return string
//

 stack_and_set(rc,mqrc,mqac,"RXMQRDISC",returnc)       ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("DISC", "Leaving rxmqrdisc " ) ;

 return 0 ;

 } // End of RXMQRDISC function

//
// Do a Syncpoint    MQCMIT
//
//   Call:   rc = rxmqrcmit()
//

LONG _Export _System rxmqrcmit
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC

 DEBUG("CMIT", "Entering rxmqrcmit " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;


 strcpy((char *)returnc,"-99 0 0 RXMQRCMIT UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 0 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRCMIT Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && ( isthreadconn() == FALSE ) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRCMIT "
                           "Not Connected to a QM" ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

//
// Do the Actual Syncpoint on the required Queue Manager
//        This also forces it for all others on the thread
//

 if (rc == 0)
    {
     for ( i=MINQS ; i <= MAXQS ; i++ )
         {
           if ( (gmqo??(i??).qh != 0 ) && ( iscurrthread(&gmqo??(i??)) == TRUE ) )
              {
                DEBUG("CMIT", "Doing the Syncpoint on object " << i << " " ) ;
                ac   = gmqo??(i??).commit();
                mqrc = gmqo??(i??).rc      ;
                mqac = gmqo??(i??).ac      ;
                rc   = ac                  ;

                sprintf((char *)returnc,"%lu %lu %lu RXMQRCMIT %s",
                                ac, mqrc, mqac,
                                ( (mqrc == MQCC_OK      ) ? "OK"      :
                                  (mqrc == MQCC_WARNING ) ? "WARNING" :
                                  (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                            "UNKNOWN" ) ) ;
                break ; // Only bother with the first one
              }
         }
    }

//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRCMIT",returnc)       ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("CMIT", "Leaving rxmqrcmit " ) ;

 return 0 ;

 } // End of RXMQRCMIT function

//
// Do a Rollback     MQBACK
//
//   Call:   rc = rxmqrback()
//

LONG _Export _System rxmqrback
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC

 DEBUG("BACK", "Entering rxmqrback " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;


 strcpy((char *)returnc,"-99 0 0 RXMQRBACK UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 0 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRBACK Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && ( isthreadconn() == FALSE ) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRBACK "
                           "Not Connected to a QM" ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

//
// Do the Actual Backout on the required Queue Manager
//        This also forces it for all others on the thread
//

 if (rc == 0)
    {
     for ( i=MINQS ; i <= MAXQS ; i++ )
         {
           if ( (gmqo??(i??).qh != 0 ) && ( iscurrthread(&gmqo??(i??)) == TRUE ) )
              {
                DEBUG("BACK", "Doing the Backout on object " << i << " " ) ;
                ac   = gmqo??(i??).rollback()  ;
                mqrc = gmqo??(i??).rc          ;
                mqac = gmqo??(i??).ac          ;
                rc   = ac                      ;

                sprintf((char *)returnc,"%lu %lu %lu RXMQRBACK %s",
                                ac, mqrc, mqac,
                                ( (mqrc == MQCC_OK      ) ? "OK"      :
                                  (mqrc == MQCC_WARNING ) ? "WARNING" :
                                  (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                            "UNKNOWN" ) ) ;
                break ; // Only bother for the first one
              }
         }
     }

//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRBACK",returnc)       ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("BACK", "Leaving rxmqrback " ) ;

 return 0 ;

 } // End of RXMQRBACK function

//
// Do a Put     MQPUT
//
//   Call:   rc = rxmqrput(handle, data,
//                                 input_msgdesc,output_msgdesc,
//                                 input_pmo,output_pmo)
//

LONG _Export _System rxmqrput
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC

 RXSTRING                vdata            ;  // Variable Data
 RXSTRING                vides            ;  // Variable Input  Msg Desc
 RXSTRING                vodes            ;  // Variable Output Msg Desc
 RXSTRING                vipmo            ;  // Variable Input  PMO
 RXSTRING                vopmo            ;  // Variable Output PMO
 RXSTRING                dhandle          ;  // Data     Obj Handle

 MQLONG                  data0            ;  // Variable Data len
 RXSTRING                data1 = {0,0}    ;  // Variable Data stuff
 APIRET                  dosrc            ;  // Freemain return Code

 MQLONG                  handle           ;  //MQ object number
 MQMD                    desc             ;  //MQ Message descriptor
 MQPMO                   pmo              ;  //MQ Put Message options
 char                 *  data             ;  //-> Data buffer
 unsigned long           datalen          ;  //   Data length

 DEBUG("PUT", "Entering rxmqrput " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;
 memset(&desc   ,0,sizeof(desc)    ) ;
 memset(&pmo    ,0,sizeof(pmo )    ) ;
 memset(&dhandle,0,sizeof(dhandle) ) ;


 strcpy((char *)returnc,"-99 0 0 RXMQRPUT UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 6 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRPUT Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQRPUT Null handle" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQRPUT Zero data handle" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQRPUT Null data stem var" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQRPUT Zero data stem var" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-6 0 0 RXMQRPUT Null Input MsgDesc" ) ;
     rc   = -64 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-7 0 0 RXMQRPUT Zero length Input MsgDesc" ) ;
     rc   = -7 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(3??)) )
    {
     strcpy((char *)returnc,"-8 0 0 RXMQRPUT Null Output MsgDesc" ) ;
     rc   = -8 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(3??)) )
    {
     strcpy((char *)returnc,"-9 0 0 RXMQRPUT Zero length Output MsgDesc" ) ;
     rc   = -9 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(4??)) )
    {
     strcpy((char *)returnc,"-10 0 0 RXMQRPUT Null input PMO" ) ;
     rc   = -10 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(4??)) )
    {
     strcpy((char *)returnc,"-11 0 0 RXMQRPUT Zero length input PMO" ) ;
     rc   = -11 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(5??)) )
    {
     strcpy((char *)returnc,"-12 0 0 RXMQRPUT Null output PMO" ) ;
     rc   = -12 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(5??)) )
    {
     strcpy((char *)returnc,"-13 0 0 RXMQRPUT Zero length output PMO" ) ;
     rc   = -13 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

 if ( (rc == 0) && ( isthreadconn() == FALSE ) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRPUT "
                           "Not Connected to a QM" ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

//
// Now the parms are correct, get them
//

 if (rc == 0)
    {
      memcpy(&dhandle,&aargv??(0??),sizeof(dhandle)) ;
      memcpy(&vdata,&aargv??(1??),sizeof(vdata)) ;
      memcpy(&vides,&aargv??(2??),sizeof(vides)) ;
      memcpy(&vodes,&aargv??(3??),sizeof(vodes)) ;
      memcpy(&vipmo,&aargv??(4??),sizeof(vipmo)) ;
      memcpy(&vopmo,&aargv??(5??),sizeof(vopmo)) ;

      DEBUG("PUT",  "dhandle " << dhandle.strptr ) ;
      DEBUG("PUT",  "vdata   " << vdata.strptr   ) ;
      DEBUG("PUT",  "vides   " << vides.strptr   ) ;
      DEBUG("PUT",  "vodes   " << vodes.strptr   ) ;
      DEBUG("PUT",  "vimpo   " << vipmo.strptr   ) ;
      DEBUG("PUT",  "vopmo   " << vopmo.strptr   ) ;

      handle  = atol((const char *)dhandle.strptr)   ;
      make_md_from_stem(&desc, vides ) ;
      make_po_from_stem(&pmo , vipmo ) ;
      get_var0(vdata, &data0 )        ;
      get_var1(vdata, &data1 )        ;

      DEBUG("PUT", "Value at " << &data1
                               << " Length " << data1.strlength
                               << " Data <"  << data1.strptr
                               << "> " ) ;

    }


//
// See if the handle is valid (ie: the gmqo to use)
//

 if (rc == 0)
    {
     if ( ( handle > MAXQS ) || ( handle <= 0 ) )
        {
         strcpy((char *)returnc,"-14 0 0 RXMQRPUT "
                                "Handle out of range" ) ;
         rc   = -14 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

 if (rc == 0)
    {
     if ( gmqo??(handle??).qh == 0 )
        {
         strcpy((char *)returnc,"-15 0 0 RXMQRPUT "
                                "Invalid handle" ) ;
         rc   = -15 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

//
// The Handle points to a valid object, see if it is owned by the thread
//

if (rc == 0)
    {
     if ( iscurrthread(&gmqo??(handle??)) == FALSE )
        {
         strcpy((char *)returnc,"-97 0 0 RXMQRPUT "
                               "Handle not owned by current thread " ) ;
         rc   = -97 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }


//
// Now put the data to the queue
//

 if (rc == 0)
    {
      DEBUG("PUT", "PUT Maxdatalen is " << data0 << " ") ;

      ac   = gmqo??(handle??).put(&desc,&pmo,
                                        data0,(char *)data1.strptr)    ;
      mqrc = gmqo??(handle??).rc                                       ;
      mqac = gmqo??(handle??).ac                                       ;
      rc   = ac                                                        ;

      sprintf((char *)returnc,"%lu %lu %lu RXMQRPUT %s",
                      ac, mqrc, mqac,
                      ( (mqrc == MQCC_OK      ) ? "OK"      :
                        (mqrc == MQCC_WARNING ) ? "WARNING" :
                        (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                  "UNKNOWN" ) ) ;

      DEBUG("PUT", "PUT RC " << mqrc << " AC " << mqac << " " ) ;

      make_stem_from_md(&desc, vodes ) ;   //Set the return Variables
      make_stem_from_po(&pmo , vopmo ) ;


    }

//
// Freemain the RXSTRING for the data area if it was allocated
//

if ( data1.strptr != 0 )
   {
    DEBUG("PUT", "Freemaining area " << data1.strptr ) ;
    dosrc = DosFreeMem(data1.strptr) ;
    DEBUG("PUT", "Freemaining RC = " << dosrc ) ;
    data1.strptr = 0 ;
   }

//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRPUT",returnc)        ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("PUT", "Leaving rxmqrput " ) ;

 return 0 ;

 } // End of RXMQRPUT function

//
// Do a Get     MQGET
//
//   Call:   rc = rxmqrget(handle, data,
//                                 input_msgdesc,output_msgdesc,
//                                 input_gmo,output_gmo)
//

LONG _Export _System rxmqrget
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC
 APIRET                  dosrc            ;  // Dos Getmain/Freemain rc

 RXSTRING                vdata            ;  // Variable Data
 RXSTRING                vides            ;  // Variable Input  Msg Desc
 RXSTRING                vodes            ;  // Variable Output Msg Desc
 RXSTRING                vigmo            ;  // Variable Input  GMO
 RXSTRING                vogmo            ;  // Variable Output GMO
 RXSTRING                dhandle          ;  // Data     Obj Handle

 MQLONG                  data0        = 0 ;  // Variable Data len
 RXSTRING                data1            ;  // Variable Data stuff

 MQLONG                  handle       = 0 ;  //MQ object number
 MQMD                    desc             ;  //MQ Message descriptor
 MQGMO                   gmo              ;  //MQ Get Message options
 void                 *  buffer       = 0 ;  //-> Data buffer
 MQLONG                  bufflen      = 0 ;  //   Data length max
 MQLONG                  buffcurlen   = 0 ;  //   Data length actual

 DEBUG("GET", "Entering rxmqrget " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;
 memset(&desc   ,0,sizeof(desc)    ) ;
 memset(&gmo    ,0,sizeof(gmo )    ) ;
 memset(&dhandle,0,sizeof(dhandle) ) ;


 strcpy((char *)returnc,"-99 0 0 RXMQRGET UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 6 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRGET Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQRGET Null handle" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQRGET Zero data handle" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQRGET Null data stem var" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQRGET Zero data stem var" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-6 0 0 RXMQRGET Null Input MsgDesc" ) ;
     rc   = -6 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-7 0 0 RXMQRGET Zero length Input MsgDesc" ) ;
     rc   = -7 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(3??)) )
    {
     strcpy((char *)returnc,"-8 0 0 RXMQRGET Null Output MsgDesc" ) ;
     rc   = -8 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(3??)) )
    {
     strcpy((char *)returnc,"-9 0 0 RXMQRGET Zero length Output MsgDesc" ) ;
     rc   = -9 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(4??)) )
    {
     strcpy((char *)returnc,"-10 0 0 RXMQRGET Null input GMO" ) ;
     rc   = -10 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(4??)) )
    {
     strcpy((char *)returnc,"-11 0 0 RXMQRGET Zero length input GMO" ) ;
     rc   = -11 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(5??)) )
    {
     strcpy((char *)returnc,"-12 0 0 RXMQRGET Null output GMO" ) ;
     rc   = -12 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(5??)) )
    {
     strcpy((char *)returnc,"-13 0 0 RXMQRGET Zero length output GMO" ) ;
     rc   = -13 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

 if ( (rc == 0) && ( isthreadconn() == FALSE ) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRGET "
                           "Not Connected to a QM" ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

//
// Now the parms are correct, get them
//

 if (rc == 0)
    {
      memcpy(&dhandle,&aargv??(0??),sizeof(dhandle)) ;
      memcpy(&vdata,&aargv??(1??),sizeof(vdata)) ;
      memcpy(&vides,&aargv??(2??),sizeof(vides)) ;
      memcpy(&vodes,&aargv??(3??),sizeof(vodes)) ;
      memcpy(&vigmo,&aargv??(4??),sizeof(vigmo)) ;
      memcpy(&vogmo,&aargv??(5??),sizeof(vogmo)) ;

      DEBUG("GET",  "dhandle " << dhandle.strptr ) ;
      DEBUG("GET",  "vdata   " << vdata.strptr   ) ;
      DEBUG("GET",  "vides   " << vides.strptr   ) ;
      DEBUG("GET",  "vodes   " << vodes.strptr   ) ;
      DEBUG("GET",  "vigmo   " << vigmo.strptr   ) ;
      DEBUG("GET",  "vogmo   " << vogmo.strptr   ) ;


      handle  = atol((const char *)dhandle.strptr)   ;
      make_md_from_stem(&desc, vides ) ;
      make_go_from_stem(&gmo , vigmo ) ;
      get_var0(vdata, &data0 )         ;
      bufflen    = data0               ;

    }


//
// See if the handle is valid (ie: the gmqo to use)
//

 if (rc == 0)
    {
     if ( ( handle > MAXQS ) || ( handle <= 0 ) )
        {
         strcpy((char *)returnc,"-14 0 0 RXMQRGET "
                                "Handle out of range" ) ;
         rc   = -14 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

 if (rc == 0)
    {
     if ( gmqo??(handle??).qh == 0 )
        {
         strcpy((char *)returnc,"-15 0 0 RXMQRGET "
                                "Invalid handle" ) ;
         rc   = -15 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

//
// The Handle points to a valid object, see if it is owned by the thread
//

if (rc == 0)
    {
     if ( iscurrthread(&gmqo??(handle??)) == FALSE )
        {
         strcpy((char *)returnc,"-97 0 0 RXMQRGET "
                               "Handle not owned by current thread " ) ;
         rc   = -97 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

//
// Now GETMAIN the buffer to receive the data records
//

 if (( rc == 0) && ( bufflen != 0 ) )
    {
     DEBUG("GET", "Doing DOS Getmain for " << bufflen << " bytes ") ;
     dosrc = DosAllocMem(&buffer,bufflen,PAG_READ|PAG_WRITE|PAG_COMMIT) ;
     DEBUG("GET","Dos Getmain RC " << dosrc << " PTR is "<<hex<< buffer);

     if ( dosrc != 0 )
        {
         sprintf((char *)returnc,"-16 0 0 RXMQRGET "
                                 "DosAllocMem failure, RC(%lu) " ,
                                  dosrc ) ;
         rc   = -16 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }


//
// Now get the data from the queue
//

 if (rc == 0)
    {
      DEBUG("GET", "GET Maxdatalen is " << bufflen << " ") ;

      ac   = gmqo??(handle??).get(&desc, &gmo,
                                         bufflen,
                                         (char *)buffer,
                                         &buffcurlen)         ;
      mqrc = gmqo??(handle??).rc                              ;
      mqac = gmqo??(handle??).ac                              ;
      rc   = ac                                               ;

      sprintf((char *)returnc,"%lu %lu %lu RXMQRGET %s",
                      ac, mqrc, mqac,
                      ( (mqrc == MQCC_OK      ) ? "OK"      :
                        (mqrc == MQCC_WARNING ) ? "WARNING" :
                        (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                  "UNKNOWN" ) ) ;

      DEBUG("GET", "GET RC " << mqrc << " AC " << mqac << " Datalen " << buffcurlen << " " ) ;

      make_stem_from_md(&desc, vodes  ) ;  //Set the return Variables
      make_stem_from_go(&gmo , vogmo          ) ;
      put_var0(vdata,buffcurlen               ) ;
      put_var1(vdata,buffcurlen,(char *)buffer) ;

    }

//
// Now FREEMAIN the buffer if it was allocated
//

// if (buffer != 0)
    {
     DEBUG("GET", "Dos Freemaining " ) ;
     dosrc = DosFreeMem(buffer) ;
     DEBUG("GET", "Dos Freemain RC " << dosrc ) ;
     buffer = 0 ;
    }

//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRGET",returnc)        ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("GET", "Leaving rxmqrget " ) ;

 return 0 ;

 } // End of RXMQRGET function

//
// Do an inquire MQINQ
//
//   Call:   rc = rxmqrinq(handle, attribute, atttrsetting )
//
//           NB: only 1 attribute at a time!
//
//

LONG _Export _System rxmqrinq
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG            i                    ;  // Looper
 unsigned char    returnc??(600??)     ;  // Return String
 LONG             rc                   ;  // Function Return Code
 MQLONG           ac                   ;  // MQ instance Return Code
 MQLONG           mqrc                 ;  // MQ RC
 MQLONG           mqac                 ;  // MQ AC

 RXSTRING         viattr               ;  // Variable Input  Attr
 RXSTRING         voattr               ;  // Variable Output Attr
 RXSTRING         dhandle              ;  // Data     Obj Handle

 MQLONG           handle       = 0     ;  //MQ object number

 unsigned char    voattrres??(600??)   ;  // Return Attribute
 MQLONG           voattrresl           ;  // Return Attribute length
 RXSTRING         voattrresr           ;  // Variable Output RXSTRING

 MQLONG           curattr      = 0     ;  // Current Attr

 MQLONG           inqselcount  = 1     ;  //INQ - number of sels
 MQLONG           inqsels      = 0     ;  //INQ - the selectors
 MQLONG           inqselicount = 0     ;  //INQ - number of int sels
 MQLONG           inqints      = 0     ;  //INQ - integer return
 MQLONG           inqcharlen   = 100   ;  //INQ - char return length
 unsigned char    inqchars??(100??)    ;  //INQ - char return

 DEBUG("INQ", "Entering rxmqrinq " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;
 memset(&viattr ,0,sizeof(viattr)  ) ;
 memset(&voattr ,0,sizeof(voattr)  ) ;
 memset(&dhandle,0,sizeof(dhandle) ) ;


 strcpy((char *)returnc,"-99 0 0 RXMQRINQ UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 3 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRINQ Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQRINQ Null handle" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQRINQ Zero data handle" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQRINQ Null data input attr" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQRINQ Zero data input attr" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-6 0 0 RXMQRINQ Null output attr" ) ;
     rc   = -6 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-7 0 0 RXMQRINQ Zero length output attr") ;
     rc   = -7 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && ( isthreadconn() == FALSE ) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRINQ "
                           "Not Connected to a QM" ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

//
// Now the parms are correct, get them
//

 if (rc == 0)
    {
      memcpy(&dhandle,&aargv??(0??),sizeof(dhandle)) ;
      memcpy(&viattr,&aargv??(1??),sizeof(viattr)) ;
      memcpy(&voattr,&aargv??(2??),sizeof(voattr)) ;

      DEBUG("INQ",  "dhandle " << dhandle.strptr ) ;
      DEBUG("INQ",  "viattr  " << viattr.strptr  ) ;
      DEBUG("INQ",  "voattr  " << voattr.strptr  ) ;


      handle  = atol((const char *)dhandle.strptr) ;
      curattr = atol((const char *)viattr.strptr)  ;

      if (curattr == 0)
         {
           strcpy((char *)returnc,"-8 0 0 RXMQRINQ "
                                 "No attribute supplied" ) ;
           rc   = -8 ;
           mqrc =  0 ;
           mqac =  0 ;
         }

      if ( !(    ( (curattr >= MQIA_FIRST  ) && (curattr <= MQIA_LAST        ) )
              || ( (curattr >= MQCA_FIRST  ) && (curattr <= MQCA_LAST        ) )
              || ( (curattr >= MQIACF_FIRST) && (curattr <= MQIACF_LAST_USED ) )
              || ( (curattr >= MQIACH_FIRST) && (curattr <= MQIACH_LAST_USED ) )
              || ( (curattr >= MQCACF_FIRST) && (curattr <= MQCACF_LAST_USED ) )
              || ( (curattr >= MQCACH_FIRST) && (curattr <= MQCACH_LAST_USED ) ) ) )
         {
           strcpy((char *)returnc,"-9 0 0 RXMQRINQ "
                                 "Attribute out of valid range" ) ;
           rc   = -9  ;
           mqrc =  0  ;
           mqac =  0  ;
         }

    }

//
// See if the handle is valid (ie: the gmqo to use)
//

 if (rc == 0)
    {
     if ( ( handle > MAXQS ) || ( handle <= 0 ) )
        {
         strcpy((char *)returnc,"-10 0 0 RXMQRINQ "
                                "Handle out of range" ) ;
         rc   = -10 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

 if (rc == 0)
    {
     if ( gmqo??(handle??).qh == 0 )
        {
         strcpy((char *)returnc,"-11 0 0 RXMQRINQ "
                                "Invalid handle" ) ;
         rc   = -11 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

//
// The Handle points to a valid object, see if it is owned by the thread
//

if (rc == 0)
    {
     if ( iscurrthread(&gmqo??(handle??)) == FALSE )
        {
         strcpy((char *)returnc,"-97 0 0 RXMQRINQ "
                               "Handle not owned by current thread " ) ;
         rc   = -97 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

//
// Format up either a character or an integer area
//

 if (rc == 0)
    {
      if (    ( (curattr >= MQCA_FIRST  ) && (curattr <= MQCA_LAST       ) )
           || ( (curattr >= MQCACF_FIRST) && (curattr <= MQCACF_LAST_USED) )
           || ( (curattr >= MQCACH_FIRST) && (curattr <= MQCACH_LAST_USED) ) )
         {
           inqselcount  = 1                    ;
           inqsels      = curattr              ;
           inqselicount = 0                    ;
           inqints      = 0                    ;
           inqcharlen   = 100                  ;
           memset(inqchars,0,sizeof(inqchars)) ;
           switch ( curattr )
            {
               case MQCA_Q_DESC                     :  inqcharlen = MQ_Q_DESC_LENGTH            ; break;
               case MQCA_Q_MGR_DESC                 :  inqcharlen = MQ_Q_MGR_DESC_LENGTH        ; break;
               case MQCA_Q_MGR_NAME                 :  inqcharlen = MQ_Q_MGR_NAME_LENGTH        ; break;
               case MQCA_Q_NAME                     :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCA_BACKOUT_REQ_Q_NAME         :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCA_BASE_Q_NAME                :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCA_NAMES                      :  inqcharlen = inqcharlen                  ; break;
               case MQCA_DEAD_LETTER_Q_NAME         :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCA_DEF_XMIT_Q_NAME            :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCA_REMOTE_Q_MGR_NAME          :  inqcharlen = MQ_Q_MGR_NAME_LENGTH        ; break;
               case MQCA_REMOTE_Q_NAME              :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCA_XMIT_Q_NAME                :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCA_ENV_DATA                   :  inqcharlen = MQ_ENV_DATA_LENGTH          ; break;
               case MQCA_INITIATION_Q_NAME          :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCA_COMMAND_INPUT_Q_NAME       :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCA_APPL_ID                    :  inqcharlen = MQ_APPL_ID_LENGTH           ; break;
               case MQCA_CREATION_DATE              :  inqcharlen = MQ_CREATION_DATE_LENGTH     ; break;
               case MQCA_CREATION_TIME              :  inqcharlen = MQ_CREATION_TIME_LENGTH     ; break;
               case MQCA_PROCESS_DESC               :  inqcharlen = MQ_PROCESS_DESC_LENGTH      ; break;
               case MQCA_PROCESS_NAME               :  inqcharlen = MQ_PROCESS_NAME_LENGTH      ; break;
               case MQCA_TRIGGER_DATA               :  inqcharlen = MQ_TRIGGER_DATA_LENGTH      ; break;
               case MQCA_USER_DATA                  :  inqcharlen = MQ_USER_DATA_LENGTH         ; break;
               case MQCA_STORAGE_CLASS              :  inqcharlen = MQ_STORAGE_CLASS_LENGTH     ; break;
               case MQCACF_ALIAS_Q_NAMES            :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_APPL_NAME                :  inqcharlen = MQ_APPL_NAME_LENGTH         ; break;
               case MQCACF_AUX_ERROR_DATA_STR_1     :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_AUX_ERROR_DATA_STR_2     :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_AUX_ERROR_DATA_STR_3     :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_ESCAPE_TEXT              :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_FROM_CHANNEL_NAME        :  inqcharlen = MQ_CHANNEL_NAME_LENGTH      ; break;
               case MQCACF_FROM_PROCESS_NAME        :  inqcharlen = MQ_PROCESS_NAME_LENGTH      ; break;
               case MQCACF_FROM_Q_NAME              :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;
               case MQCACF_LOCAL_Q_NAMES            :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_MODEL_Q_NAMES            :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_OBJECT_Q_MGR_NAME        :  inqcharlen = MQ_Q_MGR_NAME_LENGTH        ; break;
               case MQCACF_PROCESS_NAMES            :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_Q_NAMES                  :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_RECEIVER_CHANNEL_NAMES   :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_REMOTE_Q_NAMES           :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_REQUESTER_CHANNEL_NAMES  :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_SENDER_CHANNEL_NAMES     :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_SERVER_CHANNEL_NAMES     :  inqcharlen = inqcharlen                  ; break;
               case MQCACF_TO_CHANNEL_NAME          :  inqcharlen = MQ_CHANNEL_NAME_LENGTH      ; break;
               case MQCACF_TO_PROCESS_NAME          :  inqcharlen = MQ_PROCESS_NAME_LENGTH      ; break;
               case MQCACF_TO_Q_NAME                :  inqcharlen = MQ_TO_Q_NAME_LENGTH         ; break;
               case MQCACF_USER_IDENTIFIER          :  inqcharlen = MQ_USER_ID_LENGTH           ; break;
               case MQCACH_CHANNEL_NAME             :  inqcharlen = MQ_CHANNEL_NAME_LENGTH      ; break;
               case MQCACH_CHANNEL_NAMES            :  inqcharlen = inqcharlen                  ; break;
               case MQCACH_CHANNEL_START_DATE       :  inqcharlen = MQ_CHANNEL_DATE_LENGTH      ; break;
               case MQCACH_CHANNEL_START_TIME       :  inqcharlen = MQ_CHANNEL_TIME_LENGTH      ; break;
               case MQCACH_CONNECTION_NAME          :  inqcharlen = MQ_CONN_NAME_LENGTH         ; break;
               case MQCACH_CURRENT_LUWID            :  inqcharlen = MQ_LUWID_LENGTH             ; break;
               case MQCACH_DESC                     :  inqcharlen = MQ_CHANNEL_DESC_LENGTH      ; break;
               case MQCACH_FORMAT_NAME              :  inqcharlen = MQ_FORMAT_LENGTH            ; break;
               case MQCACH_LAST_LUWID               :  inqcharlen = MQ_LUWID_LENGTH             ; break;
               case MQCACH_LAST_MSG_DATE            :  inqcharlen = MQ_CHANNEL_DATE_LENGTH      ; break;
               case MQCACH_LAST_MSG_TIME            :  inqcharlen = MQ_CHANNEL_TIME_LENGTH      ; break;
               case MQCACH_MCA_JOB_NAME             :  inqcharlen = MQ_MCA_JOB_NAME_LENGTH      ; break;
               case MQCACH_MCA_NAME                 :  inqcharlen = MQ_MCA_NAME_LENGTH          ; break;
               case MQCACH_MCA_USER_ID              :  inqcharlen = MQ_USER_ID_LENGTH           ; break;
               case MQCACH_MODE_NAME                :  inqcharlen = MQ_MODE_NAME_LENGTH         ; break;
               case MQCACH_MR_EXIT_NAME             :  inqcharlen = MQ_EXIT_NAME_LENGTH         ; break;
               case MQCACH_MR_EXIT_USER_DATA        :  inqcharlen = MQ_EXIT_USER_DATA_LENGTH    ; break;
               case MQCACH_MSG_EXIT_NAME            :  inqcharlen = MQ_EXIT_NAME_LENGTH         ; break;
               case MQCACH_MSG_EXIT_USER_DATA       :  inqcharlen = MQ_EXIT_USER_DATA_LENGTH    ; break;
               case MQCACH_PASSWORD                 :  inqcharlen = MQ_PASSWORD_LENGTH          ; break;
               case MQCACH_RCV_EXIT_NAME            :  inqcharlen = MQ_EXIT_NAME_LENGTH         ; break;
               case MQCACH_RCV_EXIT_USER_DATA       :  inqcharlen = MQ_EXIT_USER_DATA_LENGTH    ; break;
               case MQCACH_SEC_EXIT_NAME            :  inqcharlen = MQ_EXIT_NAME_LENGTH         ; break;
               case MQCACH_SEC_EXIT_USER_DATA       :  inqcharlen = MQ_EXIT_USER_DATA_LENGTH    ; break;
               case MQCACH_SEND_EXIT_NAME           :  inqcharlen = MQ_EXIT_NAME_LENGTH         ; break;
               case MQCACH_SEND_EXIT_USER_DATA      :  inqcharlen = MQ_EXIT_USER_DATA_LENGTH    ; break;
               case MQCACH_TP_NAME                  :  inqcharlen = MQ_TP_NAME_LENGTH           ; break;
               case MQCACH_USER_ID                  :  inqcharlen = MQ_USER_ID_LENGTH           ; break;
               case MQCACH_XMIT_Q_NAME              :  inqcharlen = MQ_Q_NAME_LENGTH            ; break;

              default     :
                     strcpy((char *)returnc,"-12 0 0 RXMQRINQ "
                          "Invalid Char Attribute" ) ;
                     rc   = -12 ;
                     mqrc =  0  ;
                     mqac =  0  ;
                     break      ;
            }
         }      //End of Char Attr processing
           else //Int attr processing
         {
           inqselcount  = 1                    ;
           inqsels      = curattr              ;
           inqselicount = 1                    ;
           inqints      = 0                    ;
           inqcharlen   = 0                    ;
           memset(inqchars,0,sizeof(inqchars)) ;
         }      //End of Int Attr processing

    } //End of Attribute building


//
// Now do the Inquiry and return the setting
//

 if (rc == 0)
    {

      DEBUG("INQ", "Attr " << inqsels << " IntSelNum " << inqselicount
                                      << " ChrAtrLen " << inqcharlen   );


      ac   = gmqo??(handle??).inq(inqselcount , inqsels           ,
                                  inqselicount, &inqints          ,
                                  inqcharlen  , (char *)inqchars) ;
      mqrc = gmqo??(handle??).rc                                  ;
      mqac = gmqo??(handle??).ac                                  ;
      rc   = ac                                                   ;

      sprintf((char *)returnc,"%lu %lu %lu RXMQRINQ %s",
                      ac, mqrc, mqac,
                      ( (mqrc == MQCC_OK      ) ? "OK"      :
                        (mqrc == MQCC_WARNING ) ? "WARNING" :
                        (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                  "UNKNOWN" ) ) ;

      DEBUG("INQ", "INQ RC " << mqrc << " AC " << mqac << " Intval " << inqints << " Charval " << inqchars ) ;

      if (    ( (curattr >= MQCA_FIRST  ) && (curattr <= MQCA_LAST)        )
           || ( (curattr >= MQCACF_FIRST) && (curattr <= MQCACF_LAST_USED) )
           || ( (curattr >= MQCACH_FIRST) && (curattr <= MQCACH_LAST_USED) ) )
         {
           memset(&voattrres,0,sizeof(voattrres))           ;
           memcpy(&voattrres,&inqchars,inqcharlen)          ;
           voattrresl = inqcharlen                          ;
           MAKERXSTRING(voattrresr,voattrres,voattrresl)    ;
           put_var(voattr, voattrresr)                      ;
         } else
         {
           memset(&voattrres,0,sizeof(voattrres))           ;
           sprintf((char *)voattrres,"%lu",inqints)         ;
           voattrresl = strlen((const char *)voattrres)     ;
           MAKERXSTRING(voattrresr,voattrres,voattrresl)    ;
           put_var(voattr, voattrresr)                      ;
         }

    }


//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRINQ",returnc)        ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("INQ", "Leaving rxmqrinq " ) ;

 return 0 ;

 } // End of RXMQRINQ function

//
// Do a set      MQSET
//
//   Call:   rc = rxmqrset(handle, attribute, atttrsetting )
//
//           NB: only 1 attribute at a time!
//
//

LONG _Export _System rxmqrset
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG            i                    ;  // Looper
 unsigned char    returnc??(600??)     ;  // Return String
 LONG             rc                   ;  // Function Return Code
 MQLONG           ac                   ;  // MQ instance Return Code
 MQLONG           mqrc                 ;  // MQ RC
 MQLONG           mqac                 ;  // MQ AC

 RXSTRING         viattr               ;  // Variable Input  Attr
 RXSTRING         vsattr               ;  // Variable Set    Attr
 RXSTRING         dhandle              ;  // Data     Obj Handle

 MQLONG           handle       = 0     ;  //MQ object number

 MQLONG           number               ;  // Char->Integer Conver

 MQLONG           curattr      = 0     ;  // Current Attr

 MQLONG           setselcount  = 1     ;  //SET - number of sels
 MQLONG           setsels      = 0     ;  //SET - the selectors
 MQLONG           setselicount = 0     ;  //SET - number of int sels
 MQLONG           setints      = 0     ;  //SET - integer return
 MQLONG           setcharlen   = 100   ;  //SET - char return length
 unsigned char    setchars??(100??)    ;  //SET - char return

 DEBUG("SET", "Entering rxmqrset " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;
 memset(&viattr ,0,sizeof(viattr)  ) ;
 memset(&vsattr ,0,sizeof(vsattr)  ) ;
 memset(&dhandle,0,sizeof(dhandle) ) ;


 strcpy((char *)returnc,"-99 0 0 RXMQRSET UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 3 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRSET Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQRSET Null handle" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQRSET Zero data handle" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQRSET Null data attribute" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQRSET Zero data attribute" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-6 0 0 RXMQRSET Null setting" ) ;
     rc   = -6 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(2??)) )
    {
     strcpy((char *)returnc,"-7 0 0 RXMQRSET Zero length setting") ;
     rc   = -7 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && ( isthreadconn() == FALSE ) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRSET "
                           "Not Connected to a QM" ) ;
     rc   = -98 ;
     mqrc =  0  ;
     mqac =  0  ;
    } ;

//
// Now the parms are correct, get them
//

 if (rc == 0)
    {
      memcpy(&dhandle,&aargv??(0??),sizeof(dhandle)) ;
      memcpy(&viattr,&aargv??(1??),sizeof(viattr)) ;
      memcpy(&vsattr,&aargv??(2??),sizeof(vsattr)) ;

      DEBUG("SET",  "dhandle " << dhandle.strptr ) ;
      DEBUG("SET",  "viattr  " << viattr.strptr  ) ;
      DEBUG("SET",  "vsattr  " << vsattr.strptr  ) ;

      handle  = atol((const char *)dhandle.strptr) ;
      curattr = atol((const char *)viattr.strptr ) ;

      DEBUG("SET", "Value at " << &vsattr
                               << " Length " << vsattr.strlength
                               << " Data <"  << vsattr.strptr
                               << "> " ) ;


      if (curattr == 0)
         {
           strcpy((char *)returnc,"-8 0 0 RXMQRSET "
                                 "No attribute supplied" ) ;
           rc   = -8 ;
           mqrc =  0 ;
           mqac =  0 ;
         }

      if ( !(    ( (curattr >= MQIA_FIRST  ) && (curattr <= MQIA_LAST        ) )
              || ( (curattr >= MQCA_FIRST  ) && (curattr <= MQCA_LAST        ) )
              || ( (curattr >= MQIACF_FIRST) && (curattr <= MQIACF_LAST_USED ) )
              || ( (curattr >= MQIACH_FIRST) && (curattr <= MQIACH_LAST_USED ) )
              || ( (curattr >= MQCACF_FIRST) && (curattr <= MQCACF_LAST_USED ) )
              || ( (curattr >= MQCACH_FIRST) && (curattr <= MQCACH_LAST_USED ) ) ) )
         {
           strcpy((char *)returnc,"-9 0 0 RXMQRSET "
                                 "Attribute out of valid range" ) ;
           rc   = -9  ;
           mqrc =  0  ;
           mqac =  0  ;
         }

    }

//
// See if the handle is valid (ie: the gmqo to use)
//

 if (rc == 0)
    {
     if ( ( handle > MAXQS ) || ( handle <= 0 ) )
        {
         strcpy((char *)returnc,"-10 0 0 RXMQRSET "
                                "Handle out of range" ) ;
         rc   = -10 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

 if (rc == 0)
    {
     if ( gmqo??(handle??).qh == 0 )
        {
         strcpy((char *)returnc,"-11 0 0 RXMQRSET "
                                "Invalid handle" ) ;
         rc   = -11 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

//
// The Handle points to a valid object, see if it is owned by the thread
//

if (rc == 0)
    {
     if ( iscurrthread(&gmqo??(handle??)) == FALSE )
        {
         strcpy((char *)returnc,"-97 0 0 RXMQRSET "
                               "Handle not owned by current thread " ) ;
         rc   = -97 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

//
// Format up either a character or an integer area
//

 if (rc == 0)
    {
      if (    ( (curattr >= MQCA_FIRST  ) && (curattr <= MQCA_LAST)        )
           || ( (curattr >= MQCACF_FIRST) && (curattr <= MQCACF_LAST_USED) )
           || ( (curattr >= MQCACH_FIRST) && (curattr <= MQCACH_LAST_USED) ) )
         {
           setselcount  = 1                    ;
           setsels      = curattr              ;
           setselicount = 0                    ;
           setints      = 0                    ;
           setcharlen   = 100                  ;
           memset(setchars,0,sizeof(setchars)) ;
           switch ( curattr )
            {
             case MQCA_TRIGGER_DATA         : setcharlen = MQ_TRIGGER_DATA_LENGTH      ; break ;

              default     :
                     strcpy((char *)returnc,"-13 0 0 RXMQRSET "
                          "Invalid Char Attribute" ) ;
                     rc   = -13 ;
                     mqrc =  0  ;
                     mqac =  0  ;
                     break      ;
            }

           memcpy(setchars,vsattr.strptr,setcharlen)          ;

         }      //End of Char Attr processing
           else //Int attr processing
         {
           setselcount  = 1                                   ;
           setsels      = curattr                             ;
           setselicount = 1                                   ;
           setints      = 0                                   ;
           setcharlen   = 0                                   ;
           memset(setchars,0,sizeof(setchars))                ;

           number       = atol((const char *)vsattr.strptr)   ;
           setints      = number                              ;

         }      //End of Int Attr processing

    } //End of Attribute building


//
// Now do the Setting
//

 if (rc == 0)
    {

      DEBUG("SET", "Attr " << setsels
                           << " IntSelNum " << setselicount
                           << " IntSetVal " << setints
                           << " ChrAtrLen " << setcharlen
                           << " ChrSetVar " << setchars     );


      ac   = gmqo??(handle??).set(setselcount , setsels           ,
                                  setselicount, &setints          ,
                                  setcharlen  , (char *)setchars) ;
      mqrc = gmqo??(handle??).rc                                  ;
      mqac = gmqo??(handle??).ac                                  ;
      rc   = ac                                                   ;

      sprintf((char *)returnc,"%lu %lu %lu RXMQRSET %s",
                      ac, mqrc, mqac,
                      ( (mqrc == MQCC_OK      ) ? "OK"      :
                        (mqrc == MQCC_WARNING ) ? "WARNING" :
                        (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                  "UNKNOWN" ) ) ;

      DEBUG("SET", "SET RC " << mqrc << " AC " << mqac ) ;


    }


//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRSET",returnc)        ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("SET", "Leaving rxmqrset " ) ;

 return 0 ;

 } // End of RXMQRSET function

//
// Extension functions
//

//
// Do a Browse
//
//   Call:   rc = rxmqrbrowse(handle, data)
//
//           This function issues a browse on the Queue, and
//                returns the next message (if available). If you
//                need the Message descriptor as well, then use the
//                native Get interface.
//
//

LONG _Export _System rxmqrbrowse
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC
 APIRET                  dosrc            ;  // Dos Getmain/Freemain rc

 RXSTRING                vdata            ;  // Variable Data
 RXSTRING                dhandle          ;  // Data     Obj Handle

 MQLONG                  data0        = 0 ;  // Variable Data len
 RXSTRING                data1            ;  // Variable Data stuff

 MQLONG                  handle       = 0 ;  //MQ object number
 void                 *  buffer       = 0 ;  //-> Data buffer
 MQLONG                  bufflen      = 0 ;  //   Data length max
 MQLONG                  buffcurlen   = 0 ;  //   Data length actual

 DEBUG("BRO", "Entering rxmqrbrowse " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;
 memset(&dhandle,0,sizeof(dhandle) ) ;
 memset(&vdata  ,0,sizeof(vdata  ) ) ;


 strcpy((char *)returnc,"-99 0 0 RXMQRBROWSE UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 2 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRBROWSE Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQRBROWSE Null handle" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQRBROWSE Zero data handle" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQRBROWSE Null data stem var" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQRBROWSE Zero data stem var" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;


 if ( (rc == 0) && ( isthreadconn() == FALSE) )
    {
     strcpy((char *)returnc,"-98 0 0 RXMQRBROWSE "
                           "Not Connected to a QM" ) ;
     rc   = -98  ;
     mqrc =  0   ;
     mqac =  0   ;
    } ;

//
// Now the parms are correct, get them
//

 if (rc == 0)
    {
      memcpy(&dhandle,&aargv??(0??),sizeof(dhandle)) ;
      memcpy(&vdata,&aargv??(1??),sizeof(vdata)) ;

      DEBUG("BRO",  "dhandle " << dhandle.strptr ) ;
      DEBUG("BRO",  "vdata   " << vdata.strptr   ) ;


      handle  = atol((const char *)dhandle.strptr)   ;
      get_var0(vdata, &data0 )         ;
      bufflen    = data0               ;

    }


//
// See if the handle is valid (ie: the gmqo to use)
//

 if (rc == 0)
    {
     if ( ( handle > MAXQS ) || ( handle <= 0 ) )
        {
         strcpy((char *)returnc,"-6 0 0 RXMQRBROWSE "
                                "Handle out of range" ) ;
         rc   = -6  ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

 if (rc == 0)
    {
     if ( gmqo??(handle??).qh == 0 )
        {
         strcpy((char *)returnc,"-7 0 0 RXMQRBROWSE "
                                "Invalid handle" ) ;
         rc   = -7  ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }

//
// The Handle points to a valid object, see if it is owned by the thread
//

if (rc == 0)
    {
     if ( iscurrthread(&gmqo??(handle??)) == FALSE )
        {
         strcpy((char *)returnc,"-97 0 0 RXMQBROWSE "
                               "Handle not owned by current thread " ) ;
         rc   = -97 ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }


//
// Now GETMAIN the buffer to receive the data records
//

 if (( rc == 0) && ( bufflen != 0 ) )
    {
     DEBUG("BRO", "Doing DOS Getmain for " << bufflen << " bytes ") ;
     dosrc = DosAllocMem(&buffer,bufflen,PAG_READ|PAG_WRITE|PAG_COMMIT) ;
     DEBUG("BRO","Dos Getmain RC " << dosrc << " PTR is "<<hex<< buffer);

     if ( dosrc != 0 )
        {
         sprintf((char *)returnc,"-8 0 0 RXMQRBROWSE "
                                 "DosAllocMem failure, RC(%lu) " ,
                                  dosrc ) ;
         rc   = -8  ;
         mqrc =  0  ;
         mqac =  0  ;
        } ;
    }


//
// Now get the data from the queue
//

 if (rc == 0)
    {
      DEBUG("BRO", "BROWSE Maxdatalen is " << bufflen << " ") ;

      ac   = gmqo??(handle??).browse(bufflen,
                                     (char *)buffer,
                                     &buffcurlen)         ;
      mqrc = gmqo??(handle??).rc                          ;
      mqac = gmqo??(handle??).ac                          ;
      rc   = ac                                           ;

      sprintf((char *)returnc,"%lu %lu %lu RXMQRBROWSE %s",
                      ac, mqrc, mqac,
                      ( (mqrc == MQCC_OK      ) ? "OK"      :
                        (mqrc == MQCC_WARNING ) ? "WARNING" :
                        (mqrc == MQCC_FAILED  ) ? "FAILED"  :
                                                  "UNKNOWN" ) ) ;

      DEBUG("BRO", "GET RC " << mqrc << " AC " << mqac << " Datalen " << buffcurlen << " " ) ;

      put_var0(vdata,buffcurlen               ) ; //Set Data returns
      put_var1(vdata,buffcurlen,(char *)buffer) ;

    }

//
// Now FREEMAIN the buffer if it was allocated
//

// if (buffer != 0)
    {
     DEBUG("BRO", "Dos Freemaining " ) ;
     dosrc = DosFreeMem(buffer) ;
     DEBUG("BRO", "Dos Freemain RC " << dosrc ) ;
     buffer = 0 ;
    }

//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRBROWSE",returnc)        ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("BRO", "Leaving rxmqrbrowse " ) ;

 return 0 ;

 } // End of RXMQRBROWSE function

//
// Do a Header extract  MQHXT
//
//   Call:   rc = rxmqrhxt( input_Stem, output_stem )
//
//
//
//   This function takes an input Stem variable, and builds an
//        output stem variable, extracting the Headers from the
//        'message' data.
//
//        The actual message & its length are placed in .1 and .0 , with
//            all the other info being expanded into the .item variables.
//            In addition, for compatability with the EVENT splitting up
//            capability .NAME and .TYPE are also set (both to either
//            DLH or XQH). .ZLIST is also set to contain the exploded
//            components (including TYPE NAME 0 1).
//
//
//
//   The headers processed are:
//
//       XQH - Transmission Header
//
//             .0      -> Data Length
//             .1      -> Message Data
//
//             .RQM    -> Remote Queue Manager Name
//             .RQN    -> Remote Queue Name
//
//             .TYPE   -> XQH      for compatability with
//             .NAME   -> XQH      Event splitting up
//
//             + all the MD options
//
//             .ZLIST  -> 0 1 AID AOD AT BC CID CCSI ENC EXP FBK
//                        FORM MSG MSGID NAME PAN PAT PD PER PRI
//                        PT REP RQM RQN RTOQ RTOQM TYPE UID
//
//
//       DLH - Dead Letter Header
//
//             .0      -> Data Length
//             .1      -> Message Data
//
//             .TYPE   -> DLH      for compatability with
//             .NAME   -> DLH      Event splitting up
//
//             .REA    -> Reason
//             .DQM    -> Destination Queue Manager Name
//             .DQN    -> Destination Queue Name
//             .ENC    -> Encoding
//             .CCSI   -> Coded Character Set ID
//             .FORM   -> Format
//             .PAT    -> Put Appl Type
//             .PAN    -> Put Appl Name
//             .PD     -> Put Date
//             .PT     -> Put Time
//
//             .ZLIST  -> 0 1 CCSI DQM DQN ENC FORM NAME PAN
//                        PAT PD PT REA TYPE
//
//

LONG _Export _System rxmqrhxt
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC

 char                    header??(5??)    ;  // Header Type

 MQXQH                 * thexqh           ;  // -> XQH
 MQDLH                 * thedlh           ;  // -> DLH


 RXSTRING                vinp             ;  // Variable Data - Input
 MQLONG                  inp0             ;  // Input Data - len
 RXSTRING                inp1  = {0,0}    ;  // Input Data - data

 RXSTRING                vout             ;  // Variable Data - Output
 MQLONG                  out0             ;  // Output Data - len
 RXSTRING                out1  = {0,0}    ;  // Output Data - data

 char                    type??(6??)      ;  // Type of the Structure (XQH DLH)
 char                    name??(6??)      ;  // Name of the Structure (XQH DLH)

 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(600??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(600??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver



 DEBUG("HXT", "Entering rxmqrhxt " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;


 strcpy((char *)returnc,"-99 0 0 RXMQRHXT UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 2 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRHXT Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQRHXT Null input stem var" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQRHXT Zero input stem var" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQRHXT Null output stem var" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQRHXT Zero output stem var" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

//
// The input parms are OK, so obtain them
//

 if (rc == 0)
    {
      memcpy(&vinp,&aargv??(0??),sizeof(vinp)) ;
      memcpy(&vout,&aargv??(1??),sizeof(vout)) ;

      get_var0(vinp, &inp0 )        ;
      get_var1(vinp, &inp1 )        ;
    }

//
// Now check the input Stem variable to see that there is
//     some valid data to obtain
//

 if ( (rc == 0) && ( inp0 == 0 ) )
    {
      strcpy((char *)returnc,"-6 0 0 RXMQRHXT No input data" ) ;
      rc   = -6 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

 if ( (rc == 0) && ( inp1.strlength == 0 ) )
    {
      strcpy((char *)returnc,"-7 0 0 RXMQRHXT Zero input data" ) ;
      rc   = -7 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

 if ( (rc == 0) && ( inp0 <= 3 ) )
    {
      strcpy((char *)returnc,"-8 0 0 RXMQRHXT Cannot locate Header" );
      rc   = -8 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

 if ( (rc == 0) && ( inp1.strlength <= 3 ) )
    {
      strcpy((char *)returnc,"-9 0 0 RXMQRHXT Cannot find Header" ) ;
      rc   = -9 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

//
// There is the possibility of a Header, so see if it is a known
//       one, and ignore it it is not one to process.
//

 if ( rc == 0 )
    {
      memset(header, 0, sizeof(header)                           ) ;
      strncpy(header, (const char *)inp1.strptr, sizeof(MQCHAR4) ) ;

      if (    ( strcmp(header, MQDLH_STRUC_ID) != 0 )
           && ( strcmp(header, MQXQH_STRUC_ID) != 0 ) )
         {
           strcpy((char *)returnc,"-10 0 0 RXMQRHXT Unknown Header" ) ;
           rc   = -10 ;
           mqrc =  0  ;
           mqac =  0  ;
         } ;
    }


//
// Although we have a valid header, just check the lengths
//          to ensure the WHOLE header is present
//

if ( rc == 0 )
   {
     if (    ( strcmp(header, MQDLH_STRUC_ID) == 0 )
          && ( inp0 < sizeof(MQDLH)                ) )
        {
          sprintf((char *)returnc,"-11 0 0 RXMQRHXT "
                                  "Too short for a DLH (%lu bytes!) " ,
                                   inp0 ) ;
          rc   = -11 ;
          mqrc =  0  ;
          mqac =  0  ;
        } ;

     if (    ( strcmp(header, MQXQH_STRUC_ID) == 0 )
          && ( inp0 < sizeof(MQXQH)                ) )
        {
          sprintf((char *)returnc,"-12 0 0 RXMQRHXT "
                                  "Too short for a XQH (%lu bytes!) " ,
                                   inp0 ) ;
          rc   = -12 ;
          mqrc =  0  ;
          mqac =  0  ;
        } ;

   }


//
// Now we have got a valid header, split it up
//

if ( ( rc == 0 ) && ( strcmp(header, MQDLH_STRUC_ID) == 0 ) )
   {
     DEBUG("HXT", "Unravelling a DLH"  ) ;

     strcpy(type,"DLH")            ; // Set Type and Name info
     strcpy(name,"DLH")            ; //     for EVENT compatability

     thedlh = (MQDLH *)inp1.strptr ; // Set Header pointer

     out0 = inp0 - sizeof(MQDLH)   ; // Calc actual Data length
     out1.strlength = out0         ; // Set real data pointer
     out1.strptr    = inp1.strptr + sizeof(MQDLH) ;

     put_var0(vout, out0                        ) ; //Set Data returns
     put_var1(vout, out1.strlength, out1.strptr ) ;

     DEBUG( "HXT",    " Msg  length " << inp0
                   << " DLH  length " << (long)sizeof(MQDLH)
                   << " Data length " << out1.strlength
                   << " " ) ;

     DEBUG( "HXT",    " Data area "     << hex << (void *)inp1.strptr
                   << " Start of DLH "  << thedlh
                   << " Start of data " << hex << (void *)out1.strptr
                   << " <"              << hex << out1.strptr
                   << "> " ) ;

     sprintf((char *)varvalc,"%lu",thedlh->Reason)                     ;
     sprintf((char *)varnamc,"%sREA",vout.strptr)                      ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;


     sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),thedlh->DestQMgrName);
     sprintf((char *)varnamc,"%sDQM",vout.strptr)                      ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),thedlh->DestQName);
     sprintf((char *)varnamc,"%sDQN",vout.strptr)                      ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"%lu",thedlh->Encoding)                   ;
     sprintf((char *)varnamc,"%sENC",vout.strptr)                      ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;


     sprintf((char *)varvalc,"%lu",thedlh->CodedCharSetId)             ;
     sprintf((char *)varnamc,"%sCCSI",vout.strptr)                     ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;


     sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR8),thedlh->Format)    ;
     sprintf((char *)varnamc,"%sFORM",vout.strptr)                     ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"%lu",thedlh->PutApplType)                ;
     sprintf((char *)varnamc,"%sPAT",vout.strptr)                      ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR28),thedlh->PutApplName);
     sprintf((char *)varnamc,"%sPAN",vout.strptr)                      ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;


     sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR8),thedlh->PutDate)   ;
     sprintf((char *)varnamc,"%sPD",vout.strptr)                       ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;


     sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR8),thedlh->PutTime)   ;
     sprintf((char *)varnamc,"%sPT",vout.strptr)                       ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"%s",type)                                ;
     sprintf((char *)varnamc,"%sTYPE",vout.strptr)                     ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"%s",name)                                ;
     sprintf((char *)varnamc,"%sNAME",vout.strptr)                     ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"0 1 CCSI DQM DQN ENC FORM NAME PAN "
                             "PAT PD PT REA TYPE")                     ;
     sprintf((char *)varnamc,"%sZLIST",vout.strptr)                    ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     DEBUG("HXT", "Unravelled the DLH" ) ;
   }

if ( ( rc == 0 ) && ( strcmp(header, MQXQH_STRUC_ID) == 0 ) )
   {


     DEBUG("HXT", "Unravelling a XQH"  ) ;

     strcpy(type,"XQH")            ; // Set Type and Name info
     strcpy(name,"XQH")            ; //     for EVENT compatability

     thexqh = (MQXQH *)inp1.strptr ; // Set Header pointer

     out0 = inp0 - sizeof(MQXQH)   ; // Calc actual Data length
     out1.strlength = out0         ; // Set real data pointer
     out1.strptr    = inp1.strptr + sizeof(MQXQH) ;

     put_var0(vout, out0                        ) ; //Set Data returns
     put_var1(vout, out1.strlength, out1.strptr ) ;

     DEBUG( "HXT",    " Msg  length " << inp0
                   << " XQH  length " << (long)sizeof(MQXQH)
                   << " Data length " << out1.strlength
                   << " " ) ;

     DEBUG( "HXT",    " Data area "     << hex << (void *)inp1.strptr
                   << " Start of XQH "  << thexqh
                   << " Start of data " << hex << (void *)out1.strptr
                   << " <"              << hex << out1.strptr
                   << "> " ) ;


     sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),thexqh->RemoteQMgrName);
     sprintf((char *)varnamc,"%sRQM",vout.strptr)                      ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),thexqh->RemoteQName);
     sprintf((char *)varnamc,"%sRQN",vout.strptr)                      ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"%s",type)                                ;
     sprintf((char *)varnamc,"%sTYPE",vout.strptr)                     ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     sprintf((char *)varvalc,"%s",name)                                ;
     sprintf((char *)varnamc,"%sNAME",vout.strptr)                     ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     DEBUG("HXT", "Generating the XQH.MD" ) ;
     make_stem_from_md(&thexqh->MsgDesc, vout ) ; // Set the MD
     DEBUG("HXT", "End of XQH.MD generation") ;

     //Got to do this after the MD to overwrite the variable!
     sprintf((char *)varvalc,"0 1 AID AOD AT BC CID CCSI ENC EXP FBK "
                             "FORM MSG MSGID NAME PAN PAT PD PER PRI "
                             "PT REP RQM RQN RTOQ RTOQM TYPE UID" ) ;
     sprintf((char *)varnamc,"%sZLIST",vout.strptr)                    ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("HXT", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     DEBUG("HXT", "Unravelled the XQH" ) ;
   }

//
// Set a Good return Code if apt
//

 if ( rc == 0 )
    {
      strcpy((char *)returnc,"0 0 0 RXMQRHXT OK" ) ;
      rc   =  0 ;
      mqrc =  0 ;
      mqac =  0 ;
    }


//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRHXT",returnc)       ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("HXT", "Leaving rxmqrhxt " ) ;

 return 0 ;

 } // End of RXMQRHXT function

//
// Do an Event extract  RXMQREVENT
//
//   Call:   rc = rxmqrevent( input_Stem, output_stem )
//
//
//
//   This function takes an input Stem variable, and builds an
//        output stem variable, extracting the Event Data from the
//        'message' data.
//
//        All the info is expanded into .item variables according to the
//            table below. In addition, .TYPE is set to 'EVENT' and     .
//            .NAME is the name of the event.
//
//        Additionally (.)ZLIST is set to contain a list of all the
//            components (without the leading dot).
//
//
//   The events processed and their names are :
//
//       MQRC_Q_MGR_ACTIVE                -> QMACT
//       MQRC_Q_MGR_NOT_ACTIVE            -> QMINA
//       MQRC_GET_INHIBITED               -> INGET
//       MQRC_PUT_INHIBITED               -> INPUT
//       MQRC_ALIAS_BASE_Q_TYPE_ERROR     -> LLAQT
//       MQRC_UNKNOWN_ALIAS_BASE_Q        -> LLABQ
//       MQRC_UNKNOWN_OBJECT_NAME         -> LLUON
//       MQRC_CHANNEL_CONV_ERROR          -> CHCONV
//       MQRC_CHANNEL_STARTED             -> CHSTRT
//       MQRC_CHANNEL_STOPPED             -> CHSTOP
//       MQRC_CHANNEL_ACTIVATED           -> CHACT
//       MQRC_CHANNEL_NOT_ACTIVATED       -> CHNACT
//       MQRC_Q_DEPTH_HIGH                -> PFQDH
//       MQRC_Q_DEPTH_LOW                 -> PFQDL
//       MQRC_Q_FULL                      -> PFQFU
//       MQRC_Q_SERVICE_INTERVAL_HIGH     -> PFQSH
//       MQRC_Q_SERVICE_INTERVAL_OK       -> PFQSO
//       MQRC_DEF_XMIT_Q_TYPE_ERROR       -> RMDXQT
//       MQRC_DEF_XMIT_Q_USAGE_ERROR      -> RMDXQU
//       MQRC_Q_TYPE_ERROR                -> RMQUTY
//       MQRC_REMOTE_Q_NAME_ERROR         -> RMRQNA
//       MQRC_XMIT_Q_TYPE_ERROR           -> RMXQTY
//       MQRC_XMIT_Q_USAGE_ERROR          -> RMXQUS
//       MQRC_UNKNOWN_DEF_XMIT_Q          -> RMUDXQ
//       MQRC_UNKNOWN_REMOTE_Q_MGR        -> RMURQM
//       MQRC_UNKNOWN_XMIT_Q              -> RRUXQN
//       MQRC_NOT_AUTHORIZED              -> NAAUT
//
//
//     The events are splitup into the normal stem.component name, but
//                if the component is not a SINGLE string or Integer (as
//                defined by the IN & ST PCF types for the given field), then
//                the explosion is stem.component.0 for the number of items
//                                 stem.component.n for the individual items
//
//     The naming of the fields within the event is controlled by the
//                geteventname routine, which converts a known set of
//                fields (as defined by the Event part of the PCF book)
//                into character component names. Note that wherever the
//                field occurs generates its expansion - no matter whether
//                or not the PCF book say it should or should not occur!
//

LONG _Export _System rxmqrevent
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 ULONG                   j                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC

 RXSTRING                vinp             ;  // Variable Data - Input
 MQLONG                  inp0             ;  // Input Data - len
 RXSTRING                inp1  = {0,0}    ;  // Input Data - data

 RXSTRING                vout             ;  // Variable Data - Output
 MQLONG                  out0             ;  // Output Data - len
 RXSTRING                out1  = {0,0}    ;  // Output Data - data

 char                    type??(9??)      ;  // Type of the Structure (EVENT)
 char                    name??(9??)      ;  // Name of the Structure

 MQCFH                 * bufpcf1          ;  // Pointers to
 MQCFIN                * bufpcf2i         ;  //  PCF areas within the
 MQCFST                * bufpcf2s         ;  //  Event Message
 MQCFIL                * bufpcf2il        ;
 MQCFSL                * bufpcf2sl        ;

 MQLONG                  parms            ;  //Number of elements
 MQLONG                  parmtype         ;  //TYye of element
 MQLONG                  parmsize         ;  //Length of element
 MQLONG                  parmnumb         ;  //Key of element
 MQLONG                  iparm            ;  //Number in array
 MQLONG                  ival             ;  //Integer element
 MQCFIL                * ip               ;  //-> Integer element
 MQCFSL                * sp               ;  //-> String element
 unsigned char           comp??(11??)     ;  //Component name

 char                    resvar??(600??)  ;  //Name of component
 RXSTRING                resvarn          ;  //
 RXSTRING                mess             ;  //Integer Comp value
 char                    messn??(11??)    ;  //  as Char

 char                    zvars??(900??)   ;  //List of components

 DEBUG("EVENT", "Entering rxmqrevent " ) ;

 rc   = 0 ;
 mqrc = 0 ;
 mqac = 0 ;


 strcpy((char *)returnc,"-99 0 0 RXMQREVENT UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 2 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQREVENT Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQREVENT Null input stem var" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQREVENT Zero input stem var" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQREVENT Null output stem var" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQREVENT Zero output stem var" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

//
// The input parms are OK, so obtain them
//

 if (rc == 0)
    {
      memcpy(&vinp,&aargv??(0??),sizeof(vinp)) ;
      memcpy(&vout,&aargv??(1??),sizeof(vinp)) ;

      get_var0(vinp, &inp0 )        ;
      get_var1(vinp, &inp1 )        ;
    }

//
// Now check the input Stem variable to see that there is
//     some valid data to obtain
//

 if ( (rc == 0) && ( inp0 == 0 ) )
    {
      strcpy((char *)returnc,"-6 0 0 RXMQREVENT No input data" ) ;
      rc   = -6 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

 if ( (rc == 0) && ( inp1.strlength == 0 ) )
    {
      strcpy((char *)returnc,"-7 0 0 RXMQREVENT Zero input data" ) ;
      rc   = -7 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

 if ( (rc == 0) && ( inp0 <= 3 ) )
    {
      strcpy((char *)returnc,"-8 0 0 RXMQREVENT Cannot locate Header" );
      rc   = -8 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

 if ( (rc == 0) && ( inp1.strlength <= 3 ) )
    {
      strcpy((char *)returnc,"-9 0 0 RXMQREVENT Cannot find Header" ) ;
      rc   = -9 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

//
// There is the possibility of a Header, so see if it is an event
//       one, and ignore it it is not one to process.
//

 if ( rc == 0 )
    {
      if ( ((MQCFH *)inp1.strptr)->Type != MQCFT_EVENT )
         {
           strcpy((char *)returnc,"-10 0 0 RXMQREVENT Not an Event Header " ) ;
           rc   = -10 ;
           mqrc =  0  ;
           mqac =  0  ;
         } ;
    }


//
// Although we have a valid header, just check the prefix length
//          to ensure the WHOLE header is present
//

if ( rc == 0 )
   {
     if ( inp0 < sizeof(MQCFH_STRUC_LENGTH) )
        {
          sprintf((char *)returnc,"-11 0 0 RXMQREVENT "
                                  "Too short for an Event (%lu bytes!) ", inp0 ) ;
          rc   = -11 ;
          mqrc =  0  ;
          mqac =  0  ;
        } ;

   }

//
// Now there is an event Header, check the general category
//

if ( rc == 0 )
   {
      DEBUG("EVENT", "Determining Event general Category " ) ;

      bufpcf1   = (MQCFH  *)inp1.strptr                 ; //Point to
      bufpcf2i  = (MQCFIN *)&((char *)inp1.strptr)??(MQCFH_STRUC_LENGTH??) ; //Buffer
      bufpcf2s  = (MQCFST *)&((char *)inp1.strptr)??(MQCFH_STRUC_LENGTH??) ; //Structures
      bufpcf2il = (MQCFIL *)&((char *)inp1.strptr)??(MQCFH_STRUC_LENGTH??) ;
      bufpcf2sl = (MQCFSL *)&((char *)inp1.strptr)??(MQCFH_STRUC_LENGTH??) ;
      parms     = bufpcf1->ParameterCount ;

      switch ( bufpcf1->Command )  //See what the event category is
        {
          case MQCMD_Q_MGR_EVENT       : DEBUG("EVENT","It is a QMGR    Event ") ; break ;
          case MQCMD_PERFM_EVENT       : DEBUG("EVENT","It is a PERF    Event ") ; break ;
          case MQCMD_CHANNEL_EVENT     : DEBUG("EVENT","It is a CHANNEL Event ") ; break ;
          default :
                    sprintf((char *)returnc,"-12 0 0 RXMQREVENT "
                                            "Unknown Event Category (%lu) ", bufpcf1->Command ) ;
                    rc   = -12 ;
                    mqrc =  0  ;
                    mqac =  0  ;
                    break      ;

        } // End of Category determination select

   }

//
// Now there is a valid event Header, decide which one it is
//     and then create the NAME (and TYPE=EVENT) components
//

if ( rc == 0 )
   {
      DEBUG("EVENT", "Starting to examine the Event ") ;

      strcpy(type,"EVENT") ; // Set Type info
      strcpy(name,""     ) ;

      switch ( bufpcf1->Reason )  //See what the event is
        {
          case MQRC_Q_MGR_ACTIVE                : strcpy(name,"QMACT"  ) ; break ;
          case MQRC_Q_MGR_NOT_ACTIVE            : strcpy(name,"QMINA"  ) ; break ;
          case MQRC_GET_INHIBITED               : strcpy(name,"INGET"  ) ; break ;
          case MQRC_PUT_INHIBITED               : strcpy(name,"INPUT"  ) ; break ;
          case MQRC_ALIAS_BASE_Q_TYPE_ERROR     : strcpy(name,"LLAQT"  ) ; break ;
          case MQRC_UNKNOWN_ALIAS_BASE_Q        : strcpy(name,"LLABQ"  ) ; break ;
          case MQRC_UNKNOWN_OBJECT_NAME         : strcpy(name,"LLUON"  ) ; break ;
          case MQRC_CHANNEL_CONV_ERROR          : strcpy(name,"CHCONV" ) ; break ;
          case MQRC_CHANNEL_STARTED             : strcpy(name,"CHSTRT" ) ; break ;
          case MQRC_CHANNEL_STOPPED             : strcpy(name,"CHSTOP" ) ; break ;
          case MQRC_CHANNEL_ACTIVATED           : strcpy(name,"CHACT"  ) ; break ;
          case MQRC_CHANNEL_NOT_ACTIVATED       : strcpy(name,"CHNACT" ) ; break ;
          case MQRC_Q_DEPTH_HIGH                : strcpy(name,"PFQDH"  ) ; break ;
          case MQRC_Q_DEPTH_LOW                 : strcpy(name,"PFQDL"  ) ; break ;
          case MQRC_Q_FULL                      : strcpy(name,"PFQFU"  ) ; break ;
          case MQRC_Q_SERVICE_INTERVAL_HIGH     : strcpy(name,"PFQSH"  ) ; break ;
          case MQRC_Q_SERVICE_INTERVAL_OK       : strcpy(name,"PFQSO"  ) ; break ;
          case MQRC_DEF_XMIT_Q_TYPE_ERROR       : strcpy(name,"RMDXQT" ) ; break ;
          case MQRC_DEF_XMIT_Q_USAGE_ERROR      : strcpy(name,"RMDXQU" ) ; break ;
          case MQRC_Q_TYPE_ERROR                : strcpy(name,"RMQUTY" ) ; break ;
          case MQRC_REMOTE_Q_NAME_ERROR         : strcpy(name,"RMRQNA" ) ; break ;
          case MQRC_XMIT_Q_TYPE_ERROR           : strcpy(name,"RMXQTY" ) ; break ;
          case MQRC_XMIT_Q_USAGE_ERROR          : strcpy(name,"RMXQUS" ) ; break ;
          case MQRC_UNKNOWN_DEF_XMIT_Q          : strcpy(name,"RMUDXQ" ) ; break ;
          case MQRC_UNKNOWN_REMOTE_Q_MGR        : strcpy(name,"RMURQM" ) ; break ;
          case MQRC_UNKNOWN_XMIT_Q              : strcpy(name,"RRUXQN" ) ; break ;
          case MQRC_NOT_AUTHORIZED              : strcpy(name,"NAAUT"  ) ; break ;

          default :
                    sprintf((char *)returnc,"-13 0 0 RXMQREVENT "
                                            "Unknown Event Type (%lu) ", bufpcf1->Reason ) ;
                    rc   = -13 ;
                    mqrc =  0  ;
                    mqac =  0  ;
                    break      ;

        } // End of Type determination select

      DEBUG("EVENT", "Event " << bufpcf1->Reason
                      << " maps to " << name
                      << ( (strlen(name) != 0) ? " "
                                             : " which is unknown ") ) ;

   }

//
// There is something that looks like a valid event, but just
//       ensure there are some elements in it
//

if ( rc == 0 )
   {
     DEBUG("EVENT","There are " << bufpcf1->ParameterCount << " elements in the Event ") ;
     if ( bufpcf1->ParameterCount == 0 )
        {
          sprintf((char *)returnc,"-14 0 0 RXMQREVENT No elements in the Event " ) ;
          rc   = -14 ;
          mqrc =  0  ;
          mqac =  0  ;
        } ;
   }

//
// Now there is a valid event , create the TYPE and NAME components,
//           as well as REAson from the Prefix Header. Also initialise
//           the zvars (to goto .ZLIST) component store.
//

if ( rc == 0 )
   {

    sprintf(resvar,"%sTYPE",vout.strptr)         ;
    resvarn.strlength = strlen(resvar)           ;
    resvarn.strptr    = (char *)resvar           ;
    mess.strlength    =  strlen(type)            ;
    mess.strptr       = (char *)&type            ;
    put_var (resvarn,mess)                       ;

    sprintf(resvar,"%sNAME",vout.strptr)         ;
    resvarn.strlength = strlen(resvar)           ;
    resvarn.strptr    = (char *)resvar           ;
    mess.strlength    =  strlen(name)            ;
    mess.strptr       = (char *)&name            ;
    put_var (resvarn,mess)                       ;

    sprintf(resvar,"%sREA",vout.strptr)          ;
    resvarn.strlength = strlen(resvar)           ;
    resvarn.strptr    = (char *)resvar           ;
    sprintf(messn,"%lu",bufpcf1->Reason)         ;
    mess.strlength    = strlen(messn)            ;
    mess.strptr       = (char *)messn            ;
    put_var (resvarn,mess)                       ;

    sprintf(zvars,"NAME TYPE REA")               ;

   }

//
// Now parse the event and create the relevant components
//
//               Build the ZLIST results into zvars for each parm
//
//               List entries are setup as stem.component.0 = number of items
//                                                       .n = the nth item
//

 if ( rc == 0 )                      // Scan and Extract
    {
      DEBUG("EVENT", "Now starting to scan the Event ") ;


      for (i=0 ; i < parms ; i++ )
          {
           parmtype = bufpcf2i->Type        ; //All types share a
           parmsize = bufpcf2i->StrucLength ; // common prefix
           parmnumb = bufpcf2i->Parameter   ;
           switch ( parmtype )
            {
              case MQCFT_INTEGER  : //Convert to attr=value
                   geteventname((char *)comp,parmnumb)          ;
                   sprintf(resvar,"%s%s",vout.strptr,comp)      ;
                   strcat(zvars," ")                            ;
                   strcat(zvars,(const char *)comp)             ;
                   resvarn.strlength = strlen(resvar)           ;
                   resvarn.strptr    = (char *)resvar           ;
                   switch ( parmnumb ) //Display in Hex or (mostly) decimal
                    {
                      case MQIACF_AUX_ERROR_DATA_INT_1      :
                      case MQIACF_AUX_ERROR_DATA_INT_2      :
                      case MQIACF_ERROR_IDENTIFIER          :
                      case MQIACH_CHANNEL_ERROR_DATA        :
                                 sprintf(messn,"%08lX",bufpcf2i->Value)  ;
                                 DEBUG("EVENT", " Integer Parm "     << bufpcf2i->Parameter
                                             << " Value "            << bufpcf2i->Value
                                             << " displayed as hex " << messn << " " ) ;
                                 break                                   ;

                      default  : sprintf(messn,"%lu",bufpcf2i->Value)    ;
                                 break                                   ;

                    }
                   mess.strlength    = strlen(messn)            ;
                   mess.strptr       = (char *)messn            ;
                   put_var (resvarn,mess)                       ;
                   DEBUG("EVENT", " Integer Parm " << bufpcf2i->Parameter
                               << " Value "        << bufpcf2i->Value
                               << " into  "        << comp << " " ) ;
                   break        ;

              case MQCFT_INTEGER_LIST : //Convert to attr=value pairs
                   if ( bufpcf2il->Count != 0 )
                      {
                        DEBUG("EVENT", " Integer List " << bufpcf2il->Count
                                       << " Values "  ) ;

                        geteventname((char *)comp,bufpcf2il->Parameter)     ;
                        sprintf(resvar,"%s%s%.0",vout.strptr,comp)          ;
                        strcat(zvars," ")                                   ;
                        strcat(zvars,(const char *)comp)                    ;
                        strcat(zvars,".0")                                  ;
                        resvarn.strlength = strlen(resvar)                  ;
                        resvarn.strptr    = (char *)resvar                  ;
                        sprintf(messn,"%lu",bufpcf2il->Count)               ;
                        mess.strlength    = strlen(messn)                   ;
                        mess.strptr       = (char *)messn                   ;
                        put_var (resvarn,mess)                              ;

                        for ( j=0 ; j < bufpcf2il->Count ; j++ )
                            {
                              geteventname((char *)comp,bufpcf2il->Parameter)     ;
                              sprintf(resvar," %s%.%lu",comp,(j+1))               ;
                              strcat(zvars,resvar)                                ;
                              sprintf(resvar,"%s%s%.%lu",vout.strptr,comp,(j+1))  ;
                              resvarn.strlength = strlen(resvar)                  ;
                              resvarn.strptr    = (char *)resvar                  ;
                              switch ( parmnumb ) //Display in Hex or (mostly) decimal
                               {
                                 case MQIACF_AUX_ERROR_DATA_INT_1      :
                                 case MQIACF_AUX_ERROR_DATA_INT_2      :
                                 case MQIACF_ERROR_IDENTIFIER          :
                                 case MQIACH_CHANNEL_ERROR_DATA        :
                                            sprintf(messn,"%08lX",bufpcf2il->Values??(j??))       ;
                                            DEBUG("EVENT", " Integer List "     << bufpcf2il->Parameter
                                                        << " Value "            << bufpcf2il->Values??(j??)
                                                        << " displayed as hex " << messn << " " ) ;
                                            break                                                 ;

                                 default  : sprintf(messn,"%lu",bufpcf2il->Values??(j??))    ;
                                            break                                            ;

                               }
                              mess.strlength    = strlen(messn)                   ;
                              mess.strptr       = (char *)messn                   ;
                              put_var (resvarn,mess)                              ;
                              DEBUG("EVENT", " Integer List " << bufpcf2il->Parameter
                                          << " Value "        << bufpcf2il->Values??(j??)
                                          << " into  "        << comp << " " ) ;
                            } ;
                      } ;
                   break        ;


              case MQCFT_STRING : //Print in the Stem variable

                   geteventname((char *)comp,parmnumb)                      ;
                   sprintf(resvar,"%s%s",vout.strptr,comp)                  ;
                   strcat(zvars," ")                                        ;
                   strcat(zvars,(const char *)comp)                         ;
                   resvarn.strlength = strlen(resvar)                       ;
                   resvarn.strptr    = (char *)resvar                       ;
                   mess.strlength    =  bufpcf2s->StringLength              ;
                   mess.strptr       = (char *)&(bufpcf2s->String)          ;
                   put_var (resvarn,mess)                                   ;
                   DEBUG("EVENT", " String Parm " << bufpcf2s->Parameter
                               << " Value "       << bufpcf2s->String
                               << " into  "       << comp << " " ) ;
                   break                                                 ;

              case MQCFT_STRING_LIST : //Print Strings in the Stem Variable
                   if ( bufpcf2sl->Count != 0 )
                      {
                       DEBUG("EVENT", " String List " << bufpcf2sl->Count
                                       << " Values "  ) ;

                       geteventname((char *)comp,bufpcf2sl->Parameter)     ;
                       sprintf(resvar,"%s%s%.0",vout.strptr,comp)          ;
                       strcat(zvars," ")                                   ;
                       strcat(zvars,(const char *)comp)                    ;
                       strcat(zvars,".0")                                  ;
                       resvarn.strlength = strlen(resvar)                  ;
                       resvarn.strptr    = (char *)resvar                  ;
                       sprintf(messn,"%lu",bufpcf2sl->Count)               ;
                       mess.strlength    = strlen(messn)                   ;
                       mess.strptr       = (char *)messn                   ;
                       put_var (resvarn,mess)                              ;

                       for ( j=0 ; j < bufpcf2sl->Count ; j++ )
                           {
                             sp = bufpcf2sl + ( (bufpcf2sl->StringLength) * j)   ;
                             geteventname((char *)comp,bufpcf2sl->Parameter)     ;
                             sprintf(resvar," %s%.%lu",comp,(j+1))               ;
                             strcat(zvars,resvar)                                ;
                             sprintf(resvar,"%s%s%.%ul",vout.strptr,comp,(j+1))  ;
                             resvarn.strlength = strlen(resvar)                  ;
                             resvarn.strptr    = (char *)resvar                  ;
                             mess.strlength    =  bufpcf2sl->StringLength        ;
                             mess.strptr       = (char *)&(sp->Strings)          ;
                             put_var (resvarn,mess)                      ;
                             DEBUG("EVENT", " String List " << bufpcf2sl->Parameter
                                         << " Value "       << sp->Strings
                                         << " into  "       << comp << " " ) ;
                             break                                      ;
                           } ;
                      } ;

                   break        ;
              default :
                   break ;

            } ;

           bufpcf2i  = (MQCFIN *)( (char *)bufpcf2i  + parmsize ) ; //Bump
           bufpcf2s  = (MQCFST *)( (char *)bufpcf2s  + parmsize ) ; // up
           bufpcf2il = (MQCFIL *)( (char *)bufpcf2il + parmsize ) ; // structure
           bufpcf2sl = (MQCFSL *)( (char *)bufpcf2sl + parmsize ) ; // pointers

          } ; //End of Parameter Printing loop


      // All Parms/Components extracted, so create ZLIST

      sprintf(resvar,"%sZLIST",vout.strptr)         ;
      resvarn.strlength = strlen(resvar)            ;
      resvarn.strptr    = (char *)resvar            ;
      mess.strlength    =  strlen(zvars)            ;
      mess.strptr       = (char *)&zvars            ;
      put_var (resvarn,mess)                        ;

      DEBUG("EVENT", "All Event fields extracted ") ;


    } // End of Event Processing Block


//
// Set a Good return Code if apt
//

 if ( rc == 0 )
    {
      strcpy((char *)returnc,"0 0 0 RXMQREVENT OK" ) ;
      rc   =  0 ;
      mqrc =  0 ;
      mqac =  0 ;
    }


//
// Set the LAST variables, and the function return string
//

 stack_and_set(rc,mqrc,mqac,"RXMQREVENT",returnc)      ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                 ;

//
// Return to caller
//

 DEBUG("EVENT", "Leaving rxmqrevent " ) ;

 return 0 ;

 } // End of RXMQREVENT function


//
// Process a Trigger Message RXMQRTM
//
//   Call:   rc = rxmqrtm( input_Stem,   output_stem )
//
//           rc = rxmqrtm( trig_message, output_stem )
//
//
//   This function works in two ways, depending upon whether or not the first
//        parameter ends with a dot (indicating a stem variable or some real
//        data).
//
//        If its a stem variable, then the usual .0 .1 convention applies and
//           and the message is deemed to be a Trigger Message derived from a
//           Queue. In this case, it is broken up into its components of
//           QN PN TD AT AID ED UD. These components are only generated if
//           the relevant fields are non-blank and non-zero. ZLIST processing
//           is provided for these components. Additionally, a component called
//           PL is created, which is a string (in TM2 format) suitable for
//           passing to an attached procedure (the QM field will be filled in
//           with the current Queue manager, if connected). PL is NOT part of
//           ZLIST (as it should not be fiddled about with in any way).
//
//        If its NOT a stem variable (not ending in a dot), then the input is
//           deamed to be the TM2 (PL) structure (which must be of the correct
//           length). In this case, the data (and its NOT a REXX variable) is
//           parsed for components QN PN TD AT ED UD QM with ZLIST processing;
//           if the item is all blanks (or all nulls!?!) then the component is
//           not built. Each component should be stripped of blanks before
//           usage in the Rexx environment.
//
//

LONG _Export _System rxmqrtm
                     (PSZ        afuncname   ,  //Rexx Func name
                      LONG       aargc       ,  //     Number of parms
                      PRXSTRING  aargv       ,  //     Parms
                      PSZ        aqname      ,  //     Cur Queue
                      PRXSTRING  aretstr     )  //     Return String
 {

 ULONG                   i                ;  // Looper
 unsigned char           returnc??(600??) ;  // Return String
 LONG                    rc               ;  // Function Return Code
 MQLONG                  ac               ;  // MQ instance Return Code
 MQLONG                  mqrc             ;  // MQ RC
 MQLONG                  mqac             ;  // MQ AC

 MQCHAR48                qm               ;  // Connected QM

 char                    header??(5??)    ;  // Header  Type
 char                    versionc??(5??)  ;  // Version Type in Char
 MQLONG                  version          ;  // Version Type

 MQTM                  * thetm            ;  // -> Trigger Message
 MQTMC2                * thetm2           ;  // -> Trigger Parm

 RXSTRING                oldtm2           ;  // Supplied Trigger Message
 MQTMC2                  newtm2           ;  // Build Trigger Parm

 int                     action = '1'     ;  // TM or TM2 processing

 RXSTRING                vinp             ;  // Variable Data - Input
 MQLONG                  inp0             ;  // Input Data - len
 RXSTRING                inp1  = {0,0}    ;  // Input Data - data

 RXSTRING                vout             ;  // Variable Data - Output
 MQLONG                  out0             ;  // Output Data - len
 RXSTRING                out1  = {0,0}    ;  // Output Data - data


 RXSTRING                varname          ;  // Variable name
 unsigned char           varnamc??(900??) ;  // Char version of variable
 RXSTRING                varvalu          ;  // Variable setting
 unsigned char           varvalc??(900??) ;  // Char version of variable
 SHVBLOCK                sv1              ;  // Var interface CB
 ULONG                   sv1rc            ;  // Var interface RC
 MQLONG                  number           ;  // Char->Integer Conver

 char                    zvars??(100??)   ;  //List of components

 MQCHAR48                blank48          ;  //Blank field comparers
 MQCHAR64                blank64          ;
 MQCHAR128               blank128         ;
 MQCHAR256               blank256         ;


 DEBUG("TM", "Entering rxmqrtm " ) ;

 rc     = 0 ;
 mqrc   = 0 ;
 mqac   = 0 ;
 action = 1 ;

 memset(blank48 , ' ', sizeof(blank48)  ) ;
 memset(blank64 , ' ', sizeof(blank64)  ) ;
 memset(blank128, ' ', sizeof(blank128) ) ;
 memset(blank256, ' ', sizeof(blank256) ) ;
 memset(qm,       ' ', sizeof(qm)       ) ;
 memset(&newtm2,  ' ', sizeof(newtm2)   ) ;

 memset(zvars,   0,    sizeof(zvars)    )  ;

 strcpy((char *)returnc,"-99 0 0 RXMQRTM UNKNOWN FAILURE" ) ;

//
// Check the parms
//

 if ( (rc == 0) && (aargc != 2 ) )
    {
     strcpy((char *)returnc,"-1 0 0 RXMQRTM Bad number of parms" ) ;
     rc   = -1 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-2 0 0 RXMQRTM Null input stem var" ) ;
     rc   = -2 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(0??)) )
    {
     strcpy((char *)returnc,"-3 0 0 RXMQRTM Zero input stem var" ) ;
     rc   = -3 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXNULLSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-4 0 0 RXMQRTM Null output stem var" ) ;
     rc   = -4 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

 if ( (rc == 0) && RXZEROLENSTRING(aargv??(1??)) )
    {
     strcpy((char *)returnc,"-5 0 0 RXMQRTM Zero output stem var" ) ;
     rc   = -5 ;
     mqrc =  0 ;
     mqac =  0 ;
    } ;

//
// The input parms are OK, so obtain them, and obtain the resolutions
//

 if (rc == 0)
    {
      int dotat ;

      memcpy(&vinp,&aargv??(0??),sizeof(vinp)) ;
      memcpy(&vout,&aargv??(1??),sizeof(vout)) ;

      dotat = vinp.strlength - 1 ;

      if ( vinp.strptr??(dotat??) == '.' )
         {
           DEBUG("TM" , "Processing a Trigger Message" ) ;
           action = 1               ;
           get_var0(vinp, &inp0   ) ;
           get_var1(vinp, &inp1   ) ;
           DEBUG("TM" , "Obtained the Trigger Message. Length " << inp0
                        << "/"       << inp1.strlength ) ;
           DEBUGW("TM" , write("Data /",6), write((const char *)inp1.strptr,inp1.strlength) ,
                         write("/\n",2) ) ;

         } else
         {
           DEBUG("TM" , "Processing Trigger Data"   ) ;
           action = 2                        ;
           oldtm2.strptr    = vinp.strptr    ;
           oldtm2.strlength = vinp.strlength ;
           DEBUG("TM" , "Obtained the Trigger Data. Length " << vinp.strlength ) ;
           DEBUGW("TM" , write("Data /",6) , write((const char *)vinp.strptr,vinp.strlength) ,
                         write("/\n",2 ) ) ;
         } ;

    } ;

//
// Now check the input Stem variable to see that there is
//     some valid data to obtain
//

 if ( (rc == 0) && (action == 1) && ( inp0 == 0 ) )
    {
      strcpy((char *)returnc,"-6 0 0 RXMQRTM No input data" ) ;
      rc   = -6 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

 if ( (rc == 0) && (action == 1) && ( inp1.strlength == 0 ) )
    {
      strcpy((char *)returnc,"-7 0 0 RXMQRTM Zero input data" ) ;
      rc   = -7 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

 if ( (rc == 0) && (action == 1) && ( inp0 <= 3 ) )
    {
      strcpy((char *)returnc,"-8 0 0 RXMQRTM Cannot locate Header" );
      rc   = -8 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

 if ( (rc == 0) && (action == 1) && ( inp1.strlength <= 3 ) )
    {
      strcpy((char *)returnc,"-9 0 0 RXMQRTM Cannot find Header" ) ;
      rc   = -9 ;
      mqrc =  0 ;
      mqac =  0 ;
    } ;

//
// Now check the input data to see that there is
//     some valid data to obtain
//

 if ( (rc == 0) && (action == 2) && ( oldtm2.strlength == 0 ) )
    {
      strcpy((char *)returnc,"-10 0 0 RXMQRTM Zero input data" ) ;
      rc   = -10 ;
      mqrc =   0 ;
      mqac =   0 ;
    } ;

 if ( (rc == 0) && (action == 2) && ( oldtm2.strlength <= 3 ) )
    {
      strcpy((char *)returnc,"-11 0 0 RXMQRTM Cannot find Header" ) ;
      rc   = -11 ;
      mqrc =   0 ;
      mqac =   0 ;
    } ;

//
// There is the possibility of an Trigger Message, so see if it is a known
//       one, and ignore it it is not one to process.
//

 if ( rc == 0 )
    {
      memset(header,    0, sizeof(header)   ) ;
      memset(&versionc, 0, sizeof(versionc) ) ;
      version = 0                             ;

      if ( action == 1 ) strncpy( header,   (const char *)inp1.strptr   , sizeof(MQCHAR4) ) ;
          else           strncpy( header,   (const char *)oldtm2.strptr , sizeof(MQCHAR4) ) ;

      if ( action == 1 ) version = ((MQTM *)inp1.strptr)->Version  ;
          else           strncpy( versionc, (const char *)&(((MQTMC2 *)oldtm2.strptr )->Version) , sizeof(MQCHAR4) ) ;

      DEBUG("TM", "Action " << action << " Header /" << header
                  << "/ Versionc /" << versionc << "/ Version /" << version << "/ "  ) ;

      if (    ( rc == 0 )
           && ( strcmp(header, MQTM_STRUC_ID ) != 0 )
           && ( strcmp(header, MQTMC_STRUC_ID) != 0 ) )
         {
           strcpy((char *)returnc,"-12 0 0 RXMQRTM Unknown Header" ) ;
           rc   = -12 ;
           mqrc =  0  ;
           mqac =  0  ;
         } ;

      if (    ( rc == 0 )
           && ( version  != MQTM_VERSION_1 )
           && ( strncmp(versionc, MQTMC_VERSION_2, sizeof(MQCHAR4) ) != 0 ) )
         {
           strcpy((char *)returnc,"-13 0 0 RXMQRTM Unknown Version" ) ;
           rc   = -13 ;
           mqrc =  0  ;
           mqac =  0  ;
         } ;
    } ;

//
// Now there is a trigger area of some sort, just check to see its the right one
//

 if ( ( rc == 0 ) && (action == 1) && ( strcmp(header, MQTM_STRUC_ID ) != 0 ) )
    {
      strcpy((char *)returnc,"-14 0 0 RXMQRTM Header mismatch (1<>1) " ) ;
      rc   = -14 ;
      mqrc =  0  ;
      mqac =  0  ;
    }

 if ( ( rc == 0 ) && (action == 1) && ( version != MQTM_VERSION_1 ) )
    {
      strcpy((char *)returnc,"-15 0 0 RXMQRTM Version mismatch (1<>1) " ) ;
      rc   = -15 ;
      mqrc =  0  ;
      mqac =  0  ;
    }

 if ( ( rc == 0 ) && (action == 2) && ( strcmp(header, MQTMC_STRUC_ID ) != 0 ) )
    {
      strcpy((char *)returnc,"-16 0 0 RXMQRTM Header mismatch (2<>C) " ) ;
      rc   = -16 ;
      mqrc =  0  ;
      mqac =  0  ;
    }

 if ( ( rc == 0 ) && (action == 2) && ( strncmp(versionc, MQTMC_VERSION_2, sizeof(MQCHAR4) ) != 0 ) )
    {
      strcpy((char *)returnc,"-17 0 0 RXMQRTM Version mismatch (2<>C) " ) ;
      rc   = -17 ;
      mqrc =  0  ;
      mqac =  0  ;
    }


//
// Although we have a valid item, just check the lengths
//          to ensure the WHOLE thing is present
//

if ( rc == 0 )
   {
     if (    ( rc     == 0            )
          && ( action == 1            )
          && ( inp0   <  sizeof(MQTM) ) )
        {
          sprintf((char *)returnc,"-18 0 0 RXMQRTM "
                                  "Too short for a TM (%lu bytes!) " ,
                                   inp0 ) ;
          rc   = -18 ;
          mqrc =  0  ;
          mqac =  0  ;
        } ;

     if (    ( rc               == 0              )
          && ( action           == 2              )
          && ( oldtm2.strlength <  sizeof(MQTMC2) ) )
        {
          sprintf((char *)returnc,"-19 0 0 RXMQRTM "
                                  "Too short for a TMC (%lu bytes!) " ,
                                   oldtm2.strlength ) ;
          rc   = -19 ;
          mqrc =  0  ;
          mqac =  0  ;
        } ;

   }


//
// Now we have got a valid Trigger Message, split it up
//

if ( ( rc == 0 ) && ( action == 1 ) )
   {
     DEBUG("TM", "Unravelling a TM message"  ) ;

     DEBUG("TM", "Finding the QM "  ) ;

     if ( isthreadconn() != FALSE ) for ( i=MINQS ; i <= MAXQS ; i++ )
        {
          if (    ( iscurrthread(&gmqo??(i??)) != FALSE )
               && ( gmqo??(i??).qm??(0??)      != 0     ) )
             {
               DEBUG("TM", "Found QM in Object " << i << " "      ) ;
               memcpy(qm, &gmqo??(i??).qm, strlen(gmqo??(i??).qm) ) ;
               break      ;
             }
        }

     DEBUG("TM", "QM name is /" << qm << "/ " ) ;

     DEBUG("TM", "Now starting the unpacking ") ;
     thetm = (MQTM *)inp1.strptr   ; // Set Header pointer

                                     // Build the output TMC2 area Header

     memcpy(&newtm2.StrucId, MQTMC_STRUC_ID,  sizeof(newtm2.StrucId) ) ;
     memcpy(&newtm2.Version, MQTMC_VERSION_2, sizeof(newtm2.Version) ) ;

                                    //Build the components

     DEBUG("TM", "Looking at QN  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm->QName,sizeof(MQCHAR48)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm->QName??(0??)                     != 0 )
          && (strncmp(thetm->QName, blank48, sizeof(blank48)) != 0 ) )
        {
          strcat(zvars, "QN " )                                          ;
          memcpy(&newtm2.QName,     thetm->QName,      sizeof(MQCHAR48)) ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),thetm->QName)  ;
          sprintf((char *)varnamc,"%sQN",vout.strptr)                    ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))    ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))    ;
          sv1.shvnext      = 0                                           ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                 ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                 ;
          sv1.shvvaluelen = varvalu.strlength                            ;
          sv1.shvcode     = RXSHV_SYSET                                  ;
          sv1.shvret      = 0                                            ;
          sv1rc           = RexxVariablePool(&sv1)                       ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Looking at PN  ") ;
     DEBUGW("TM" , write("Data /",6),  write((const char *)thetm->ProcessName,sizeof(MQCHAR48)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm->ProcessName??(0??)                     != 0 )
          && (strncmp(thetm->ProcessName, blank48, sizeof(blank48)) != 0 ) )
        {
          strcat(zvars, "PN " )                                              ;
          memcpy(&newtm2.ProcessName, thetm->ProcessName, sizeof(MQCHAR48))  ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),thetm->ProcessName);
          sprintf((char *)varnamc,"%sPN",vout.strptr)                        ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))        ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))        ;
          sv1.shvnext      = 0                                               ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                     ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                     ;
          sv1.shvvaluelen = varvalu.strlength                                ;
          sv1.shvcode     = RXSHV_SYSET                                      ;
          sv1.shvret      = 0                                                ;
          sv1rc           = RexxVariablePool(&sv1)                           ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Looking at TD  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm->TriggerData,sizeof(MQCHAR64)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm->TriggerData??(0??)                     != 0 )
          && (strncmp(thetm->TriggerData, blank64, sizeof(blank64)) != 0 ) )
        {
          strcat(zvars, "TD " )                                              ;
          memcpy(&newtm2.TriggerData, thetm->TriggerData, sizeof(MQCHAR64))  ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR64),thetm->TriggerData);
          sprintf((char *)varnamc,"%sTD",vout.strptr)                        ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))        ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))        ;
          sv1.shvnext      = 0                                               ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                     ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                     ;
          sv1.shvvaluelen = varvalu.strlength                                ;
          sv1.shvcode     = RXSHV_SYSET                                      ;
          sv1.shvret      = 0                                                ;
          sv1rc           = RexxVariablePool(&sv1)                           ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Looking at AT  ") ;
     strcat(zvars, "AT " )                                             ;
     sprintf((char *)varvalc,"%lu",thetm->ApplType)                    ;
     sprintf((char *)varnamc,"%sAT",vout.strptr)                       ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;


     DEBUG("TM", "Looking at AID ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm->ApplId,sizeof(MQCHAR256)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm->ApplId??(0??)                       != 0 )
          && (strncmp(thetm->ApplId, blank256, sizeof(blank256)) != 0 ) )
        {
          strcat(zvars, "AID ")                                              ;
          memcpy(&newtm2.ApplId,      thetm->ApplId,      sizeof(MQCHAR256)) ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR256),thetm->ApplId)    ;
          sprintf((char *)varnamc,"%sAID",vout.strptr)                       ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))        ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))        ;
          sv1.shvnext      = 0                                               ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                     ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                     ;
          sv1.shvvaluelen = varvalu.strlength                                ;
          sv1.shvcode     = RXSHV_SYSET                                      ;
          sv1.shvret      = 0                                                ;
          sv1rc           = RexxVariablePool(&sv1)                           ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Looking at ED  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm->EnvData,sizeof(MQCHAR128)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm->EnvData??(0??)                       != 0 )
          && (strncmp(thetm->EnvData, blank128, sizeof(blank128)) != 0 ) )
        {
          strcat(zvars, "ED " )                                              ;
          memcpy(&newtm2.EnvData,     thetm->EnvData,     sizeof(MQCHAR128)) ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR128),thetm->EnvData)   ;
          sprintf((char *)varnamc,"%sED",vout.strptr)                        ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))        ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))        ;
          sv1.shvnext      = 0                                               ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                     ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                     ;
          sv1.shvvaluelen = varvalu.strlength                                ;
          sv1.shvcode     = RXSHV_SYSET                                      ;
          sv1.shvret      = 0                                                ;
          sv1rc           = RexxVariablePool(&sv1)                           ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Looking at UD  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm->UserData,sizeof(MQCHAR128)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm->UserData??(0??)                       != 0 )
          && (strncmp(thetm->UserData, blank128, sizeof(blank128)) != 0 ) )
        {
          strcat(zvars, "UD " )                                              ;
          memcpy(&newtm2.UserData,    thetm->UserData,    sizeof(MQCHAR128)) ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR128),thetm->UserData)  ;
          sprintf((char *)varnamc,"%sUD",vout.strptr)                        ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))        ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))        ;
          sv1.shvnext      = 0                                               ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                     ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                     ;
          sv1.shvvaluelen = varvalu.strlength                                ;
          sv1.shvcode     = RXSHV_SYSET                                      ;
          sv1.shvret      = 0                                                ;
          sv1rc           = RexxVariablePool(&sv1)                           ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Looking at ZLIST ") ;
     strcpy((char *)varvalc, zvars)                                    ;
     sprintf((char *)varnamc,"%sZLIST",vout.strptr)                    ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     memcpy(&newtm2.QMgrName, qm, sizeof(qm) )                         ;

     DEBUG("TM", "Setting the PL ") ;
     memcpy((char *)varvalc, &newtm2, sizeof(newtm2))                  ;
     sprintf((char *)varnamc,"%sPL",vout.strptr)                       ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,sizeof(newtm2)               )       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     DEBUG("TM", "Unravelled the TM" ) ;
   }

//
// Now we have got valid Trigger Data, split it up
//

if ( ( rc == 0 ) && ( action == 2 ) )
   {
     DEBUG("TM", "Unravelling Trigger Data"  ) ;

     thetm2 = (MQTMC2 *)oldtm2.strptr   ; // Set Header pointer

                                    //Build the components

     DEBUG("TM", "Doing QN  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm2->QName,sizeof(MQCHAR48)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm2->QName??(0??)                     != 0 )
          && (strncmp(thetm2->QName, blank48, sizeof(blank48)) != 0 ) )
        {
          strcat(zvars, "QN " )                                          ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),thetm2->QName) ;
          sprintf((char *)varnamc,"%sQN",vout.strptr)                    ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))    ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))    ;
          sv1.shvnext      = 0                                           ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                 ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                 ;
          sv1.shvvaluelen = varvalu.strlength                            ;
          sv1.shvcode     = RXSHV_SYSET                                  ;
          sv1.shvret      = 0                                            ;
          sv1rc           = RexxVariablePool(&sv1)                       ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Doing PN  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm2->ProcessName,sizeof(MQCHAR48)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm2->ProcessName??(0??)                     != 0 )
          && (strncmp(thetm2->ProcessName, blank48, sizeof(blank48)) != 0 ) )
        {
          strcat(zvars, "PN " )                                               ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),thetm2->ProcessName);
          sprintf((char *)varnamc,"%sPN",vout.strptr)                         ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))         ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))         ;
          sv1.shvnext      = 0                                                ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                      ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                      ;
          sv1.shvvaluelen = varvalu.strlength                                 ;
          sv1.shvcode     = RXSHV_SYSET                                       ;
          sv1.shvret      = 0                                                 ;
          sv1rc           = RexxVariablePool(&sv1)                            ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Doing TD  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm2->TriggerData,sizeof(MQCHAR64)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm2->TriggerData??(0??)                     != 0 )
          && (strncmp(thetm2->TriggerData, blank64, sizeof(blank64)) != 0 ) )
        {
          strcat(zvars, "TD " )                                               ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR64),thetm2->TriggerData);
          sprintf((char *)varnamc,"%sTD",vout.strptr)                         ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))         ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))         ;
          sv1.shvnext      = 0                                                ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                      ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                      ;
          sv1.shvvaluelen = varvalu.strlength                                 ;
          sv1.shvcode     = RXSHV_SYSET                                       ;
          sv1.shvret      = 0                                                 ;
          sv1rc           = RexxVariablePool(&sv1)                            ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Doing AID ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm2->ApplId,sizeof(MQCHAR256)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm2->ApplId??(0??)                       != 0 )
          && (strncmp(thetm2->ApplId, blank256, sizeof(blank256)) != 0 ) )
        {
          strcat(zvars, "AID " )                                             ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR256),thetm2->ApplId)   ;
          sprintf((char *)varnamc,"%sAID",vout.strptr)                       ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))        ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))        ;
          sv1.shvnext      = 0                                               ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                     ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                     ;
          sv1.shvvaluelen = varvalu.strlength                                ;
          sv1.shvcode     = RXSHV_SYSET                                      ;
          sv1.shvret      = 0                                                ;
          sv1rc           = RexxVariablePool(&sv1)                           ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Doing ED  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm2->EnvData,sizeof(MQCHAR128)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm2->EnvData??(0??)                       != 0 )
          && (strncmp(thetm2->EnvData, blank128, sizeof(blank128)) != 0 ) )
        {
          strcat(zvars, "ED " )                                              ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR128),thetm2->EnvData)  ;
          sprintf((char *)varnamc,"%sED",vout.strptr)                        ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))        ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))        ;
          sv1.shvnext      = 0                                               ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                     ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                     ;
          sv1.shvvaluelen = varvalu.strlength                                ;
          sv1.shvcode     = RXSHV_SYSET                                      ;
          sv1.shvret      = 0                                                ;
          sv1rc           = RexxVariablePool(&sv1)                           ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Doing UD  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm2->UserData,sizeof(MQCHAR128)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm2->UserData??(0??)                       != 0 )
          && (strncmp(thetm2->UserData, blank128, sizeof(blank128)) != 0 ) )
        {
          strcat(zvars, "UD " )                                              ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR128),thetm2->UserData) ;
          sprintf((char *)varnamc,"%sUD",vout.strptr)                        ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))        ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))        ;
          sv1.shvnext      = 0                                               ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                     ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                     ;
          sv1.shvvaluelen = varvalu.strlength                                ;
          sv1.shvcode     = RXSHV_SYSET                                      ;
          sv1.shvret      = 0                                                ;
          sv1rc           = RexxVariablePool(&sv1)                           ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Doing QM  ") ;
     DEBUGW("TM" , write("Data /",6) , write((const char *)thetm2->QMgrName,sizeof(MQCHAR48)) ,
                                       write("/\n",2) ) ;
     if (    (        thetm2->QMgrName??(0??)                     != 0 )
          && (strncmp(thetm2->QMgrName, blank48, sizeof(blank48)) != 0 ) )
        {
          strcat(zvars, "QM " )                                              ;
          sprintf((char *)varvalc,"%.*s",sizeof(MQCHAR48),thetm2->QMgrName)  ;
          sprintf((char *)varnamc,"%sQM",vout.strptr)                        ;
          MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))        ;
          MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))        ;
          sv1.shvnext      = 0                                               ;
          memcpy(&sv1.shvname, &varname,sizeof(varname))                     ;
          memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                     ;
          sv1.shvvaluelen = varvalu.strlength                                ;
          sv1.shvcode     = RXSHV_SYSET                                      ;
          sv1.shvret      = 0                                                ;
          sv1rc           = RexxVariablePool(&sv1)                           ;

          DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
                    << "<- " << strlen((const char *)varvalu.strptr)
                    << "/"   << varvalu.strlength ) ;
        } ;

     DEBUG("TM", "Doing ZLIST ") ;
     strcpy((char *)varvalc, zvars)                                    ;
     sprintf((char *)varnamc,"%sZLIST",vout.strptr)                    ;
     MAKERXSTRING(varname,varnamc,strlen((const char *)varnamc))       ;
     MAKERXSTRING(varvalu,varvalc,strlen((const char *)varvalc))       ;
     sv1.shvnext      = 0                                              ;
     memcpy(&sv1.shvname, &varname,sizeof(varname))                    ;
     memcpy(&sv1.shvvalue,&varvalu,sizeof(varvalu))                    ;
     sv1.shvvaluelen = varvalu.strlength                               ;
     sv1.shvcode     = RXSHV_SYSET                                     ;
     sv1.shvret      = 0                                               ;
     sv1rc           = RexxVariablePool(&sv1)                          ;

     DEBUG("TM", "rc " << sv1rc << " " << varnamc << " ->" << varvalu.strptr
               << "<- " << strlen((const char *)varvalu.strptr)
               << "/"   << varvalu.strlength ) ;

     DEBUG("TM", "Unravelled the Trigger Data" ) ;
   }


//
// Set a Good return Code if apt
//

 if ( rc == 0 )
    {
      strcpy((char *)returnc,"0 0 0 RXMQRTM OK" ) ;
      rc   =  0 ;
      mqrc =  0 ;
      mqac =  0 ;
    }


//
// Set the LAST variables, and the function return string
//


 stack_and_set(rc,mqrc,mqac,"RXMQRTM",returnc)       ;
 aretstr->strlength  = strlen((const char *)returnc)   ;
 aretstr->strptr     = (char *)returnc                         ;

//
// Return to caller
//

 DEBUG("TM", "Leaving rxmqrtm " ) ;

 return 0 ;

 } // End of RXMQRTM function

